<?php

namespace  {
/*
 * This file is part of Contao.
 *
 * (c) Leo Feyer
 *
 * @license LGPL-3.0-or-later
 */

use Contao\ArticleModel;
use Contao\Automator;
use Contao\CheckBox;
use Contao\CheckBoxWizard;
use Contao\ChmodTable;
use Contao\ContentAccordion;
use Contao\ContentAccordionStart;
use Contao\ContentAccordionStop;
use Contao\ContentAlias;
use Contao\ContentArticle;
use Contao\ContentCode;
use Contao\ContentDownload;
use Contao\ContentDownloads;
use Contao\ContentGallery;
use Contao\ContentHeadline;
use Contao\ContentHtml;
use Contao\ContentHyperlink;
use Contao\ContentImage;
use Contao\ContentList;
use Contao\ContentMarkdown;
use Contao\ContentMedia;
use Contao\ContentModel;
use Contao\ContentModule;
use Contao\ContentSliderStart;
use Contao\ContentSliderStop;
use Contao\ContentTable;
use Contao\ContentTeaser;
use Contao\ContentText;
use Contao\ContentToplink;
use Contao\ContentVimeo;
use Contao\ContentYouTube;
use Contao\CoreBundle\Controller\BackendCsvImportController;
use Contao\Crawl;
use Contao\FileSelector;
use Contao\FilesModel;
use Contao\FileTree;
use Contao\Form;
use Contao\FormCaptcha;
use Contao\FormCheckBox;
use Contao\FormExplanation;
use Contao\FormFieldModel;
use Contao\FormFieldsetStart;
use Contao\FormFieldsetStop;
use Contao\FormFileUpload;
use Contao\FormHidden;
use Contao\FormHtml;
use Contao\FormModel;
use Contao\FormPassword;
use Contao\FormRadioButton;
use Contao\FormRange;
use Contao\FormSelectMenu;
use Contao\FormSubmit;
use Contao\FormTextArea;
use Contao\FormTextField;
use Contao\ImageSize;
use Contao\ImageSizeItemModel;
use Contao\ImageSizeModel;
use Contao\InputUnit;
use Contao\KeyValueWizard;
use Contao\LayoutModel;
use Contao\ListWizard;
use Contao\Maintenance;
use Contao\MemberGroupModel;
use Contao\MemberModel;
use Contao\Messages;
use Contao\MetaWizard;
use Contao\ModuleArticleList;
use Contao\ModuleArticlenav;
use Contao\ModuleBooknav;
use Contao\ModuleBreadcrumb;
use Contao\ModuleChangePassword;
use Contao\ModuleCloseAccount;
use Contao\ModuleCustomnav;
use Contao\ModuleHtml;
use Contao\ModuleLogin;
use Contao\ModuleLogout;
use Contao\ModuleMaintenance;
use Contao\ModuleModel;
use Contao\ModuleNavigation;
use Contao\ModulePassword;
use Contao\ModulePersonalData;
use Contao\ModuleQuicklink;
use Contao\ModuleQuicknav;
use Contao\ModuleRandomImage;
use Contao\ModuleRegistration;
use Contao\ModuleRssReader;
use Contao\ModuleSearch;
use Contao\ModuleSitemap;
use Contao\ModuleTwoFactor;
use Contao\ModuleWizard;
use Contao\OptInModel;
use Contao\OptionWizard;
use Contao\PageError401;
use Contao\PageError403;
use Contao\PageError404;
use Contao\PageForward;
use Contao\PageLogout;
use Contao\PageModel;
use Contao\PageRedirect;
use Contao\PageRegular;
use Contao\PageRoot;
use Contao\PageSelector;
use Contao\PageTree;
use Contao\Password;
use Contao\Picker;
use Contao\PurgeData;
use Contao\RadioButton;
use Contao\RadioTable;
use Contao\SectionWizard;
use Contao\SelectMenu;
use Contao\SerpPreview;
use Contao\StringUtil;
use Contao\StyleModel;
use Contao\StyleSheetModel;
use Contao\StyleSheets;
use Contao\System;
use Contao\TableWizard;
use Contao\TextArea;
use Contao\TextField;
use Contao\TextStore;
use Contao\Theme;
use Contao\ThemeModel;
use Contao\TimePeriod;
use Contao\TrblField;
use Contao\Upload;
use Contao\UserGroupModel;
use Contao\UserModel;

// Back end modules
$GLOBALS['BE_MOD'] = array
(
	// Content modules
	'content' => array
	(
		'article' => array
		(
			'tables'      => array('tl_article', 'tl_content'),
			'table'       => array(BackendCsvImportController::class, 'importTableWizardAction'),
			'list'        => array(BackendCsvImportController::class, 'importListWizardAction')
		),
		'form' => array
		(
			'tables'      => array('tl_form', 'tl_form_field'),
			'option'      => array(BackendCsvImportController::class, 'importOptionWizardAction')
		)
	),

	// Design modules
	'design' => array
	(
		'themes' => array
		(
			'tables'      => array('tl_theme', 'tl_module', 'tl_style_sheet', 'tl_style', 'tl_layout', 'tl_image_size', 'tl_image_size_item'),
			'importTheme' => array(Theme::class, 'importTheme'),
			'exportTheme' => array(Theme::class, 'exportTheme'),
			'import'      => array(StyleSheets::class, 'importStyleSheet'),
			'export'      => array(StyleSheets::class, 'exportStyleSheet')
		),
		'page' => array
		(
			'tables'      => array('tl_page')
		),
		'tpl_editor' => array
		(
			'tables'      => array('tl_templates'),
			'new_tpl'     => array('tl_templates', 'addNewTemplate'),
			'compare'     => array('tl_templates', 'compareTemplate')
		)
	),

	// Account modules
	'accounts' => array
	(
		'member' => array
		(
			'tables'                  => array('tl_member')
		),
		'mgroup' => array
		(
			'tables'                  => array('tl_member_group')
		),
		'user' => array
		(
			'tables'                  => array('tl_user')
		),
		'group' => array
		(
			'tables'                  => array('tl_user_group')
		),
		'login' => array
		(
			'tables'                  => array('tl_user'),
			'hideInNavigation'        => true,
			'disablePermissionChecks' => true
		),
		'security' => array
		(
			'callback'                => ModuleTwoFactor::class,
			'hideInNavigation'        => true,
			'disablePermissionChecks' => true
		)
	),

	// System modules
	'system' => array
	(
		'files' => array
		(
			'tables'                  => array('tl_files')
		),
		'settings' => array
		(
			'tables'                  => array('tl_settings')
		),
		'maintenance' => array
		(
			'callback'                => ModuleMaintenance::class
		),
		'log' => array
		(
			'tables'                  => array('tl_log')
		),
		'opt_in' => array
		(
			'tables'                  => array('tl_opt_in'),
			'resend'                  => array('tl_opt_in', 'resendToken'),
		),
		'undo' => array
		(
			'tables'                  => array('tl_undo'),
			'disablePermissionChecks' => true
		)
	)
);

// Front end modules
$GLOBALS['FE_MOD'] = array
(
	'navigationMenu' => array(
		'navigation'     => ModuleNavigation::class,
		'customnav'      => ModuleCustomnav::class,
		'breadcrumb'     => ModuleBreadcrumb::class,
		'quicknav'       => ModuleQuicknav::class,
		'quicklink'      => ModuleQuicklink::class,
		'booknav'        => ModuleBooknav::class,
		'articlenav'     => ModuleArticlenav::class,
		'sitemap'        => ModuleSitemap::class
	),
	'user' => array
	(
		'login'          => ModuleLogin::class,
		'logout'         => ModuleLogout::class,
		'personalData'   => ModulePersonalData::class,
		'registration'   => ModuleRegistration::class,
		'changePassword' => ModuleChangePassword::class,
		'lostPassword'   => ModulePassword::class,
		'closeAccount'   => ModuleCloseAccount::class
	),
	'application' => array
	(
		'form'           => Form::class,
		'search'         => ModuleSearch::class
	),
	'miscellaneous' => array
	(
		'articlelist'    => ModuleArticleList::class,
		'randomImage'    => ModuleRandomImage::class,
		'html'           => ModuleHtml::class,
		'rssReader'      => ModuleRssReader::class
	)
);

// Content elements
$GLOBALS['TL_CTE'] = array
(
	'texts' => array
	(
		'headline'        => ContentHeadline::class,
		'text'            => ContentText::class,
		'html'            => ContentHtml::class,
		'list'            => ContentList::class,
		'table'           => ContentTable::class,
		'code'            => ContentCode::class,
		'markdown'        => ContentMarkdown::class
	),
	'accordion' => array
	(
		'accordionSingle' => ContentAccordion::class,
		'accordionStart'  => ContentAccordionStart::class,
		'accordionStop'   => ContentAccordionStop::class
	),
	'slider' => array
	(
		'sliderStart'     => ContentSliderStart::class,
		'sliderStop'      => ContentSliderStop::class
	),
	'links' => array
	(
		'hyperlink'       => ContentHyperlink::class,
		'toplink'         => ContentToplink::class
	),
	'media' => array
	(
		'image'           => ContentImage::class,
		'gallery'         => ContentGallery::class,
		'player'          => ContentMedia::class,
		'youtube'         => ContentYouTube::class,
		'vimeo'           => ContentVimeo::class
	),
	'files' => array
	(
		'download'        => ContentDownload::class,
		'downloads'       => ContentDownloads::class
	),
	'includes' => array
	(
		'article'         => ContentArticle::class,
		'alias'           => ContentAlias::class,
		'form'            => Form::class,
		'module'          => ContentModule::class,
		'teaser'          => ContentTeaser::class
	)
);

// Back end form fields
$GLOBALS['BE_FFL'] = array
(
	'text'           => TextField::class,
	'password'       => Password::class,
	'textStore'      => TextStore::class,
	'textarea'       => TextArea::class,
	'select'         => SelectMenu::class,
	'checkbox'       => CheckBox::class,
	'checkboxWizard' => CheckBoxWizard::class,
	'radio'          => RadioButton::class,
	'radioTable'     => RadioTable::class,
	'inputUnit'      => InputUnit::class,
	'trbl'           => TrblField::class,
	'chmod'          => ChmodTable::class,
	'picker'         => Picker::class,
	'pageTree'       => PageTree::class,
	'pageSelector'   => PageSelector::class,
	'fileTree'       => FileTree::class,
	'fileSelector'   => FileSelector::class,
	'fileUpload'     => Upload::class,
	'tableWizard'    => TableWizard::class,
	'listWizard'     => ListWizard::class,
	'optionWizard'   => OptionWizard::class,
	'moduleWizard'   => ModuleWizard::class,
	'keyValueWizard' => KeyValueWizard::class,
	'imageSize'      => ImageSize::class,
	'timePeriod'     => TimePeriod::class,
	'metaWizard'     => MetaWizard::class,
	'sectionWizard'  => SectionWizard::class,
	'serpPreview'    => SerpPreview::class
);

// Front end form fields
$GLOBALS['TL_FFL'] = array
(
	'explanation'   => FormExplanation::class,
	'html'          => FormHtml::class,
	'fieldsetStart' => FormFieldsetStart::class,
	'fieldsetStop'  => FormFieldsetStop::class,
	'text'          => FormTextField::class,
	'password'      => FormPassword::class,
	'textarea'      => FormTextArea::class,
	'select'        => FormSelectMenu::class,
	'radio'         => FormRadioButton::class,
	'checkbox'      => FormCheckBox::class,
	'upload'        => FormFileUpload::class,
	'range'         => FormRange::class,
	'hidden'        => FormHidden::class,
	'captcha'       => FormCaptcha::class,
	'submit'        => FormSubmit::class,
);

// Page types
$GLOBALS['TL_PTY'] = array
(
	'regular'   => PageRegular::class,
	'forward'   => PageForward::class,
	'redirect'  => PageRedirect::class,
	'root'      => PageRoot::class,
	'logout'    => PageLogout::class,
	'error_401' => PageError401::class,
	'error_403' => PageError403::class,
	'error_404' => PageError404::class
);

// Maintenance
$GLOBALS['TL_MAINTENANCE'] = array
(
	Maintenance::class,
	Crawl::class,
	PurgeData::class
);

// Purge jobs
$GLOBALS['TL_PURGE'] = array
(
	'tables' => array
	(
		'index' => array
		(
			'callback' => array(Automator::class, 'purgeSearchTables'),
			'affected' => array('tl_search', 'tl_search_index', 'tl_search_term')
		),
		'undo' => array
		(
			'callback' => array(Automator::class, 'purgeUndoTable'),
			'affected' => array('tl_undo')
		),
		'versions' => array
		(
			'callback' => array(Automator::class, 'purgeVersionTable'),
			'affected' => array('tl_version')
		),
		'log' => array
		(
			'callback' => array(Automator::class, 'purgeSystemLog'),
			'affected' => array('tl_log')
		),
		'crawl_queue' => array
		(
			'callback' => array(Automator::class, 'purgeCrawlQueue'),
			'affected' => array('tl_crawl_queue')
		)
	),
	'folders' => array
	(
		'images' => array
		(
			'callback' => array(Automator::class, 'purgeImageCache'),
			'affected' => array(StringUtil::stripRootDir(System::getContainer()->getParameter('contao.image.target_dir')))
		),
		'scripts' => array
		(
			'callback' => array(Automator::class, 'purgeScriptCache'),
			'affected' => array('assets/js', 'assets/css')
		),
		'search' => array
		(
			'callback' => array(Automator::class, 'purgeSearchCache'),
			'affected' => array('%s/contao/search')
		),
		'temp' => array
		(
			'callback' => array(Automator::class, 'purgeTempFolder'),
			'affected' => array('system/tmp')
		)
	),
	'custom' => array
	(
		'pages' => array
		(
			'callback' => array(Automator::class, 'purgePageCache'),
		),
		'xml' => array
		(
			'callback' => array(Automator::class, 'generateXmlFiles')
		),
		'symlinks' => array
		(
			'callback' => array(Automator::class, 'generateSymlinks')
		)
	)
);

// Image crop modes
$GLOBALS['TL_CROP'] = array
(
	'image_sizes' => array
	(
		// will be added dynamically
	),
	'relative' => array
	(
		'proportional', 'box'
	),
	'exact' => array
	(
		'crop',
		'left_top',    'center_top',    'right_top',
		'left_center', 'center_center', 'right_center',
		'left_bottom', 'center_bottom', 'right_bottom'
	)
);

// Cron jobs
$GLOBALS['TL_CRON'] = array
(
	'monthly' => array(),
	'weekly' => array(),
	'daily' => array
	(
		'purgeTempFolder' => array(Automator::class, 'purgeTempFolder'),
		'purgeSearchCache' => array(Automator::class, 'purgeSearchCache'),
		'generateSitemap' => array(Automator::class, 'generateSitemap'),
		'purgeRegistrations' => array(Automator::class, 'purgeRegistrations'),
		'purgeOptInTokens' => array(Automator::class, 'purgeOptInTokens')
	),
	'hourly' => array(),
	'minutely' => array()
);

// Hooks
$GLOBALS['TL_HOOKS'] = array
(
	'getSystemMessages' => array
	(
		array(Messages::class, 'maintenanceCheck'),
		array(Messages::class, 'languageFallback')
	)
);

// Register the auto_item keywords
$GLOBALS['TL_AUTO_ITEM'] = array('items', 'events');

// Do not index a page if one of the following parameters is set
$GLOBALS['TL_NOINDEX_KEYS'] = array('id', 'file', 'token', 'day', 'month', 'year', 'page', 'page_.*', 'keywords', 'PHPSESSID');

// Register the supported CSS units
$GLOBALS['TL_CSS_UNITS'] = array('px', '%', 'em', 'rem', 'vw', 'vh', 'vmin', 'vmax', 'ex', 'pt', 'pc', 'in', 'cm', 'mm');

// Wrapper elements
$GLOBALS['TL_WRAPPERS'] = array
(
	'start' => array
	(
		'accordionStart',
		'sliderStart',
		'fieldsetStart'
	),
	'stop' => array
	(
		'accordionStop',
		'sliderStop',
		'fieldsetStop'
	),
	'single' => array
	(
		'accordionSingle'
	),
	'separator' => array()
);

// Models
$GLOBALS['TL_MODELS'] = array(
	'tl_article' => ArticleModel::class,
	'tl_content' => ContentModel::class,
	'tl_files' => FilesModel::class,
	'tl_form_field' => FormFieldModel::class,
	'tl_form' => FormModel::class,
	'tl_image_size_item' => ImageSizeItemModel::class,
	'tl_image_size' => ImageSizeModel::class,
	'tl_layout' => LayoutModel::class,
	'tl_member_group' => MemberGroupModel::class,
	'tl_member' => MemberModel::class,
	'tl_module' => ModuleModel::class,
	'tl_opt_in' => OptInModel::class,
	'tl_page' => PageModel::class,
	'tl_style' => StyleModel::class,
	'tl_style_sheet' => StyleSheetModel::class,
	'tl_theme' => ThemeModel::class,
	'tl_user_group' => UserGroupModel::class,
	'tl_user' => UserModel::class
);

// Other global arrays
$GLOBALS['TL_PERMISSIONS'] = array();
}

namespace  {
/*
 * This file is part of Contao.
 *
 * (c) Leo Feyer
 *
 * @license LGPL-3.0-or-later
 */

use Contao\ManagerBundle\Controller\DebugController;

// Back end modules
$GLOBALS['BE_MOD']['accounts']['debug'] = array
(
	'enable'                  => array(DebugController::class, 'enableAction'),
	'disable'                 => array(DebugController::class, 'disableAction'),
	'hideInNavigation'        => true,
	'disablePermissionChecks' => true
);
}

namespace  {
/**
 * Form fields
 */
$GLOBALS['TL_FFL']['calendarfield'] = 'Hofff\Contao\Calendarfield\FormCalendarField';
}

namespace  {
/*
 * Copyright MADE/YOUR/DAY OG <mail@madeyourday.net>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/**
 * RockSolid Custom Elements configuration
 *
 * @author Martin Auswöger <martin@madeyourday.net>
 */

$GLOBALS['TL_HOOKS']['initializeSystem'][] = array('MadeYourDay\\RockSolidCustomElements\\CustomElements', 'loadConfig');
$GLOBALS['TL_HOOKS']['loadLanguageFile'][] = array('MadeYourDay\\RockSolidCustomElements\\CustomElements', 'loadLanguageFileHook');
$GLOBALS['TL_HOOKS']['exportTheme'][] = array('MadeYourDay\\RockSolidCustomElements\\CustomElements', 'exportThemeHook');
$GLOBALS['TL_HOOKS']['extractThemeFiles'][] = array('MadeYourDay\\RockSolidCustomElements\\CustomElements', 'extractThemeFilesHook');

$GLOBALS['BE_FFL']['rsce_list_start'] = 'MadeYourDay\\RockSolidCustomElements\\Widget\\ListStart';
$GLOBALS['BE_FFL']['rsce_list_stop'] = 'MadeYourDay\\RockSolidCustomElements\\Widget\\ListStop';
$GLOBALS['BE_FFL']['rsce_list_item_start'] = 'MadeYourDay\\RockSolidCustomElements\\Widget\\ListItemStart';
$GLOBALS['BE_FFL']['rsce_list_item_stop'] = 'MadeYourDay\\RockSolidCustomElements\\Widget\\ListItemStop';
$GLOBALS['BE_FFL']['rsce_group_start'] = 'MadeYourDay\\RockSolidCustomElements\\Widget\\GroupStart';
$GLOBALS['BE_FFL']['rsce_data'] = 'MadeYourDay\\RockSolidCustomElements\\Widget\\Data';

$GLOBALS['TL_MAINTENANCE'][] = 'MadeYourDay\\RockSolidCustomElements\\CustomElementsConvert';

$GLOBALS['TL_PURGE']['custom']['rocksolid_custom_elements'] = array(
	'callback' => array('MadeYourDay\\RockSolidCustomElements\\CustomElements', 'purgeCache'),
);

// Insert the custom_elements category
array_insert($GLOBALS['TL_CTE'], 1, array('custom_elements' => array()));
array_insert($GLOBALS['FE_MOD'], 0, array('custom_elements' => array()));
}

namespace  {
/**
 * This file is part of menatwork/contao-multicolumnwizard-bundle.
 *
 * (c) 2012-2019 MEN AT WORK.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * This project is provided in good faith and hope to be usable by anyone.
 *
 * @package    menatwork/contao-multicolumnwizard-bundle
 * @author     Andreas Isaak <info@andreas-isaak.de>
 * @author     Andreas Schempp <andreas.schempp@terminal42.ch>
 * @author     Christian Schiffler <c.schiffler@cyberspectrum.de>
 * @author     David Maack <david.maack@arcor.de>
 * @author     fritzmg <email@spikx.net>
 * @author     Stefan Heimes <stefan_heimes@hotmail.com>
 * @author     Tristan Lins <tristan.lins@bit3.de>
 * @author     Yanick Witschi <yanick.witschi@certo-net.ch>
 * @copyright  2011 Andreas Schempp
 * @copyright  2011 certo web & design GmbH
 * @copyright  2013-2019 MEN AT WORK
 * @license    https://github.com/menatwork/contao-multicolumnwizard-bundle/blob/master/LICENSE LGPL-3.0-or-later
 * @filesource
 */

$GLOBALS['BE_FFL']['multiColumnWizard'] = '\MenAtWork\MultiColumnWizardBundle\Contao\Widgets\MultiColumnWizard';

$GLOBALS['TL_HOOKS']['loadDataContainer'][]  = array(
    'MenAtWork\MultiColumnWizardBundle\EventListener\Contao\LoadDataContainer',
    'supportModalSelector'
);
$GLOBALS['TL_HOOKS']['initializeSystem'][]   = array(
    'MenAtWork\MultiColumnWizardBundle\EventListener\Contao\InitializeSystem',
    'changeAjaxPostActions'
);
$GLOBALS['TL_HOOKS']['executePostActions'][] = array(
    'MenAtWork\MultiColumnWizardBundle\EventListener\Contao\ExecutePostActions',
    'executePostActions'
);
$GLOBALS['TL_HOOKS']['executePostActions'][] = array(
    'MenAtWork\MultiColumnWizardBundle\EventListener\Contao\ExecutePostActions',
    'handleRowCreation'
);

if (TL_MODE == 'BE') {
    $GLOBALS['TL_HOOKS']['parseTemplate'][] = array(
        'MenAtWork\MultiColumnWizardBundle\EventListener\Contao\ParseTemplate',
        'addVersion'
    );

    // Add the JS.
    $GLOBALS['TL_JAVASCRIPT']['multicolumnwizard'] = $GLOBALS['TL_CONFIG']['debugMode']
        ? 'bundles/multicolumnwizard/js/multicolumnwizard_be_src.js'
        : 'bundles/multicolumnwizard/js/multicolumnwizard_be.js';

    // Add the css.
    $GLOBALS['TL_CSS']['multicolumnwizard'] = $GLOBALS['TL_CONFIG']['debugMode']
        ? 'bundles/multicolumnwizard/css/multicolumnwizard_src.css'
        : 'bundles/multicolumnwizard/css/multicolumnwizard.css';
}
}

namespace  {
/**
 * Haste utilities for Contao Open Source CMS
 *
 * Copyright (C) 2012-2013 Codefog & terminal42 gmbh
 *
 * @package    Haste
 * @link       http://github.com/codefog/contao-haste/
 * @license    http://opensource.org/licenses/lgpl-3.0.html LGPL
 */


/**
 * Add the "haste_undo" operation to "undo" module
 */
$GLOBALS['BE_MOD']['system']['undo']['haste_undo'] = array('Haste\Util\Undo', 'callback');


/**
 * Backend widgets
 */
$GLOBALS['BE_FFL']['numberField'] = 'Haste\Number\BackendWidget';


/**
 * Hooks
 */
$GLOBALS['TL_HOOKS']['executePostActions'][] = array('Haste\Dca\AjaxOperations', 'executePostActions');
$GLOBALS['TL_HOOKS']['getContentElement'][]  = ['Haste\EventListener\AjaxReloadListener', 'onGetContentElement'];
$GLOBALS['TL_HOOKS']['getFrontendModule'][]  = ['Haste\EventListener\AjaxReloadListener', 'onGetFrontendModule'];
$GLOBALS['TL_HOOKS']['loadDataContainer'][]  = array('Haste\Dca\DateRangeFilter', 'addCallbacks');
$GLOBALS['TL_HOOKS']['loadDataContainer'][]  = array('Haste\Dca\AjaxOperations', 'modifyDca');
$GLOBALS['TL_HOOKS']['loadDataContainer'][]  = array('Haste\Model\Relations', 'addRelationCallbacks');
$GLOBALS['TL_HOOKS']['modifyFrontendPage'][] = ['Haste\EventListener\AjaxReloadListener', 'onModifyFrontendPage'];
$GLOBALS['TL_HOOKS']['replaceInsertTags'][]  = array('Haste\Util\InsertTag', 'replaceHasteInsertTags');
$GLOBALS['TL_HOOKS']['reviseTable'][]        = array('Haste\Model\Relations', 'reviseRelatedRecords');

if (TL_MODE !== 'FE') {
    $GLOBALS['TL_HOOKS']['sqlGetFromFile'][] = ['Haste\Model\Relations', 'addRelationTables'];
}


/**
 * Haste hooks
 */
$GLOBALS['HASTE_HOOKS']['undoData'] = array
(
    array('Haste\Model\Relations', 'undoRelations')
);
}

namespace  {
/*
 * This file is part of the IncludeInfoBundle.
 *
 * (c) inspiredminds
 *
 * @license LGPL-3.0-or-later
 */

use InspiredMinds\IncludeInfoBundle\EventListener\ReplaceInsertTagsListener;
use InspiredMinds\IncludeInfoBundle\Model\InsertTagIndexModel;
use InspiredMinds\IncludeInfoBundle\Widget\IncludeInfoWidget;

$GLOBALS['BE_FFL'][IncludeInfoWidget::class] = IncludeInfoWidget::class;
$GLOBALS['TL_HOOKS']['replaceInsertTags'][] = [ReplaceInsertTagsListener::class, '__invoke'];
$GLOBALS['TL_MODELS']['tl_inserttag_index'] = InsertTagIndexModel::class;
}

namespace  {
/*
 * This file is part of Contao.
 *
 * (c) Leo Feyer
 *
 * @license LGPL-3.0-or-later
 */

use Contao\FaqCategoryModel;
use Contao\FaqModel;
use Contao\ModuleFaq;
use Contao\ModuleFaqList;
use Contao\ModuleFaqPage;
use Contao\ModuleFaqReader;

// Add back end modules
$GLOBALS['BE_MOD']['content']['faq'] = array
(
	'tables' => array('tl_faq_category', 'tl_faq')
);

// Front end modules
$GLOBALS['FE_MOD']['faq'] = array
(
	'faqlist'   => ModuleFaqList::class,
	'faqreader' => ModuleFaqReader::class,
	'faqpage'   => ModuleFaqPage::class
);

// Style sheet
if (defined('TL_MODE') && TL_MODE == 'BE')
{
	$GLOBALS['TL_CSS'][] = 'bundles/contaofaq/faq.min.css|static';
}

// Register hooks
$GLOBALS['TL_HOOKS']['getSearchablePages'][] = array(ModuleFaq::class, 'getSearchablePages');

// Add permissions
$GLOBALS['TL_PERMISSIONS'][] = 'faqs';
$GLOBALS['TL_PERMISSIONS'][] = 'faqp';

// Models
$GLOBALS['TL_MODELS']['tl_faq_category'] = FaqCategoryModel::class;
$GLOBALS['TL_MODELS']['tl_faq'] = FaqModel::class;
}

namespace  {
/**
 * mp_forms extension for Contao Open Source CMS
 *
 * @copyright  Copyright (c) 2015-2016, terminal42 gmbh
 * @author     terminal42 gmbh <info@terminal42.ch>
 * @license    http://opensource.org/licenses/lgpl-3.0.html LGPL
 * @link       https://github.com/terminal42/contao-mp_forms
 */

/**
 * Form fields
 */
$GLOBALS['TL_FFL']['mp_form_pageswitch'] = 'FormMPFormPageSwitch';

/**
 * Front end modules
 */
$GLOBALS['FE_MOD']['application']['mp_form_steps'] = 'MPFormsStepsModule';

/**
 * Hooks
 */
$GLOBALS['TL_HOOKS']['replaceInsertTags'][] = ['MPForms', 'replaceTags'];
$GLOBALS['TL_HOOKS']['compileFormFields'][] = ['MPForms', 'compileFormFields'];
$GLOBALS['TL_HOOKS']['loadFormField'][]     = ['MPForms', 'loadValuesFromSession'];
$GLOBALS['TL_HOOKS']['prepareFormData'][]   = ['MPForms', 'prepareFormData'];
}

namespace  {
use Hofff\Contao\RecursiveDownloadFolder\Frontend\Element\RecursiveDownloadFolderElement;
use Hofff\Contao\RecursiveDownloadFolder\Frontend\Module\RecursiveDownloadFolderModule;

/*
 * Content elements
 */

$GLOBALS['TL_CTE']['files']['hofff_recursive-download-folder']       = RecursiveDownloadFolderElement::class;
$GLOBALS['FE_MOD']['application']['hofff_recursive-download-folder'] = RecursiveDownloadFolderModule::class;
}

namespace  {
/**
 * Contao Open Source CMS
 *
 * @copyright  MEN AT WORK 2014
 * @package    ctoCommunication
 * @license    GNU/LGPL
 * @filesource
 */

/**
 * ctoCommunication version
 */
$GLOBALS['CTOCOM_VERSION'] = '2.0.0';

/**
 * Maintenance
 */
$GLOBALS['TL_CACHE']['ctoCom'] = 'tl_ctocom_cache';

/**
 * Init the default config array if not set.
 */
if (!isset($GLOBALS['SYC_CONFIG'])) {
    $GLOBALS['SYC_CONFIG'] = [];
}

if (!isset($GLOBALS['SYC_CONFIG']['table_hidden'])) {
    $GLOBALS['SYC_CONFIG']['table_hidden'] = [];
}

/**
 * Blacklists tables for syncCto
 */
$GLOBALS['SYC_CONFIG']['table_hidden'] = array_merge((array)$GLOBALS['SYC_CONFIG']['table_hidden'], array
(
    'tl_ctocom_cache',
    'tl_requestcache',
));

/**
 * ctoCommunication engines
 */
$GLOBALS['CTOCOM_ENGINE'] = array
(
    'empty' => array
    (
        'name'      => &$GLOBALS['TL_LANG']['CTOCOM']['codifyengine']['empty'],
        'classname' => '\MenAtWork\CtoCommunicationBundle\Codifyengine\NoneCrypt',
        'invisible' => false
    ),
    'aes'   => array
    (
        'name'      => &$GLOBALS['TL_LANG']['CTOCOM']['codifyengine']['phpseclib_aes'],
        'classname' => '\MenAtWork\CtoCommunicationBundle\Codifyengine\AES',
        'invisible' => false
    ),
);
// Check if we have support for mcrypt if not, skip this function.
if (\extension_loaded('mcrypt') && \function_exists('mcrypt_create_iv')) {
    $GLOBALS['CTOCOM_ENGINE']['mcrypt'] = array
    (
        'name'      => &$GLOBALS['TL_LANG']['CTOCOM']['codifyengine']['mcrypt'],
        'classname' => '\MenAtWork\CtoCommunicationBundle\Codifyengine\Mcrypt',
        'invisible' => false
    );
}

$GLOBALS['CTOCOM_IO'] = array
(
    'default' => array
    (
        'accept'      => array('text/html', 'text/plain', '*/*'),
        'contentType' => 'text/html',
        'classname'   => '\MenAtWork\CtoCommunicationBundle\InputOutput\Base',
    )
);

/**
 * Register for RPC-Call functions
 * Base configuration and ctoCommunication RPC Calls
 */
$GLOBALS['CTOCOM_FUNCTIONS'] = array
(
    //- Referer Functions --------
    'CTOCOM_REFERRER_DISABLE'  => array
    (
        'class'     => '\MenAtWork\CtoCommunicationBundle\RPC\CoreFunctions',
        'function'  => 'referrer_disable',
        'typ'       => 'GET',
        'parameter' => false,
    ),
    'CTOCOM_REFERRER_ENABLE'   => array
    (
        'class'     => '\MenAtWork\CtoCommunicationBundle\RPC\CoreFunctions',
        'function'  => 'referrer_enable',
        'typ'       => 'GET',
        'parameter' => false,
    ),
    //- Version Functions --------
    'CTOCOM_VERSION'           => array
    (
        'class'     => '\MenAtWork\CtoCommunicationBundle\RPC\CoreFunctions',
        'function'  => 'getCtoComVersion',
        'typ'       => 'GET',
        'parameter' => false,
    ),
    'CONTAO_VERSION'           => array
    (
        'class'     => '\MenAtWork\CtoCommunicationBundle\RPC\CoreFunctions',
        'function'  => 'getContaoVersion',
        'typ'       => 'GET',
        'parameter' => false,
    ),
    'CONTAO_FULL_VERSION'      => array
    (
        'class'     => '\MenAtWork\CtoCommunicationBundle\RPC\CoreFunctions',
        'function'  => 'getContaoFullVersion',
        'typ'       => 'GET',
        'parameter' => false,
    ),
    'CTOCOM_GET_RESPONSE_PART' => array
    (
        'class'     => '\MenAtWork\CtoCommunicationBundle\RPC\CoreFunctions',
        'function'  => 'getResponsePart',
        'typ'       => 'POST',
        'parameter' => array('splitname', 'splitcount'),
    ),
    'CTOCOM_HELLO'             => array
    (
        'class'     => '\MenAtWork\CtoCommunicationBundle\RPC\CoreFunctions',
        'function'  => 'generateUUID',
        'typ'       => 'GET',
        'parameter' => false,
    ),
    'CTOCOM_START_HANDSHAKE'   => array
    (
        'class'     => '\MenAtWork\CtoCommunicationBundle\RPC\CoreFunctions',
        'function'  => 'startHandshake',
        'typ'       => 'GET',
        'parameter' => false,
    ),
    'CTOCOM_CHECK_HANDSHAKE'   => array
    (
        'class'     => '\MenAtWork\CtoCommunicationBundle\RPC\CoreFunctions',
        'function'  => 'checkHandshake',
        'typ'       => 'GET',
        'parameter' => false,
    ),
    'CTOCOM_BYE'               => array
    (
        'class'     => '\MenAtWork\CtoCommunicationBundle\RPC\CoreFunctions',
        'function'  => 'deleteUUID',
        'typ'       => 'GET',
        'parameter' => false,
    ),
);
}

namespace  {
/*
 * Copyright MADE/YOUR/DAY OG <mail@madeyourday.net>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/**
 * RockSolid Columns configuration
 *
 * @author Martin Auswöger <martin@madeyourday.net>
 */

$GLOBALS['TL_HOOKS']['generatePage'][] = array('MadeYourDay\\RockSolidColumns\\Columns', 'generatePageHook');
$GLOBALS['TL_HOOKS']['getContentElement'][] = array('MadeYourDay\\RockSolidColumns\\Columns', 'getContentElementHook');

$GLOBALS['TL_CTE']['rs_columns']['rs_columns_start'] = 'MadeYourDay\\RockSolidColumns\\Element\\ColumnsStart';
$GLOBALS['TL_CTE']['rs_columns']['rs_columns_stop'] = 'MadeYourDay\\RockSolidColumns\\Element\\ColumnsStop';
$GLOBALS['TL_CTE']['rs_columns']['rs_column_start'] = 'MadeYourDay\\RockSolidColumns\\Element\\ColumnStart';
$GLOBALS['TL_CTE']['rs_columns']['rs_column_stop'] = 'MadeYourDay\\RockSolidColumns\\Element\\ColumnStop';

$GLOBALS['TL_WRAPPERS']['start'][] = 'rs_columns_start';
$GLOBALS['TL_WRAPPERS']['stop'][] = 'rs_columns_stop';
$GLOBALS['TL_WRAPPERS']['start'][] = 'rs_column_start';
$GLOBALS['TL_WRAPPERS']['stop'][] = 'rs_column_stop';
}

namespace  {
/*
 * @copyright  trilobit GmbH
 * @author     trilobit GmbH <https://github.com/trilobit-gmbh>
 * @license    LGPL-3.0-or-later
 * @link       http://github.com/trilobit-gmbh/contao-constants-bundle
 */

use Trilobit\ConstantsBundle\EventListener\InsertTagListener;
use Trilobit\ConstantsBundle\Model\ConstantsModel;

$GLOBALS['BE_MOD']['trilobit']['tl_constants'] = [
    'tables' => ['tl_constants'],
];

/*
 * Register hook
 */
$GLOBALS['TL_HOOKS']['replaceInsertTags'][] = [InsertTagListener::class, '__invoke'];

/*
 * Models
 */
$GLOBALS['TL_MODELS']['tl_constants'] = ConstantsModel::class;
}

namespace  {
/*
 * This file is part of Contao.
 *
 * (c) Leo Feyer
 *
 * @license LGPL-3.0-or-later
 */

use Contao\Comments;
use Contao\CommentsModel;
use Contao\CommentsNotifyModel;
use Contao\ContentComments;
use Contao\ModuleComments;

// Add content element
$GLOBALS['TL_CTE']['includes']['comments'] = ContentComments::class;

// Front end modules
$GLOBALS['FE_MOD']['application']['comments'] = ModuleComments::class;

// Back end modules
$GLOBALS['BE_MOD']['content']['comments'] = array
(
	'tables'     => array('tl_comments'),
	'stylesheet' => 'bundles/contaocomments/comments.min.css'
);

// Cron jobs
$GLOBALS['TL_CRON']['daily']['purgeCommentSubscriptions'] = array(Comments::class, 'purgeSubscriptions');

// Models
$GLOBALS['TL_MODELS']['tl_comments'] = CommentsModel::class;
$GLOBALS['TL_MODELS']['tl_comments_notify'] = CommentsNotifyModel::class;
}

namespace  {
array_insert( $GLOBALS['FE_MOD'], 3, [

    'user' => [

        'personalDataExtension' => 'Alnv\PersonalDataExtensionBundle\Modules\ModulePersonalData'
    ]
]);
}

namespace  {
/**
 * rte_table extension for Contao Open Source CMS
 *
 * @copyright  Copyright (c) 2008-2016, terminal42 gmbh
 * @author     terminal42 gmbh <info@terminal42.ch>
 * @license    http://opensource.org/licenses/lgpl-3.0.html LGPL
 * @link       http://github.com/terminal42/rte_table
 */

/**
 * Replace the default table wizard
 */
$GLOBALS['BE_FFL']['tableWizard'] = 'Terminal42\RteTable\RteTableWizard';
}

namespace  {
/**
 * Extension for Contao 4
 *
 * @copyright  Softleister 2018
 * @author     Softleister <info@softleister.de>
 * @package    contao-message-jobs-bundle
 * @licence    LGPL-3.0+
 */


/**
 * Back end modules
 */
array_insert($GLOBALS['BE_MOD']['notification_center'], 100, array
(
    'msg_jobs' => array
    (
        'tables'      => array('tl_msg_jobs', 'tl_msg_log'),
    	'runJob'      => array('Softleister\Messagejobs\runMessageJob', 'runJob'),
    )
));


//-------------------------------------------------------------------
// Notification Center
//-------------------------------------------------------------------
$GLOBALS['NOTIFICATION_CENTER']['NOTIFICATION_TYPE']['msg_jobs'] = array
(
    'msg_jobs_transmit' => array(
            'recipients'            => array('admin_email', 'mjob_*'),      // Empfänger
            'email_subject'         => array('mjob_*', 'admin_email'),      // Betreff
            'email_text'            => array('mjob_*'),
            'email_html'            => array('mjob_*'),
            'file_name'             => array('mjob_*', 'admin_email'),
            'file_content'          => array('mjob_*', 'admin_email'),
            'email_recipient_cc'    => array('admin_email', 'mjob_*'),      // Kopie an
            'email_recipient_bcc'   => array('admin_email', 'mjob_*'),      // Blindkopie an
            'email_replyTo'         => array('admin_email', 'mjob_*'),      // Antwortadresse
            'attachment_tokens'     => array('mjob_*'),
    ),
);
}

namespace  {
/**
 * @package   EuF-Grid
 * @author    Sebastian Buck
 * @license   LGPL
 * @copyright Erdmann & Freunde
 */


/**
 * EuF Grid ContentElements
 */
$GLOBALS['TL_CTE']['euf_grid'] = array(
  'rowStart'  => 'ContentRowStart',
  'rowEnd'    => 'ContentRowEnd',
  'colStart'  => 'ContentColStart',
  'colEnd'    => 'ContentColEnd',
);

$GLOBALS['TL_FFL']['rowStart'] = 'FormRowStart';
$GLOBALS['TL_FFL']['rowEnd']  = 'FormRowEnd';
$GLOBALS['TL_FFL']['colStart'] = 'FormColStart';
$GLOBALS['TL_FFL']['colEnd']  = 'FormColEnd';


/**
 * Front end wrappers
 */
$GLOBALS['TL_WRAPPERS']['start'][] = 'rowStart';
$GLOBALS['TL_WRAPPERS']['stop'][] = 'rowEnd';
$GLOBALS['TL_WRAPPERS']['start'][] = 'colStart';
$GLOBALS['TL_WRAPPERS']['stop'][] = 'colEnd';


/**
 * HOOKS
 */
$GLOBALS['TL_HOOKS']['getContentElement'][] = array('GridHooks', 'addGridClasses');
$GLOBALS['TL_HOOKS']['loadFormField'][] = array('GridHooks', 'addGridClassesToForms');
$GLOBALS['TL_HOOKS']['getPageLayout'][] = array('GridHooks', 'addCSSToFrondend');


 /**
  * EuF Grid HOOK: manipulateGridClasses
  *
  * Übergibt die derzeitige Umgebung (BE/FE), das betroffene Feld
  * (grid_columns/grid_options) und die Klasse.
  * Erwartet als Rückgabe die manipulierte Klasse als String
  *
  * ###### config.php #######
  * $GLOBALS['TL_HOOKS']['manipulateGridClasses'][] = array('manipulateGridClasses', 'manipulateClasses');
  *
  * ###### manipulateGridClasses.php #######
  * class manipulateGridClasses {
  *
  *    public function manipulateClasses($env, $strField, $class) {
  *
  *      // Do something here...
  *
  *      return $class;
  *    }
  *  }
  */


/**
 * EuF Grid standard configuration
 */
$GLOBALS['EUF_GRID_SETTING'] = array (
  'columns'       => array ('','-1','-2','-3','-4','-5','-6','-7','-8','-9','-10','-11','-12'),
  'viewports'     => array ('','-xs','-sm','-md','-lg','-xl'),
  'devider'       => '-',

  'row'           => 'row',
  'cols'          => array ('col', 'row-span'),
  'pulls'         => array ('pull-left', 'pull-right'),
  'col-start'     => array ('col-start'),
  'row-start'     => array ('row-start'),
  'positioning'   => array ('align', 'justify'),
  'directions'    => array ('-start', '-center', '-end', '-stretch'),
  'start_cols'    => array ('-1','-2','-3','-4','-5','-6','-7','-8','-9','-10','-11', '-12'),
  'options'       => array ('') // additional custom classes
);
}

namespace  {
$GLOBALS['TL_HOOKS']['importUser'][] = array('Oneup\Contao\EmailLogin\ImportUser', 'getUsernameByEmail');
}

namespace  {
/**
 * @copyright  Softleister 2013-2022
 * @author     Softleister <info@softleister.de>
 * @package    contao-linksplitter
 * @license    LGPL
 * @see	       https://github.com/do-while/contao-linksplitter
 *
 */


/*
 * Frontend Module
 */
array_insert($GLOBALS['FE_MOD'], -1, array
(
	'miscellaneous' => array
	(
		'linksplitter'    => 'Softleister\Linksplitter\ModuleLinkSplitter'
	)
));
}

namespace  {
/*
 * This file is part of Contao.
 *
 * (c) Leo Feyer
 *
 * @license LGPL-3.0-or-later
 */

use Contao\ListWizard;
use Contao\ModuleNewsArchive;
use Contao\ModuleNewsList;
use Contao\ModuleNewsMenu;
use Contao\ModuleNewsReader;
use Contao\News;
use Contao\NewsArchiveModel;
use Contao\NewsFeedModel;
use Contao\NewsModel;
use Contao\TableWizard;

// Back end modules
$GLOBALS['BE_MOD']['content']['news'] = array
(
	'tables'      => array('tl_news_archive', 'tl_news', 'tl_news_feed', 'tl_content'),
	'table'       => array(TableWizard::class, 'importTable'),
	'list'        => array(ListWizard::class, 'importList')
);

// Front end modules
$GLOBALS['FE_MOD']['news'] = array
(
	'newslist'    => ModuleNewsList::class,
	'newsreader'  => ModuleNewsReader::class,
	'newsarchive' => ModuleNewsArchive::class,
	'newsmenu'    => ModuleNewsMenu::class
);

// Cron jobs
$GLOBALS['TL_CRON']['daily']['generateNewsFeeds'] = array(News::class, 'generateFeeds');

// Style sheet
if (defined('TL_MODE') && TL_MODE == 'BE')
{
	$GLOBALS['TL_CSS'][] = 'bundles/contaonews/news.min.css|static';
}

// Register hooks
$GLOBALS['TL_HOOKS']['removeOldFeeds'][] = array(News::class, 'purgeOldFeeds');
$GLOBALS['TL_HOOKS']['getSearchablePages'][] = array(News::class, 'getSearchablePages');
$GLOBALS['TL_HOOKS']['generateXmlFiles'][] = array(News::class, 'generateFeeds');

// Add permissions
$GLOBALS['TL_PERMISSIONS'][] = 'news';
$GLOBALS['TL_PERMISSIONS'][] = 'newp';
$GLOBALS['TL_PERMISSIONS'][] = 'newsfeeds';
$GLOBALS['TL_PERMISSIONS'][] = 'newsfeedp';

// Models
$GLOBALS['TL_MODELS']['tl_news_archive'] = NewsArchiveModel::class;
$GLOBALS['TL_MODELS']['tl_news_feed'] = NewsFeedModel::class;
$GLOBALS['TL_MODELS']['tl_news'] = NewsModel::class;
}

namespace  {
/**
 * Contao Open Source CMS
 * Copyright (C) 2005-2013 Leo Feyer
 *
 * Formerly known as TYPOlight Open Source CMS.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * PHP version 5
 * @copyright  terminal42 gmbh 2013
 * @author     Yanick Witschi <yanick.witschi@terminal42.ch>
 * @license    http://opensource.org/licenses/lgpl-3.0.html
 */

/**
 * Hooks
 */
$GLOBALS['TL_HOOKS']['initializeSystem'][] = array('ConditionalFormFields', 'registerHook');
$GLOBALS['TL_HOOKS']['loadFormField'][] = array('ConditionalFormFields', 'loadFormField');
$GLOBALS['TL_HOOKS']['validateFormField'][] = array('ConditionalFormFields', 'validateFormField');
$GLOBALS['TL_HOOKS']['modifyFrontendPage'][] = array('ConditionalFormFields', 'outputFrontendTemplate');
}

namespace  {
define("CATALOG_MANAGER_VERSION", "1.32.12");

if (!\Config::get('hideCatalogManager')) {
    array_insert( $GLOBALS['BE_MOD'], 3, [
        'catalog-manager-extensions' => [
            'catalog-manager' => [
                'name' => 'catalog-manager',
                'icon' => 'system/modules/catalog-manager/assets/icons/icon.svg',
                'tables' => [
                    'tl_catalog',
                    'tl_catalog_fields'
                ]
            ],
            'filterform' => [
                'name' => 'filterform',
                'icon' => 'system/modules/catalog-manager/assets/icons/filterform.svg',
                'tables' => [
                    'tl_catalog_form',
                    'tl_catalog_form_fields'
                ]
            ],
            'support' => [
                'name' => 'support',
                'callback' => CatalogManager\SupportPage::class,
                'stylesheet' => 'system/modules/catalog-manager/assets/support.css',
                'icon' => 'system/modules/catalog-manager/assets/icons/support-icon.svg'
            ],
        ]
    ]);
}

array_insert($GLOBALS['FE_MOD'], 3, [
    'catalog-manager' => [
        'catalogBookNavigation' => 'CatalogManager\ModuleCatalogBookNavigation',
        'catalogTaxonomyTree' => 'CatalogManager\ModuleCatalogTaxonomyTree',
        'catalogUniversalView' => 'CatalogManager\ModuleUniversalView',
        'catalogMasterView' => 'CatalogManager\ModuleMasterView',
        'catalogFilter' => 'CatalogManager\ModuleCatalogFilter',
    ]
]);

array_insert($GLOBALS['TL_CTE'], 3, [
    'catalog-manager' => [
        'catalogCatalogEntity' => 'CatalogManager\ContentCatalogEntity',
        'catalogFilterForm' => 'CatalogManager\ContentCatalogFilterForm',
        'catalogSocialSharingButtons' => 'CatalogManager\ContentSocialSharingButtons',
        'catalogVisibilityPanelStart' => 'CatalogManager\ContentVisibilityPanelStart',
        'catalogVisibilityPanelStop' => 'CatalogManager\ContentVisibilityPanelStop'
    ]
]);

if (TL_MODE == 'BE') {
    $GLOBALS['TL_JAVASCRIPT']['catalogManagerBackendExtension'] = $GLOBALS['TL_CONFIG']['debugMode']
        ? 'system/modules/catalog-manager/assets/BackendExtension.js'
        : 'system/modules/catalog-manager/assets/BackendExtension.js';
    $GLOBALS['TL_CSS']['catalogManagerBackendExtension'] = $GLOBALS['TL_CONFIG']['debugMode']
        ? 'system/modules/catalog-manager/assets/backend.css'
        : 'system/modules/catalog-manager/assets/backend.css';
}

$GLOBALS['TL_HOOKS']['catalogManagerEntityOnCreate'] = [];
$GLOBALS['TL_HOOKS']['catalogManagerEntityOnUpdate'] = [];
$GLOBALS['TL_HOOKS']['catalogManagerEntityOnDelete'] = [];

$GLOBALS['TL_HOOKS']['replaceInsertTags'][] = [ 'CatalogManager\PseudoInsertTag', 'getInsertTagValue' ];
$GLOBALS['TL_HOOKS']['replaceInsertTags'][] = [ 'CatalogManager\ActiveInsertTag', 'getInsertTagValue' ];
$GLOBALS['TL_HOOKS']['replaceInsertTags'][] = [ 'CatalogManager\MasterInsertTag', 'getInsertTagValue' ];
$GLOBALS['TL_HOOKS']['replaceInsertTags'][] = [ 'CatalogManager\CatalogInsertTag', 'getInsertTagValue' ];
$GLOBALS['TL_HOOKS']['replaceInsertTags'][] = [ 'CatalogManager\TimestampInsertTag', 'getInsertTagValue' ];
$GLOBALS['TL_HOOKS']['replaceInsertTags'][] = [ 'CatalogManager\FilterValuesInsertTag', 'getInsertTagValue' ];
$GLOBALS['TL_HOOKS']['replaceInsertTags'][] = [ 'CatalogManager\RandomEntitiesIDInsertTag', 'getInsertTagValue' ];
$GLOBALS['TL_HOOKS']['getAllEvents'][] = ['CatalogManager\CatalogParser', 'getAllEvents'];
if (version_compare('4.12', VERSION, '<=')) {
    if (\System::getContainer()->getParameter('contao.legacy_routing')) {
        $GLOBALS['TL_HOOKS']['getPageIdFromUrl'][] = ['CatalogManager\RoutingBuilder', 'initialize'];
    }
} else {
    $GLOBALS['TL_HOOKS']['getPageIdFromUrl'][] = ['CatalogManager\RoutingBuilder', 'initialize'];
}
$GLOBALS['TL_HOOKS']['loadDataContainer'][] = ['CatalogManager\CatalogDcAdapter', 'initialize'];
$GLOBALS['TL_HOOKS']['generateBreadcrumb'][] = ['CatalogManager\CatalogBreadcrumb', 'initialize'];
$GLOBALS['TL_HOOKS']['sqlCompileCommands'][] = ['CatalogManager\SQLCompileCommands', 'initialize'];
$GLOBALS['TL_HOOKS']['getSearchablePages'][] = ['CatalogManager\SearchIndexBuilder', 'initialize'];
$GLOBALS['TL_HOOKS']['loadDataContainer'][] = ['CatalogManager\UserPermissionExtension', 'initialize'];
$GLOBALS['TL_HOOKS']['initializeSystem'][] = ['CatalogManager\CatalogManagerInitializer', 'initialize'];
$GLOBALS['TL_HOOKS']['loadDataContainer'][] = ['CatalogManager\MemberPermissionExtension', 'initialize'];
$GLOBALS['TL_HOOKS']['outputBackendTemplate'][] = ['CatalogManager\BackendTemplateParser', 'outputBackendTemplate'];
$GLOBALS['TL_HOOKS']['getContentElement'][] = ['CatalogManager\CatalogContentElementParser', 'parseVisibilityPanels'];
$GLOBALS['TL_HOOKS']['changelanguageNavigation'][] = ['CatalogManager\ChangeLanguageExtension', 'translateUrlParameters'];
$GLOBALS['TL_HOOKS']['getAttributesFromDca'][] = ['CatalogManager\CatalogWidgetAttributeParser', 'parseCatalogNavigationAreasWidget'];

$GLOBALS['TL_CATALOG_MANAGER']['CORE_TABLES'] = [];
$GLOBALS['TL_CATALOG_MANAGER']['PROTECTED_CATALOGS'] = [];
$GLOBALS['TL_CATALOG_MANAGER']['CATALOG_EXTENSIONS'] = [];
$GLOBALS['TL_CATALOG_MANAGER']['VISIBILITY_PANEL'] = FALSE;

$GLOBALS['TL_CATALOG_MANAGER']['tinyMCE'] = [ 'tinyMCE', 'tinyFlash' ];

$GLOBALS['TL_WRAPPERS']['stop'][] = 'fieldsetStop';
$GLOBALS['TL_WRAPPERS']['start'][] = 'fieldsetStart';
$GLOBALS['TL_WRAPPERS']['stop'][] = 'catalogVisibilityPanelStop';
$GLOBALS['TL_WRAPPERS']['start'][] = 'catalogVisibilityPanelStart';

$GLOBALS['TL_PERMISSIONS'][] = 'catalog';
$GLOBALS['TL_PERMISSIONS'][] = 'catalogp';
$GLOBALS['TL_PERMISSIONS'][] = 'filterform';
$GLOBALS['TL_PERMISSIONS'][] = 'filterformp';

$GLOBALS['BE_FFL']['catalogMessageWidget'] = 'CatalogManager\CatalogMessageWidget';
$GLOBALS['BE_FFL']['catalogTaxonomyWizard'] = 'CatalogManager\CatalogTaxonomyWizard';
$GLOBALS['BE_FFL']['catalogValueSetterWizard'] = 'CatalogManager\CatalogValueSetterWizard';
$GLOBALS['BE_FFL']['catalogDuplexSelectWizard'] = 'CatalogManager\CatalogDuplexSelectWizard';
$GLOBALS['BE_FFL']['catalogRelationRedirectWizard'] = 'CatalogManager\CatalogRelationRedirectWizard';
$GLOBALS['BE_FFL']['catalogFilterFieldSelectWizard'] = 'CatalogManager\CatalogFilterFieldSelectWizard';

$GLOBALS['TL_FFL']['catalogMessageForm'] = 'CatalogManager\CatalogMessageForm';
$GLOBALS['TL_FFL']['catalogFineUploader'] = 'CatalogManager\CatalogFineUploaderForm';

$GLOBALS['TL_CATALOG_MANAGER']['FIELD_TYPES'] = [
    'text' => [ 'dcPicker' => 'useIndex;', 'dcType' => 'dcPaletteField' ],
    'date' => [ 'dcPicker' => 'useIndex;', 'dcType' => 'dcPaletteField' ],
    'radio' => [ 'dcPicker' => 'useIndex;', 'dcType' => 'dcPaletteField' ],
    'hidden' => [ 'dcPicker' => 'useIndex;', 'dcType' => 'dcPaletteField' ],
    'number' => [ 'dcPicker' => 'useIndex;', 'dcType' => 'dcPaletteField' ],
    'select' => [ 'dcPicker' => 'useIndex;', 'dcType' => 'dcPaletteField' ],
    'message' => [ 'dcPicker' => 'message;', 'dcType' => 'dcPaletteField' ],
    'map' => [ 'dcPicker' => 'description;', 'dcType' => 'dcPaletteField' ],
    'upload' => [ 'dcPicker' => 'statement;', 'dcType' => 'dcPaletteField' ],
    'checkbox' => [ 'dcPicker' => 'useIndex;', 'dcType' => 'dcPaletteField' ],
    'textarea' => [ 'dcPicker' => 'useIndex;', 'dcType' => 'dcPaletteField' ],
    'dbColumn' => [ 'dcPicker' => 'useIndex;', 'dcType' => 'dcPaletteField' ],
    'fieldsetStart' => [  'dcPicker' => 'isHidden;', 'dcType' => 'dcPaletteLegend' ],
    'fieldsetStop' => []
];

$GLOBALS['TL_CATALOG_MANAGER']['FIELD_TYPE_CONVERTER'] = [
    'text' => 'text',
    'date' => 'text',
    'number' => 'text',
    'hidden' => 'text',
    'radio' => 'radio',
    'select' => 'select',
    'upload' => 'fileTree',
    'textarea' => 'textarea',
    'checkbox' => 'checkbox',
    'message' => 'catalogMessageWidget'
];

$GLOBALS['NOTIFICATION_CENTER']['NOTIFICATION_TYPE']['catalog_manager'] = [
    'ctlg_entity_status_insert'   => [
        'recipients' => [ 'admin_email', 'raw_*', 'clean_*' ],
        'email_replyTo' => [ 'admin_email', 'raw_*', 'clean_*' ],
        'email_sender_name' => [ 'admin_email', 'raw_*', 'clean_*' ],
        'email_recipient_cc' => [ 'admin_email', 'raw_*', 'clean_*' ],
        'email_recipient_bcc' => [ 'admin_email', 'raw_*', 'clean_*' ],
        'email_sender_address' => [ 'admin_email', 'raw_*', 'clean_*' ],
        'email_subject' => [ 'admin_email', 'domain', 'raw_*', 'clean_*' ],
        'attachment_tokens' => [ 'raw_*', 'clean_*', 'field_*', 'table_*' ],
        'file_name' => [ 'admin_email', 'raw_*', 'clean_*', 'field_*', 'table_*' ],
        'file_content' => [ 'admin_email', 'raw_*', 'clean_*', 'field_*', 'table_*' ],
        'email_text' => [ 'admin_email', 'domain', 'raw_*', 'clean_*', 'field_*', 'table_*' ],
        'email_html' => [ 'admin_email', 'domain', 'raw_*', 'clean_*', 'field_*', 'table_*' ]
    ],
    'ctlg_entity_status_duplicate'   => [
        'recipients' => [ 'admin_email', 'raw_*', 'clean_*', 'rawOld_*', 'cleanOld_*' ],
        'email_replyTo' => [ 'admin_email', 'raw_*', 'clean_*', 'rawOld_*', 'cleanOld_*' ],
        'email_sender_name' => [ 'admin_email', 'raw_*', 'clean_*', 'rawOld_*', 'cleanOld_*' ],
        'email_recipient_cc' => [ 'admin_email', 'raw_*', 'clean_*', 'rawOld_*', 'cleanOld_*' ],
        'email_recipient_bcc' => [ 'admin_email', 'raw_*', 'clean_*', 'rawOld_*', 'cleanOld_*' ],
        'email_sender_address' => [ 'admin_email', 'raw_*', 'clean_*', 'rawOld_*', 'cleanOld_*' ],
        'email_subject' => [ 'admin_email', 'domain', 'raw_*', 'clean_*', 'rawOld_*', 'cleanOld_*' ],
        'attachment_tokens' => [ 'raw_*', 'clean_*', 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ],
        'file_name' => [ 'admin_email', 'raw_*', 'clean_*', 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ],
        'file_content' => [ 'admin_email', 'raw_*', 'clean_*', 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ],
        'email_text' => [ 'admin_email', 'domain', 'raw_*', 'clean_*', 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ],
        'email_html' => [ 'admin_email', 'domain', 'raw_*', 'clean_*', 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ]
    ],
    'ctlg_entity_status_update' => [
        'recipients' => [ 'admin_email', 'raw_*', 'clean_*', 'rawOld_*', 'cleanOld_*' ],
        'email_replyTo' => [ 'admin_email', 'raw_*', 'clean_*', 'rawOld_*', 'cleanOld_*' ],
        'email_sender_name' => [ 'admin_email', 'raw_*', 'clean_*', 'rawOld_*', 'cleanOld_*' ],
        'email_recipient_cc' => [ 'admin_email', 'raw_*', 'clean_*', 'rawOld_*', 'cleanOld_*' ],
        'email_recipient_bcc' => [ 'admin_email', 'raw_*', 'clean_*', 'rawOld_*', 'cleanOld_*' ],
        'email_sender_address' => [ 'admin_email', 'raw_*', 'clean_*', 'rawOld_*', 'cleanOld_*' ],
        'email_subject' => [ 'admin_email', 'domain', 'raw_*', 'clean_*', 'rawOld_*', 'cleanOld_*' ],
        'attachment_tokens' => [ 'raw_*', 'clean_*', 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ],
        'file_name' => [ 'admin_email', 'raw_*', 'clean_*', 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ],
        'file_content' => [ 'admin_email', 'raw_*', 'clean_*', 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ],
        'email_text' => [ 'admin_email', 'domain', 'raw_*', 'clean_*', 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ],
        'email_html' => [ 'admin_email', 'domain', 'raw_*', 'clean_*', 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ]
    ],
    'ctlg_entity_status_delete' => [
        'recipients' => [ 'admin_email', 'rawOld_*', 'cleanOld_*' ],
        'email_replyTo' => [ 'admin_email', 'rawOld_*', 'cleanOld_*' ],
        'email_sender_name' => [ 'admin_email', 'rawOld_*', 'cleanOld_*' ],
        'email_recipient_cc' => [ 'admin_email', 'rawOld_*', 'cleanOld_*' ],
        'email_recipient_bcc' => [ 'admin_email', 'rawOld_*', 'cleanOld_*' ],
        'email_sender_address' => [ 'admin_email', 'rawOld_*', 'cleanOld_*' ],
        'email_subject' => [ 'admin_email', 'domain', 'rawOld_*', 'cleanOld_*' ],
        'attachment_tokens' => [ 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ],
        'file_name' => [ 'admin_email', 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ],
        'file_content' => [ 'admin_email', 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ],
        'email_text' => [ 'admin_email', 'domain', 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ],
        'email_html' => [ 'admin_email', 'domain', 'field_*', 'table_*', 'rawOld_*', 'cleanOld_*' ],
    ]
];

$GLOBALS['CM_ICON_SET'] = [
    'new' => 'system/modules/catalog-manager/assets/icons/new.svg',
    'pdf' => 'system/modules/catalog-manager/assets/icons/pdf.svg',
    'edit' => 'system/modules/catalog-manager/assets/icons/edit.svg',
    'copy' => 'system/modules/catalog-manager/assets/icons/copy.svg',
    'delete' => 'system/modules/catalog-manager/assets/icons/delete.svg'
];
}

namespace  {
/*
 * Copyright MADE/YOUR/DAY OG <mail@madeyourday.net>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/**
 * RockSolid Frontend Helper configuration
 *
 * @author Martin Auswöger <martin@madeyourday.net>
 */

$GLOBALS['TL_HOOKS']['parseFrontendTemplate'][] = array('rocksolid_frontend_helper.frontend_hooks', 'parseFrontendTemplateHook');
$GLOBALS['TL_HOOKS']['outputFrontendTemplate'][] = array('rocksolid_frontend_helper.frontend_hooks', 'outputFrontendTemplateHook');
$GLOBALS['TL_HOOKS']['parseTemplate'][] = array('rocksolid_frontend_helper.frontend_hooks', 'parseTemplateHook');
$GLOBALS['TL_HOOKS']['parseWidget'][] = array('rocksolid_frontend_helper.frontend_hooks', 'parseWidgetHook');
$GLOBALS['TL_HOOKS']['parseArticles'][] = array('rocksolid_frontend_helper.frontend_hooks', 'parseArticlesHook');
$GLOBALS['TL_HOOKS']['getAllEvents'][] = array('rocksolid_frontend_helper.frontend_hooks', 'getAllEventsHook');
$GLOBALS['TL_HOOKS']['getContentElement'][] = array('rocksolid_frontend_helper.frontend_hooks', 'getContentElementHook');
$GLOBALS['TL_HOOKS']['getFrontendModule'][] = array('rocksolid_frontend_helper.frontend_hooks', 'getFrontendModuleHook');
$GLOBALS['TL_HOOKS']['getArticle'][] = array('rocksolid_frontend_helper.frontend_hooks', 'getArticleHook');

$GLOBALS['TL_HOOKS']['loadDataContainer'][] = array('rocksolid_frontend_helper.backend_hooks', 'loadDataContainerHook');
$GLOBALS['TL_HOOKS']['initializeSystem'][] = array('rocksolid_frontend_helper.backend_hooks', 'initializeSystemHook');

$GLOBALS['TL_PERMISSIONS'][] = 'rocksolidFrontendHelperOperations';
$GLOBALS['TL_PERMISSIONS'][] = 'rocksolidFrontendHelperContentElements';
}

namespace  {
/*
 * Copyright MADE/YOUR/DAY OG <mail@madeyourday.net>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/**
 * RockSolid Mega Menu back end modules configuration
 *
 * @author Martin Auswöger <martin@madeyourday.net>
 */

$GLOBALS['TL_MODELS']['tl_rocksolid_mega_menu'] = 'MadeYourDay\\RockSolidMegaMenu\\Model\\MenuModel';
$GLOBALS['TL_MODELS']['tl_rocksolid_mega_menu_column'] = 'MadeYourDay\\RockSolidMegaMenu\\Model\\MenuColumnModel';

array_insert($GLOBALS['BE_MOD']['design'], 2, array(
	'rocksolid_mega_menu' => array(
		'tables' => array(
			'tl_rocksolid_mega_menu',
			'tl_rocksolid_mega_menu_column',
			'tl_rocksolid_mega_menu_license',
		),
		'icon' => 'bundles/rocksolidmegamenu/img/icon.png',
	),
));

array_insert($GLOBALS['FE_MOD'], 2, array(
	'navigationMenu' => array(
		'rocksolid_mega_menu' => 'MadeYourDay\\RockSolidMegaMenu\\Module\\Menu',
		'rocksolid_mega_menu_custom' => 'MadeYourDay\\RockSolidMegaMenu\\Module\\MenuCustom',
	),
));
}

namespace  {
/*
 * Copyright MADE/YOUR/DAY <mail@madeyourday.co>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/**
 * RockSolid Theme Assistant back end modules configuration
 *
 * @author Martin Auswöger <martin@madeyourday.co>
 */

$GLOBALS['TL_HOOKS']['loadDataContainer'][] = array('MadeYourDay\\RockSolidThemeAssistant\\ThemeAssistant', 'loadDataContainerHook');
$GLOBALS['TL_HOOKS']['executePostActions'][] = array('MadeYourDay\\RockSolidThemeAssistant\\ThemeAssistant', 'executePostActionsHook');

$GLOBALS['BE_MOD']['design']['rocksolid_theme_assistant'] = array(
	'tables' => array('rocksolid_theme_assistant'),
	'icon'   => 'bundles/rocksolidthemeassistant/images/icon.png',
);

$GLOBALS['BE_FFL']['mydMultiListWizard'] = 'MadeYourDay\\RockSolidThemeAssistant\\Widget\\MultiListWizard';
}

namespace  {
/*
 * This file is part of Contao.
 *
 * (c) Leo Feyer
 *
 * @license LGPL-3.0-or-later
 */

use Contao\ModuleListing;

// Front end module
$GLOBALS['FE_MOD']['application']['listing'] = ModuleListing::class;
}

namespace  {
/*
 * This file is part of Contao.
 *
 * (c) Leo Feyer
 *
 * @license LGPL-3.0-or-later
 */

use Contao\Calendar;
use Contao\CalendarEventsModel;
use Contao\CalendarFeedModel;
use Contao\CalendarModel;
use Contao\ListWizard;
use Contao\ModuleCalendar;
use Contao\ModuleEventlist;
use Contao\ModuleEventMenu;
use Contao\ModuleEventReader;
use Contao\TableWizard;

// Back end modules
$GLOBALS['BE_MOD']['content']['calendar'] = array
(
	'tables'      => array('tl_calendar', 'tl_calendar_events', 'tl_calendar_feed', 'tl_content'),
	'table'       => array(TableWizard::class, 'importTable'),
	'list'        => array(ListWizard::class, 'importList')
);

// Front end modules
$GLOBALS['FE_MOD']['events'] = array
(
	'calendar'    => ModuleCalendar::class,
	'eventreader' => ModuleEventReader::class,
	'eventlist'   => ModuleEventlist::class,
	'eventmenu'   => ModuleEventMenu::class
);

// Cron jobs
$GLOBALS['TL_CRON']['daily']['generateCalendarFeeds'] = array(Calendar::class, 'generateFeeds');

// Style sheet
if (defined('TL_MODE') && TL_MODE == 'BE')
{
	$GLOBALS['TL_CSS'][] = 'bundles/contaocalendar/calendar.min.css|static';
}

// Register hooks
$GLOBALS['TL_HOOKS']['removeOldFeeds'][] = array(Calendar::class, 'purgeOldFeeds');
$GLOBALS['TL_HOOKS']['getSearchablePages'][] = array(Calendar::class, 'getSearchablePages');
$GLOBALS['TL_HOOKS']['generateXmlFiles'][] = array(Calendar::class, 'generateFeeds');

// Add permissions
$GLOBALS['TL_PERMISSIONS'][] = 'calendars';
$GLOBALS['TL_PERMISSIONS'][] = 'calendarp';
$GLOBALS['TL_PERMISSIONS'][] = 'calendarfeeds';
$GLOBALS['TL_PERMISSIONS'][] = 'calendarfeedp';

// Models
$GLOBALS['TL_MODELS']['tl_calendar_events'] = CalendarEventsModel::class;
$GLOBALS['TL_MODELS']['tl_calendar_feed'] = CalendarFeedModel::class;
$GLOBALS['TL_MODELS']['tl_calendar'] = CalendarModel::class;
}

namespace  {
/*
 * This file is part of Contao.
 *
 * (c) Leo Feyer
 *
 * @license LGPL-3.0-or-later
 */

use Contao\ModuleNewsletterList;
use Contao\ModuleNewsletterReader;
use Contao\ModuleSubscribe;
use Contao\ModuleUnsubscribe;
use Contao\Newsletter;
use Contao\NewsletterChannelModel;
use Contao\NewsletterDenyListModel;
use Contao\NewsletterModel;
use Contao\NewsletterRecipientsModel;

// Back end modules
$GLOBALS['BE_MOD']['content']['newsletter'] = array
(
	'tables'     => array('tl_newsletter_channel', 'tl_newsletter', 'tl_newsletter_recipients'),
	'send'       => array(Newsletter::class, 'send'),
	'import'     => array(Newsletter::class, 'importRecipients'),
	'stylesheet' => 'bundles/contaonewsletter/newsletter.min.css'
);

// Front end modules
$GLOBALS['FE_MOD']['newsletter'] = array
(
	'subscribe'        => ModuleSubscribe::class,
	'unsubscribe'      => ModuleUnsubscribe::class,
	'newsletterlist'   => ModuleNewsletterList::class,
	'newsletterreader' => ModuleNewsletterReader::class
);

// Register hooks
$GLOBALS['TL_HOOKS']['createNewUser'][] = array(Newsletter::class, 'createNewUser');
$GLOBALS['TL_HOOKS']['activateAccount'][] = array(Newsletter::class, 'activateAccount');
$GLOBALS['TL_HOOKS']['getSearchablePages'][] = array(Newsletter::class, 'getSearchablePages');
$GLOBALS['TL_HOOKS']['closeAccount'][] = array(Newsletter::class, 'removeSubscriptions');

// Add permissions
$GLOBALS['TL_PERMISSIONS'][] = 'newsletters';
$GLOBALS['TL_PERMISSIONS'][] = 'newsletterp';

// Cron jobs
$GLOBALS['TL_CRON']['daily']['purgeNewsletterSubscriptions'] = array(Newsletter::class, 'purgeSubscriptions');

// Models
$GLOBALS['TL_MODELS']['tl_newsletter_channel'] = NewsletterChannelModel::class;
$GLOBALS['TL_MODELS']['tl_newsletter_deny_list'] = NewsletterDenyListModel::class;
$GLOBALS['TL_MODELS']['tl_newsletter'] = NewsletterModel::class;
$GLOBALS['TL_MODELS']['tl_newsletter_recipients'] = NewsletterRecipientsModel::class;
}

namespace  {
/*
 * Copyright MADE/YOUR/DAY OG <mail@madeyourday.net>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/**
 * RockSolid Icon Picker configuration
 *
 * @author Martin Auswöger <martin@madeyourday.net>
 */

$GLOBALS['BE_FFL']['rocksolid_icon_picker'] = 'MadeYourDay\\RockSolidIconPicker\\Widget\\IconPicker';

$GLOBALS['TL_PURGE']['custom']['rocksolid_icon_picker'] = array(
	'callback' => array('MadeYourDay\\RockSolidIconPicker\\Widget\\IconPicker', 'purgeCache'),
);
}

namespace  {
/**
 * @copyright  Softleister 2007-2021
 * @author     Softleister <info@softleister.de>
 * @package    BackupDB - Database backup
 * @license    LGPL
 * @see	       https://github.com/do-while/contao-BackupDB
 */

define('BACKUPDB_RUN_LAST',  'RunBackupDB.last');
define('BACKUPDB_CRON_LAST', 'AutoBackupDB.last');


/**
 * -------------------------------------------------------------------------
 * BACK END MODULES
 * -------------------------------------------------------------------------
 */
array_insert($GLOBALS['BE_MOD']['system'], -1, array
(
	'BackupDB' => array (
		'callback'   => 'Softleister\BackupDB\ModuleBackupDB',
		'stylesheet' => 'bundles/softleisterbackupdb/styles.min.css',
	)
));
}

namespace  {
/**
 * This file is part of contao-community-alliance/dc-general.
 *
 * (c) 2013-2022 Contao Community Alliance.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * This project is provided in good faith and hope to be usable by anyone.
 *
 * @package    contao-community-alliance/dc-general
 * @author     Christian Schiffler <c.schiffler@cyberspectrum.de>
 * @author     Stefan Heimes <stefan_heimes@hotmail.com>
 * @author     Tristan Lins <tristan.lins@bit3.de>
 * @author     Andreas Isaak <andy.jared@googlemail.com>
 * @author     Sven Baumann <baumann.sv@gmail.com>
 * @author     David Molineus <david.molineus@netzmacht.de>
 * @copyright  2013-2022 Contao Community Alliance.
 * @license    https://github.com/contao-community-alliance/dc-general/blob/master/LICENSE LGPL-3.0-or-later
 * @filesource
 */

use ContaoCommunityAlliance\DcGeneral\Contao\View\Contao2BackendView\Widget\FileTree;
use ContaoCommunityAlliance\DcGeneral\Contao\View\Contao2BackendView\TreePicker;

// JS
if (defined('TL_MODE') && TL_MODE === 'BE') {
    $GLOBALS['TL_JAVASCRIPT']['cca.dc-general.generalDriver_src'] = 'bundles/ccadcgeneral/js/generalDriver_src.js';
    $GLOBALS['TL_JAVASCRIPT']['cca.dc-general.vanillaGeneral']    = 'bundles/ccadcgeneral/js/vanillaGeneral.js';
}

$GLOBALS['BE_FFL']['DcGeneralTreePicker'] = TreePicker::class;

$GLOBALS['TL_HOOKS']['executePostActions'] = \array_merge(
    (array) $GLOBALS['TL_HOOKS']['executePostActions'],
    [
        [TreePicker::class, 'updateAjax'],
        [FileTree::class, 'updateAjax']
    ]
);
}

namespace  {
/**
 * dcawizard extension for Contao Open Source CMS
 *
 * @copyright  Copyright (c) 2014, terminal42 gmbh
 * @author     terminal42 gmbh <info@terminal42.ch>
 * @license    http://opensource.org/licenses/lgpl-3.0.html LGPL
 * @link       https://github.com/terminal42/contao-dcawizard
 */

/**
 * Backend form fields
 */
$GLOBALS['BE_FFL']['dcaWizard']             = 'DcaWizard';
$GLOBALS['BE_FFL']['dcaWizardMultilingual'] = 'DcaWizardMultilingual';

/**
 * Hooks
 */
$GLOBALS['TL_HOOKS']['executePostActions'][] = array('DcaWizardHelper', 'handleAjaxActions');
$GLOBALS['TL_HOOKS']['loadDataContainer'][] = array('DcaWizardHelper', 'loadDataContainer');
}

namespace  {
/**
 * leads Extension for Contao Open Source CMS
 *
 * @copyright  Copyright (c) 2011-2015, terminal42 gmbh
 * @author     terminal42 gmbh <info@terminal42.ch>
 * @license    http://opensource.org/licenses/lgpl-3.0.html LGPL
 * @link       http://github.com/terminal42/contao-leads
 */

/**
 * Add the tl_lead_export table to form module
 */
$GLOBALS['BE_MOD']['content']['form']['tables'][] = 'tl_lead_export';

/**
 * Fake back end module
 */
array_insert($GLOBALS['BE_MOD'], 1, array('leads'=> array
(
    'lead' => array
    (
        'tables'        => array('tl_lead', 'tl_lead_data'),
        'javascript'    => 'system/modules/leads/assets/leads.js',
        'stylesheet'    => 'system/modules/leads/assets/leads.css',
        'show'          => array('tl_lead', 'show'),
        'export'        => array('tl_lead', 'export'),
        'notification'  => array('tl_lead', 'sendNotification'),
    ),
)));

// Load icon in Contao 4.2+ backend
if ('BE' === TL_MODE) {
    $GLOBALS['TL_CSS'][] = 'system/modules/leads/assets/leads.css';
}


/**
 * Hooks
 */
$GLOBALS['TL_HOOKS']['loadLanguageFile'][]  = array('Leads\\Leads', 'loadLeadName');
$GLOBALS['TL_HOOKS']['getUserNavigation'][] = array('Leads\\Leads', 'loadBackendModules');
$GLOBALS['TL_HOOKS']['processFormData'][]   = array('Leads\\Leads', 'processFormData');
$GLOBALS['TL_HOOKS']['getLeadsExportRow'][] = array('Leads\\Leads', 'handleSystemColumnExports');
$GLOBALS['TL_HOOKS']['getLeadsExportRow'][] = array('Leads\\Leads', 'handleTokenExports');

/**
 * Leads export types
 */
$GLOBALS['LEADS_EXPORT'] = array
(
    'csv'   => 'Leads\\Exporter\\Csv',
    'xls'   => 'Leads\\Exporter\\Xls',
    'xlsx'  => 'Leads\\Exporter\\Xlsx',
);

/**
 * Data transformers
 */
$GLOBALS['LEADS_DATA_TRANSFORMERS'] = array
(
    'raw'               => 'Leads\\DataTransformer\\RawTransformer',
    'date'              => 'Leads\\DataTransformer\\DateTransformer',
    'datim'             => 'Leads\\DataTransformer\\DatimTransformer',
    'time'              => 'Leads\\DataTransformer\\TimeTransformer',
    'yesno'             => 'Leads\\DataTransformer\\YesNoTransformer',
    'uuidToFilePath'    => 'Leads\\DataTransformer\\UuidToFilePathTransformer',
);
}

namespace  {
/**
 * Contao Open Source CMS
 *
 * @copyright  MEN AT WORK 2014
 * @package    syncCto
 * @license    GNU/LGPL
 * @filesource
 */

$objInput = \Input::getInstance();

/**
 * Current syncCto version
 */
$GLOBALS['SYC_VERSION'] = '3.2.0';

/**
 * Back end modules
 */
$i = array_search('system', array_keys($GLOBALS['BE_MOD']));
array_insert($GLOBALS['BE_MOD'], $i + 1, array
(
    'syncCto' => array
    (
        'syncCto_settings' => array
        (
            'tables' => array('tl_syncCto_settings'),
            'icon'   => 'bundles/synccto/images/nav/iconSettings.png'
        ),
        'synccto_clients'  => array
        (
            'tables'     => array
            (
                'tl_synccto_clients',
                'tl_syncCto_clients_syncTo',
                'tl_syncCto_clients_syncFrom',
                'tl_syncCto_clients_showExtern'
            ),
            'icon'       => 'bundles/synccto/images/nav/iconClients.png',
            'callback'   => 'SyncCtoModuleClient',
            'stylesheet' => 'bundles/synccto/css/systemcheck.css',
        ),
        //        'syncCto_backups' => array
        //        (
        //            'tables' => array
        //            (
        //                'tl_syncCto_backup_file',
        //                'tl_syncCto_backup_db',
        //                'tl_syncCto_restore_file',
        //                'tl_syncCto_restore_db'
        //            ),
        //            'icon'              => 'bundles/synccto/images/nav/iconBackups.png',
        //            'callback'          => 'SyncCtoModuleBackup',
        //        ),
        'syncCto_check'    => array
        (
            'icon'       => 'bundles/synccto/images/nav/iconCheck.png',
            'callback'   => 'SyncCtoModuleCheck',
            'stylesheet' => 'bundles/synccto/css/systemcheck.css',
            'style'      => 'hidden'
        )
    )
));

/**
 * Mime types
 */
$GLOBALS['SYC_CONFIG']['mime_types'] = array_merge((array)$GLOBALS['SYC_CONFIG']['mime_types'],
    array
    (
        // Application files
        'xl'    => array('application/excel', 'iconOFFICE.gif'),
        'xls'   => array('application/excel', 'iconOFFICE.gif'),
        'hqx'   => array('application/mac-binhex40', 'iconPLAIN.gif'),
        'cpt'   => array('application/mac-compactpro', 'iconPLAIN.gif'),
        'bin'   => array('application/macbinary', 'iconPLAIN.gif'),
        'doc'   => array('application/msword', 'iconOFFICE.gif'),
        'word'  => array('application/msword', 'iconOFFICE.gif'),
        'cto'   => array('application/octet-stream', 'iconCTO.gif'),
        'dms'   => array('application/octet-stream', 'iconPLAIN.gif'),
        'lha'   => array('application/octet-stream', 'iconPLAIN.gif'),
        'lzh'   => array('application/octet-stream', 'iconPLAIN.gif'),
        'exe'   => array('application/octet-stream', 'iconPLAIN.gif'),
        'class' => array('application/octet-stream', 'iconPLAIN.gif'),
        'so'    => array('application/octet-stream', 'iconPLAIN.gif'),
        'sea'   => array('application/octet-stream', 'iconPLAIN.gif'),
        'dll'   => array('application/octet-stream', 'iconPLAIN.gif'),
        'oda'   => array('application/oda', 'iconPLAIN.gif'),
        'pdf'   => array('application/pdf', 'iconPDF.gif'),
        'ai'    => array('application/postscript', 'iconPLAIN.gif'),
        'eps'   => array('application/postscript', 'iconPLAIN.gif'),
        'ps'    => array('application/postscript', 'iconPLAIN.gif'),
        'pps'   => array('application/powerpoint', 'iconOFFICE.gif'),
        'ppt'   => array('application/powerpoint', 'iconOFFICE.gif'),
        'smi'   => array('application/smil', 'iconPLAIN.gif'),
        'smil'  => array('application/smil', 'iconPLAIN.gif'),
        'mif'   => array('application/vnd.mif', 'iconPLAIN.gif'),
        'odc'   => array('application/vnd.oasis.opendocument.chart', 'iconOFFICE.gif'),
        'odf'   => array('application/vnd.oasis.opendocument.formula', 'iconOFFICE.gif'),
        'odg'   => array('application/vnd.oasis.opendocument.graphics', 'iconOFFICE.gif'),
        'odi'   => array('application/vnd.oasis.opendocument.image', 'iconOFFICE.gif'),
        'odp'   => array('application/vnd.oasis.opendocument.presentation', 'iconOFFICE.gif'),
        'ods'   => array('application/vnd.oasis.opendocument.spreadsheet', 'iconOFFICE.gif'),
        'odt'   => array('application/vnd.oasis.opendocument.text', 'iconOFFICE.gif'),
        'wbxml' => array('application/wbxml', 'iconPLAIN.gif'),
        'wmlc'  => array('application/wmlc', 'iconPLAIN.gif'),
        'dmg'   => array('application/x-apple-diskimage', 'iconRAR.gif'),
        'dcr'   => array('application/x-director', 'iconPLAIN.gif'),
        'dir'   => array('application/x-director', 'iconPLAIN.gif'),
        'dxr'   => array('application/x-director', 'iconPLAIN.gif'),
        'dvi'   => array('application/x-dvi', 'iconPLAIN.gif'),
        'gtar'  => array('application/x-gtar', 'iconRAR.gif'),
        'inc'   => array('application/x-httpd-php', 'iconPHP.gif'),
        'php'   => array('application/x-httpd-php', 'iconPHP.gif'),
        'php3'  => array('application/x-httpd-php', 'iconPHP.gif'),
        'php4'  => array('application/x-httpd-php', 'iconPHP.gif'),
        'php5'  => array('application/x-httpd-php', 'iconPHP.gif'),
        'phtml' => array('application/x-httpd-php', 'iconPHP.gif'),
        'phps'  => array('application/x-httpd-php-source', 'iconPHP.gif'),
        'js'    => array('application/x-javascript', 'iconJS.gif'),
        'psd'   => array('application/x-photoshop', 'iconPLAIN.gif'),
        'rar'   => array('application/x-rar', 'iconRAR.gif'),
        'fla'   => array('application/x-shockwave-flash', 'iconSWF.gif'),
        'swf'   => array('application/x-shockwave-flash', 'iconSWF.gif'),
        'sit'   => array('application/x-stuffit', 'iconRAR.gif'),
        'tar'   => array('application/x-tar', 'iconRAR.gif'),
        'tgz'   => array('application/x-tar', 'iconRAR.gif'),
        'xhtml' => array('application/xhtml+xml', 'iconPLAIN.gif'),
        'xht'   => array('application/xhtml+xml', 'iconPLAIN.gif'),
        'zip'   => array('application/zip', 'iconRAR.gif'),

        // Audio files
        'm4a'   => array('audio/x-m4a', 'iconAUDIO.gif'),
        'mp3'   => array('audio/mp3', 'iconAUDIO.gif'),
        'wma'   => array('audio/wma', 'iconAUDIO.gif'),
        'mpeg'  => array('audio/mpeg', 'iconAUDIO.gif'),
        'wav'   => array('audio/wav', 'iconAUDIO.gif'),
        'ogg'   => array('audio/ogg', 'iconAUDIO.gif'),
        'mid'   => array('audio/midi', 'iconAUDIO.gif'),
        'midi'  => array('audio/midi', 'iconAUDIO.gif'),
        'aif'   => array('audio/x-aiff', 'iconAUDIO.gif'),
        'aiff'  => array('audio/x-aiff', 'iconAUDIO.gif'),
        'aifc'  => array('audio/x-aiff', 'iconAUDIO.gif'),
        'ram'   => array('audio/x-pn-realaudio', 'iconAUDIO.gif'),
        'rm'    => array('audio/x-pn-realaudio', 'iconAUDIO.gif'),
        'rpm'   => array('audio/x-pn-realaudio-plugin', 'iconAUDIO.gif'),
        'ra'    => array('audio/x-realaudio', 'iconAUDIO.gif'),

        // Images
        'bmp'   => array('image/bmp', 'iconBMP.gif'),
        'gif'   => array('image/gif', 'iconGIF.gif'),
        'jpeg'  => array('image/jpeg', 'iconJPG.gif'),
        'jpg'   => array('image/jpeg', 'iconJPG.gif'),
        'jpe'   => array('image/jpeg', 'iconJPG.gif'),
        'png'   => array('image/png', 'iconTIF.gif'),
        'tiff'  => array('image/tiff', 'iconTIF.gif'),
        'tif'   => array('image/tiff', 'iconTIF.gif'),

        // Mailbox files
        'eml'   => array('message/rfc822', 'iconPLAIN.gif'),

        // Text files
        'asp'   => array('text/asp', 'iconPLAIN.gif'),
        'css'   => array('text/css', 'iconCSS.gif'),
        'html'  => array('text/html', 'iconHTML.gif'),
        'htm'   => array('text/html', 'iconHTML.gif'),
        'shtml' => array('text/html', 'iconHTML.gif'),
        'txt'   => array('text/plain', 'iconPLAIN.gif'),
        'text'  => array('text/plain', 'iconPLAIN.gif'),
        'log'   => array('text/plain', 'iconPLAIN.gif'),
        'rtx'   => array('text/richtext', 'iconPLAIN.gif'),
        'rtf'   => array('text/rtf', 'iconPLAIN.gif'),
        'xml'   => array('text/xml', 'iconPLAIN.gif'),
        'xsl'   => array('text/xml', 'iconPLAIN.gif'),

        // Videos
        'mp4'   => array('video/mp4', 'iconVIDEO.gif'),
        'm4v'   => array('video/x-m4v', 'iconVIDEO.gif'),
        'mov'   => array('video/mov', 'iconVIDEO.gif'),
        'wmv'   => array('video/wmv', 'iconVIDEO.gif'),
        'webm'  => array('video/webm', 'iconVIDEO.gif'),
        'qt'    => array('video/quicktime', 'iconVIDEO.gif'),
        'rv'    => array('video/vnd.rn-realvideo', 'iconVIDEO.gif'),
        'avi'   => array('video/x-msvideo', 'iconVIDEO.gif'),
        'ogv'   => array('video/ogg', 'iconVIDEO.gif'),
        'movie' => array('video/x-sgi-movie', 'iconVIDEO.gif')
    )
);

/**
 * Hooks
 */
$GLOBALS['TL_HOOKS']['executePreActions'][]          = array('MenAtWork\SyncCto\Helper\Ping', 'pingClientStatus');
$GLOBALS['TL_HOOKS']['parseBackendTemplate'][]       = array('SyncCtoHelper', 'checkExtensions');
$GLOBALS['TL_HOOKS']['parseBackendTemplate'][]       = array('SyncCtoHelper', 'checkLockStatus');
$GLOBALS['TL_HOOKS']['addCustomRegexp'][]            = array('SyncCtoHelper', 'customRegexp');
$GLOBALS['TL_HOOKS']['parseBackendTemplate'][]       = array('SyncCtoHelper', 'addLegend');
$GLOBALS['TL_HOOKS']['syncExecuteFinalOperations'][] = array('SyncCtoDatabaseUpdater', 'runAutoUpdate');

/**
 * Permissions
 */
$GLOBALS['TL_PERMISSIONS'][] = 'syncCto_clients';
$GLOBALS['TL_PERMISSIONS'][] = 'syncCto_clients_p';
$GLOBALS['TL_PERMISSIONS'][] = 'syncCto_sync_options';
$GLOBALS['TL_PERMISSIONS'][] = 'syncCto_tables';

/**
 * Callbacks are only used for overview screen
 */
$strDo    = $objInput->get("do");
$strTable = $objInput->get("table");
$strAct   = $objInput->get("act");

if ($strDo == 'syncCto_backups' && $strTable != '' && ($strAct == 'edit' || $strAct == 'create') && TL_MODE == 'BE') {
    unset($GLOBALS['BE_MOD']['syncCto']['syncCto_backups']['callback']);
}

if ($strDo == 'synccto_clients' && $strAct != 'start' && in_array($strTable, array(
        'tl_syncCto_clients_syncTo',
        'tl_syncCto_clients_syncFrom',
        'tl_syncCto_clients_showExtern',
        ''
    )) && TL_MODE == 'BE') {
    unset($GLOBALS['BE_MOD']['syncCto']['synccto_clients']['callback']);
}

/**
 * Include attention CSS
 */
if ($GLOBALS['TL_CONFIG']['syncCto_attentionFlag'] == true) {
    $GLOBALS['TL_CSS'][] = 'bundles/synccto/css/attention.css';
}

// Size limit for files in bytes, will be checked
$GLOBALS['SYC_SIZE']['limit'] = 524288000;

// Size limit for files in bytes, completely ignored
$GLOBALS['SYC_SIZE']['limit_ignore'] = 838860800;

/**
 * Blacklists
 */

// Add some files to the blacklist for the DBAFS from contao.
$arrFileSyncExclude                      = trimsplit(',', $GLOBALS['TL_CONFIG']['fileSyncExclude']);
$arrFileSyncExclude[]                    = 'syncCto_backups/debug';
$GLOBALS['TL_CONFIG']['fileSyncExclude'] = implode(',', $arrFileSyncExclude);

// Tables
$GLOBALS['SYC_CONFIG']['table_hidden'] = array_merge((array)$GLOBALS['SYC_CONFIG']['table_hidden'], array
(
    'tl_files',
    'tl_log',
    'tl_lock',
    'tl_cron',
    'tl_opt_in',
    'tl_opt_in_related',
    'tl_remember_me',
    'tl_session',
    'tl_search',
    'tl_search_index',
    'tl_undo',
    'tl_version',
    'tl_comments',
    'tl_comments_notify',
    'tl_synccto_clients',
    'tl_synccto_stats',
    'tl_trusted_device'
));

// Folders
$GLOBALS['SYC_CONFIG']['folder_blacklist'] = array_merge((array)$GLOBALS['SYC_CONFIG']['folder_blacklist'], array
(
    '*/syncCto_backups/',
    '*/.git'
));

// Files only sync.
$GLOBALS['SYC_CONFIG']['file_blacklist'] = array_merge((array)$GLOBALS['SYC_CONFIG']['file_blacklist'], array
(
    'TL_ROOT/.env',
    'TL_ROOT/.htaccess',
    'TL_ROOT/.htpasswd',
    'TL_ROOT/composer*',
    '.DS_Store'
));

// Local config
$GLOBALS['SYC_CONFIG']['local_blacklist'] = array_merge((array)$GLOBALS['SYC_CONFIG']['local_blacklist'], array
(
    'ctoCom_APIKey',
    'ctoCom_disableRefererCheck',
    'ctoCom_responseLength',
    'ctoCom_handshake',
    'disableCron',
    'disableRefererCheck',
    'syncCto_debug_mode',
    'syncCto_attentionFlag',
    'syncCto_auto_db_updater'
));

/**
 * Whitelist
 */
$GLOBALS['SYC_CONFIG']['folder_whitelist'] = array_merge((array)$GLOBALS['SYC_CONFIG']['folder_whitelist'], array
(
    'templates'
));

/**
 * Sync options
 */
// Core ! The core isn't possible since we have a compser system. We have to rewrite some functions here first.
$GLOBALS['SYC_CONFIG']['sync_options']['core'][] =  'core_change';
$GLOBALS['SYC_CONFIG']['sync_options']['core'][] =  'core_delete';
// User
$GLOBALS['SYC_CONFIG']['sync_options']['user'][] = 'user_change';
$GLOBALS['SYC_CONFIG']['sync_options']['user'][] = 'user_delete';
// User
$GLOBALS['SYC_CONFIG']['sync_options']['configfiles'][] = 'localconfig_update';

/**
 * Maintance options
 */
$GLOBALS['SYC_CONFIG']['maintance_options'] = array_merge((array)$GLOBALS['SYC_CONFIG']['maintance_options'], array
(
    'temp_tables',
    'temp_folders',
    'xml_create',
));

/**
 * Global configuration
 */
$GLOBALS['SYC_PATH']['db']    = $GLOBALS['TL_CONFIG']['uploadPath'] . '/syncCto_backups/database/';
$GLOBALS['SYC_PATH']['file']  = $GLOBALS['TL_CONFIG']['uploadPath'] . '/syncCto_backups/files/';
$GLOBALS['SYC_PATH']['debug'] = $GLOBALS['TL_CONFIG']['uploadPath'] . '/syncCto_backups/debug/';
$GLOBALS['SYC_PATH']['tmp']   = "system/tmp/";

/**
 * Language mapping for database lookup
 */
$GLOBALS['SYC_CONFIG']['database_mapping'] = array_merge((array)$GLOBALS['SYC_CONFIG']['database_mapping'], array
(
    'tl_module'              => 'modules',
    'tl_member_group'        => 'mgroup',
    'tl_user_group'          => 'group',
    'tl_repository_installs' => 'repository_manager',
    'tl_task'                => 'tasks',
    'tl_theme'               => 'themes',
    'tl_style_sheet'         => 'css'
));

/**
 * Folder/Files replacement
 */

// Default
$GLOBALS['SYC_CONFIG']['folder_file_replacement'][' '] = '_';

// Windows / Unix
$GLOBALS['SYC_CONFIG']['folder_file_replacement']['/'] = '-';

// Windows only
$GLOBALS['SYC_CONFIG']['folder_file_replacement']['\\'] = '-';
$GLOBALS['SYC_CONFIG']['folder_file_replacement'][':']  = '';
$GLOBALS['SYC_CONFIG']['folder_file_replacement']['*']  = '';
$GLOBALS['SYC_CONFIG']['folder_file_replacement']['?']  = '';
$GLOBALS['SYC_CONFIG']['folder_file_replacement']['"']  = '';
$GLOBALS['SYC_CONFIG']['folder_file_replacement']['<']  = '';
$GLOBALS['SYC_CONFIG']['folder_file_replacement']['>']  = '';
$GLOBALS['SYC_CONFIG']['folder_file_replacement']['|']  = '_';

/**
 * CtoCommunication RPC Calls
 */

// - Local Config --------------------------------------------------------------

// Import config
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_IMPORT_CONFIG"] = array
(
    "class"     => "SyncCtoHelper",
    "function"  => "importConfig",
    "typ"       => "POST",
    "parameter" => array("configlist"),
);

// Get config
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_GET_CONFIG"] = array
(
    "class"     => "SyncCtoRPCFunctions",
    "function"  => "getLocalConfig",
    "typ"       => "POST",
    "parameter" => array("ConfigBlacklist"),
);

// Get config
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_CREATE_PATHCONFIG"] = array
(
    "class"     => "SyncCtoHelper",
    "function"  => "createPathconfig",
    "typ"       => "GET",
    "parameter" => false,
);

// - Database ------------------------------------------------------------------

// Run table hashes
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_DATABASE_HASH"] = array
(
    "class"     => "\MenAtWork\SyncCto\Database\Diff",
    "function"  => "getHashForTables",
    "typ"       => "POST",
    "parameter" => array("tables"),
);

// Run Dump
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_RUN_DUMP"] = array
(
    "class"     => "SyncCtoDatabase",
    "function"  => "runDump",
    "typ"       => "POST",
    "parameter" => array("tables", "tempfolder"),
);

// Execute SQL
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_EXECUTE_SQL"] = array
(
    "class"     => "SyncCtoRPCFunctions",
    "function"  => "executeSQL",
    "typ"       => "POST",
    "parameter" => array("sql"),
);

// Load none recommended tables from client
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_NONERECOMMENDED_TABLES"] = array
(
    "class"     => "SyncCtoHelper",
    "function"  => "databaseTablesNoneRecommended",
    "typ"       => "POST",
    "parameter" => false,
);

// Load recommended tables from client
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_RECOMMENDED_TABLES"] = array
(
    "class"     => "SyncCtoHelper",
    "function"  => "databaseTablesRecommended",
    "typ"       => "POST",
    "parameter" => false,
);

// Load recommended tables from client
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_HIDDEN_TABLES"] = array
(
    "class"     => "SyncCtoHelper",
    "function"  => "getTablesHidden",
    "typ"       => "GET",
    "parameter" => false,
);

// Load recommended tables from client
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_HIDDEN_TABLES_PLACEHOLDER"] = array
(
    "class"     => "SyncCtoHelper",
    "function"  => "getPreparedHiddenTablesPlaceholder",
    "typ"       => "GET",
    "parameter" => false,
);

// Get client timestamp
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_TIMESTAMP"] = array
(
    "class"     => "SyncCtoHelper",
    "function"  => "getDatabaseTablesTimestamp",
    "typ"       => "POST",
    "parameter" => array("TableList"),
);

// Import a SQL Zip file into database
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_IMPORT_DATABASE"] = array
(
    "class"     => "SyncCtoDatabase",
    "function"  => "runRestore",
    "typ"       => "POST",
    "parameter" => array("filepath", "additionalSQL"),
);

// Drop tables
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_DROP_TABLES"] = array
(
    "class"     => "SyncCtoDatabase",
    "function"  => "dropTable",
    "typ"       => "POST",
    "parameter" => array("tablelist", "backup"),
);

// - Files ---------------------------------------------------------------------

// Check for deleted files
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_CHECK_DELETE_FILE"] = array
(
    "class"     => "SyncCtoRPCFunctions",
    "function"  => "checkDeleteFiles",
    "typ"       => "POST",
    "parameter" => array("md5", "file"),
);

// Delete a files on a list
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_DELETE_FILE"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "deleteFiles",
    "typ"       => "POST",
    "parameter" => array("filelist", "dbafs"),
);

// Import files into contao file system
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_IMPORT_FILE"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "moveTempFile",
    "typ"       => "POST",
    "parameter" => array("filelist", "dbafs"),
);

// Import files into contao file system
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_UPDATE_DBAFS"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "updateDbafs",
    "typ"       => "POST",
    "parameter" => array("filelist"),
);

// Rebuild a split file
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_REBUILD_SPLITFILE"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "rebuildSplitFiles",
    "typ"       => "POST",
    "parameter" => array("splitname", "splitcount", "movepath", "md5"),
);

// Split a file
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_SPLITFILE"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "splitFiles",
    "typ"       => "POST",
    "parameter" => array("splitname", "destfolder", "destfile", "limit"),
);

// Send a file
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_SEND_FILE"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "saveFiles",
    "typ"       => "POST",
    "parameter" => array("metafiles"),
);

// Get a file
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_GET_FILE"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "getFile",
    "typ"       => "POST",
    "parameter" => array("path"),
);

// Compare 2 filelists
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_CHECKSUM_COMPARE"] = array
(
    "class"     => "SyncCtoRPCFunctions",
    "function"  => "runCecksumCompare",
    "typ"       => "POST",
    "parameter" => array("md5", "file", "disable_dbafs_conflicts"),
);

// Get filelist of contao core
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_CHECKSUM_CORE"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "runChecksumCore",
    "typ"       => "GET",
    "parameter" => false,
);

// Get filelist of file
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_CHECKSUM_FILES"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "runChecksumFiles",
    "typ"       => "POST",
    "parameter" => false,
);

// Get folderlist
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_CHECKSUM_FOLDERS_CORE"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "runChecksumFolderCore",
    "typ"       => "POST",
    "parameter" => false,
);

// Get folderlist
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_CHECKSUM_FOLDERS_FILES"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "runChecksumFolderFiles",
    "typ"       => "POST",
    "parameter" => false,
);

// Search folders which could delete
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_SEARCH_DELETE_FOLDERS"] = array
(
    "class"     => "SyncCtoRPCFunctions",
    "function"  => "searchDeleteFolders",
    "typ"       => "POST",
    "parameter" => array("md5", "file"),
);

// Run a file backup
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_FILEBACKUP"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "runDump",
    "typ"       => "GET",
    "parameter" => false,
);

// Get information from the DBAFS for some files.
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_DBAFS_INFORMATION"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "getDbafsInformationFor",
    "typ"       => "POST",
    "parameter" => array("files"),
);

// - Miscellaneous -------------------------------------------------------------

// Set displayErrors Flag
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_SET_DISPLAY_ERRORS_FLAG"] = array
(
    "class"     => "SyncCtoRPCFunctions",
    "function"  => "setDisplayErrors",
    "typ"       => "POST",
    "parameter" => array("state"),
);

// Set the attention flag
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_SET_ATTENTION_FLAG"] = array
(
    "class"     => "SyncCtoRPCFunctions",
    "function"  => "setAttentionFlag",
    "typ"       => "POST",
    "parameter" => array("state"),
);

// Clear temp folder
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_PURGETEMP"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "purgeTemp",
    "typ"       => "GET",
    "parameter" => false,
);

// Contao Cache.
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_PURGE_CACHE"] = array
(
    "class"     => "SyncCtoContaoAutomator",
    "function"  => "purgeInternalCache",
    "typ"       => "GET",
    "parameter" => false,
);

// Contao Cache.
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_CREATE_CACHE"] = array
(
    "class"     => "SyncCtoContaoAutomator",
    "function"  => "createInternalCache",
    "typ"       => "GET",
    "parameter" => false,
);

// Run maintenance
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_MAINTENANCE"] = array
(
    "class"     => "SyncCtoFiles",
    "function"  => "runMaintenance",
    "typ"       => "POST",
    "parameter" => array("options"),
);

// Execute last step operations
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_EXECUTE_FINAL_OPERATIONS"] = array
(
    "class"     => "SyncCtoHelper",
    "function"  => "executeFinalOperations",
    "typ"       => "GET",
    "parameter" => false,
);

// - Informations --------------------------------------------------------------

$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_GET_PHP_CONFIGURATION"] = array
(
    "class"     => "SyncCtoModuleCheck",
    "function"  => "getPhpConfigurations",
    "typ"       => "get",
    "parameter" => null,
);

$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_GET_PHP_FUNCTIONS"] = array
(
    "class"     => "SyncCtoModuleCheck",
    "function"  => "getPhpFunctions",
    "typ"       => "get",
    "parameter" => null,
);

$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_GET_EXTENDED_INFORMATIONS"] = array
(
    "class"     => "SyncCtoModuleCheck",
    "function"  => "getExtendedInformation",
    "typ"       => "POST",
    "parameter" => array("DateFormate"),
);

$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_GET_PRO_FUNCTIONS"] = array
(
    "class"     => "SyncCtoModuleCheck",
    "function"  => "getMySqlFunctions",
    "typ"       => "get",
    "parameter" => null,
);

// Get folder path list
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_GET_PATHLIST"] = array
(
    "class"     => "SyncCtoRPCFunctions",
    "function"  => "getPathList",
    "typ"       => "POST",
    "parameter" => array("name"),
);

// Get SyncCto Version
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_VERSION"] = array
(
    "class"     => "SyncCtoRPCFunctions",
    "function"  => "getVersionSyncCto",
    "typ"       => "GET",
    "parameter" => false,
);

// Get a list of parameter
$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_PARAMETER"] = array
(
    "class"     => "SyncCtoRPCFunctions",
    "function"  => "getClientParameter",
    "typ"       => "GET",
    "parameter" => false,
);

$GLOBALS["CTOCOM_FUNCTIONS"]["SYNCCTO_AUTO_UPDATE"] = array
(
    "class"     => "SyncCtoAutoUpdater",
    "function"  => "update",
    "typ"       => "POST",
    "parameter" => array("zipfile"),
);
}

namespace  {
/**
 * notification_center extension for Contao Open Source CMS
 *
 * @copyright  Copyright (c) 2008-2015, terminal42
 * @author     terminal42 gmbh <info@terminal42.ch>
 * @license    LGPL
 */

/**
 * Back end modules
 */
array_insert($GLOBALS['BE_MOD'], 1, array
(
    'notification_center' => array
    (
        'nc_notifications' => array
        (
            'tables'        => array('tl_nc_notification', 'tl_nc_message', 'tl_nc_language'),
        ),
        'nc_queue' => array
        (
            'tables'        => array('tl_nc_queue'),
            're-queue'      => array('NotificationCenter\tl_nc_queue', 'reQueue')
        ),
        'nc_gateways' => array
        (
            'tables'        => array('tl_nc_gateway'),
        )
    )
));

// Load icon in Contao 4.2 backend
if ('BE' === TL_MODE) {
    $GLOBALS['TL_CSS'][] = 'system/modules/notification_center/assets/backend.css';
}

/**
 * Front end modules
 */
$GLOBALS['FE_MOD']['user']['lostPasswordNotificationCenter'] = 'ModulePasswordNotificationCenter';

if (in_array('newsletter', \ModuleLoader::getActive(), true)) {
    $GLOBALS['FE_MOD']['newsletter']['newsletterSubscribeNotificationCenter']   = 'ModuleNewsletterSubscribeNotificationCenter';
    $GLOBALS['FE_MOD']['newsletter']['newsletterActivateNotificationCenter']    = 'ModuleNewsletterActivateNotificationCenter';
    $GLOBALS['FE_MOD']['newsletter']['newsletterUnsubscribeNotificationCenter'] = 'ModuleNewsletterUnsubscribeNotificationCenter';
}

/**
 * Models
 */
$GLOBALS['TL_MODELS']['tl_nc_notification']             = 'NotificationCenter\Model\Notification';
$GLOBALS['TL_MODELS']['tl_nc_gateway']                  = 'NotificationCenter\Model\Gateway';
$GLOBALS['TL_MODELS']['tl_nc_language']                 = 'NotificationCenter\Model\Language';
$GLOBALS['TL_MODELS']['tl_nc_message']                  = 'NotificationCenter\Model\Message';
$GLOBALS['TL_MODELS']['tl_nc_queue']                    = 'NotificationCenter\Model\QueuedMessage';

/**
 * Cron jobs
 */
$GLOBALS['TL_CRON']['minutely'][] = array('NotificationCenter\Frontend\PoorMansCron', 'minutely');
$GLOBALS['TL_CRON']['hourly'][]   = array('NotificationCenter\Frontend\PoorMansCron', 'hourly');
$GLOBALS['TL_CRON']['daily'][]    = array('NotificationCenter\Frontend\PoorMansCron', 'daily');
$GLOBALS['TL_CRON']['weekly'][]   = array('NotificationCenter\Frontend\PoorMansCron', 'weekly');
$GLOBALS['TL_CRON']['monthly'][]  = array('NotificationCenter\Frontend\PoorMansCron', 'monthly');

/**
 * Hooks
 */
$GLOBALS['TL_HOOKS']['addCustomRegexp'][]       = array('NotificationCenter\AutoSuggester', 'verifyTokens');
$GLOBALS['TL_HOOKS']['processFormData'][]       = array('NotificationCenter\tl_form', 'sendFormNotification');
$GLOBALS['TL_HOOKS']['createNewUser'][]         = array('NotificationCenter\ContaoHelper', 'sendRegistrationEmail');
$GLOBALS['TL_HOOKS']['updatePersonalData'][]    = array('NotificationCenter\ContaoHelper', 'sendPersonalDataEmail');
$GLOBALS['TL_HOOKS']['getUserNavigation'][]     = array('NotificationCenter\ContaoHelper', 'addQueueToUserNavigation');
$GLOBALS['TL_HOOKS']['activateAccount'][]       = array('NotificationCenter\ContaoHelper', 'sendActivationEmail');
$GLOBALS['TL_HOOKS']['getSystemMessages'][]     = array('NotificationCenter\ContaoHelper', 'alertLegacySmtpSetting');

/**
 * Queue manager
 */
$GLOBALS['NOTIFICATION_CENTER']['QUEUE_MANAGER'] = new \NotificationCenter\Queue\QueueManager();

/**
 * Notification Center Gateways
 */
$GLOBALS['NOTIFICATION_CENTER']['GATEWAY'] = array_merge(
    (array) ($GLOBALS['NOTIFICATION_CENTER']['GATEWAY'] ?? []),
    array(
         'queue'    => 'NotificationCenter\Gateway\Queue',
         'email'    => 'NotificationCenter\Gateway\Email',
         'file'     => 'NotificationCenter\Gateway\File',
         'postmark' => 'NotificationCenter\Gateway\Postmark',
    )
);

/**
 * Notification Center Notification Types
 */
$GLOBALS['NOTIFICATION_CENTER']['NOTIFICATION_TYPE'] = array_merge_recursive(
    (array) ($GLOBALS['NOTIFICATION_CENTER']['NOTIFICATION_TYPE'] ?? []),
    array(
         'contao' => array(
             'core_form' => array(
                 'recipients'           => array('admin_email', 'form_*', 'formconfig_*'),
                 'email_subject'        => array('form_*', 'formconfig_*', 'admin_email'),
                 'email_text'           => array('form_*', 'formconfig_*', 'formlabel_*', 'raw_data', 'raw_data_filled', 'admin_email'),
                 'email_html'           => array('form_*', 'formconfig_*', 'formlabel_*', 'raw_data', 'raw_data_filled', 'admin_email'),
                 'file_name'            => array('form_*', 'formconfig_*', 'admin_email'),
                 'file_content'         => array('form_*', 'formconfig_*', 'formlabel_*', 'raw_data', 'raw_data_filled', 'admin_email'),
                 'email_sender_name'    => array('admin_email', 'form_*', 'formconfig_*'),
                 'email_sender_address' => array('admin_email', 'form_*', 'formconfig_*'),
                 'email_recipient_cc'   => array('admin_email', 'form_*', 'formconfig_*'),
                 'email_recipient_bcc'  => array('admin_email', 'form_*', 'formconfig_*'),
                 'email_replyTo'        => array('admin_email', 'form_*', 'formconfig_*'),
                 'attachment_tokens'    => array('form_*', 'formconfig_*'),
             ),
             'member_activation' => array(
                 'recipients'           => array('member_email', 'admin_email'),
                 'email_subject'        => array('domain', 'member_*', 'admin_email'),
                 'email_text'           => array('domain', 'member_*', 'admin_email'),
                 'email_html'           => array('domain', 'member_*', 'admin_email'),
                 'file_name'            => array('domain', 'member_*', 'admin_email'),
                 'file_content'         => array('domain', 'member_*', 'admin_email'),
                 'email_sender_name'    => array('admin_email', 'form_*'),
                 'email_sender_address' => array('admin_email', 'form_*'),
                 'email_recipient_cc'   => array('admin_email', 'member_*'),
                 'email_recipient_bcc'  => array('admin_email', 'member_*'),
                 'email_replyTo'        => array('admin_email', 'member_*'),
             ),
             'member_registration' => array(
                 'recipients'           => array('member_email', 'admin_email'),
                 'email_subject'        => array('domain', 'link', 'member_*', 'admin_email'),
                 'email_text'           => array('domain', 'link', 'member_*', 'admin_email'),
                 'email_html'           => array('domain', 'link', 'member_*', 'admin_email'),
                 'file_name'            => array('domain', 'link', 'member_*', 'admin_email'),
                 'file_content'         => array('domain', 'link', 'member_*', 'admin_email'),
                 'email_sender_name'    => array('admin_email', 'member_*'),
                 'email_sender_address' => array('admin_email', 'member_*'),
                 'email_recipient_cc'   => array('admin_email', 'member_*'),
                 'email_recipient_bcc'  => array('admin_email', 'member_*'),
                 'email_replyTo'        => array('admin_email', 'member_*'),
             ),
             'member_personaldata' => array(
                 'recipients'           => array('member_email', 'admin_email'),
                 'email_subject'        => array('domain', 'member_*', 'member_old_*', 'changed_*', 'admin_email'),
                 'email_text'           => array('domain', 'member_*', 'member_old_*', 'changed_*', 'admin_email'),
                 'email_html'           => array('domain', 'member_*', 'member_old_*', 'changed_*', 'admin_email'),
                 'email_sender_name'    => array('member_*'),
                 'email_sender_address' => array('member_email', 'admin_email'),
                 'email_recipient_cc'   => array('member_email', 'admin_email'),
                 'email_recipient_bcc'  => array('member_email', 'admin_email'),
                 'email_replyTo'        => array('member_email', 'admin_email'),
             ),
             'member_password'     => array(
                 'recipients'           => array('recipient_email'),
                 'email_subject'        => array('domain', 'link', 'member_*', 'recipient_email'),
                 'email_text'           => array('domain', 'link', 'member_*', 'recipient_email'),
                 'email_html'           => array('domain', 'link', 'member_*', 'recipient_email'),
                 'file_name'            => array('domain', 'link', 'member_*', 'recipient_email'),
                 'file_content'         => array('domain', 'link', 'member_*', 'recipient_email'),
                 'email_sender_name'    => array('recipient_email'),
                 'email_sender_address' => array('recipient_email'),
                 'email_recipient_cc'   => array('recipient_email'),
                 'email_recipient_bcc'  => array('recipient_email'),
                 'email_replyTo'        => array('recipient_email'),
             ),
         )
    )
);

// Add the newsletter tokens only if the extension is active
if (in_array('newsletter', \Contao\ModuleLoader::getActive(), true)) {
    $GLOBALS['NOTIFICATION_CENTER']['NOTIFICATION_TYPE']['contao']['newsletter_subscribe'] = array(
        'recipients'           => array('recipient_email', 'admin_email'),
        'email_subject'        => array('domain', 'link', 'recipient_email', 'admin_email', 'channels', 'channel_ids', 'subject'),
        'email_text'           => array('domain', 'link', 'recipient_email', 'admin_email', 'channels', 'channel_ids', 'token'),
        'email_html'           => array('domain', 'link', 'recipient_email', 'admin_email', 'channels', 'channel_ids', 'token'),
        'file_name'            => array('domain', 'link', 'recipient_email', 'admin_email', 'channels', 'channel_ids'),
        'file_content'         => array('domain', 'link', 'recipient_email', 'admin_email', 'channels', 'channel_ids', 'token'),
        'email_sender_name'    => array('recipient_email', 'admin_email', 'admin_name'),
        'email_sender_address' => array('recipient_email', 'admin_email'),
        'email_recipient_cc'   => array('recipient_email', 'admin_email'),
        'email_recipient_bcc'  => array('recipient_email', 'admin_email'),
        'email_replyTo'        => array('recipient_email', 'admin_email'),
    );

    $GLOBALS['NOTIFICATION_CENTER']['NOTIFICATION_TYPE']['contao']['newsletter_activate'] = array(
        'recipients'           => array('recipient_email', 'admin_email'),
        'email_subject'        => array('domain', 'recipient_email', 'admin_email', 'channels', 'channel_ids', 'subject'),
        'email_text'           => array('domain', 'recipient_email', 'admin_email', 'channels', 'channel_ids'),
        'email_html'           => array('domain', 'recipient_email', 'admin_email', 'channels', 'channel_ids'),
        'file_name'            => array('domain', 'recipient_email', 'admin_email', 'channels', 'channel_ids'),
        'file_content'         => array('domain', 'recipient_email', 'admin_email', 'channels', 'channel_ids'),
        'email_sender_name'    => array('recipient_email', 'admin_email', 'admin_name'),
        'email_sender_address' => array('recipient_email', 'admin_email'),
        'email_recipient_cc'   => array('recipient_email', 'admin_email'),
        'email_recipient_bcc'  => array('recipient_email', 'admin_email'),
        'email_replyTo'        => array('recipient_email', 'admin_email'),
    );

    $GLOBALS['NOTIFICATION_CENTER']['NOTIFICATION_TYPE']['contao']['newsletter_unsubscribe'] = array(
        'recipients'           => array('recipient_email', 'admin_email'),
        'email_subject'        => array('domain', 'recipient_email', 'admin_email', 'channels', 'channel_ids', 'subject'),
        'email_text'           => array('domain', 'recipient_email', 'admin_email', 'channels', 'channel_ids'),
        'email_html'           => array('domain', 'recipient_email', 'admin_email', 'channels', 'channel_ids'),
        'file_name'            => array('domain', 'recipient_email', 'admin_email', 'channels', 'channel_ids'),
        'file_content'         => array('domain', 'recipient_email', 'admin_email', 'channels', 'channel_ids'),
        'email_sender_name'    => array('recipient_email', 'admin_email', 'admin_name'),
        'email_sender_address' => array('recipient_email', 'admin_email'),
        'email_recipient_cc'   => array('recipient_email', 'admin_email'),
        'email_recipient_bcc'  => array('recipient_email', 'admin_email'),
        'email_replyTo'        => array('recipient_email', 'admin_email'),
    );
}
}

namespace  {
$GLOBALS['BE_MOD']['catalog-manager-extensions']['catalog-manager-importer'] = [

    'name' => 'catalog-manager-importer',
    'icon' => 'system/modules/catalog-manager-importer/assets/icon.svg',

    'tables' => [

        'tl_catalog_imports'
    ]
];

$GLOBALS['BE_FFL']['keyValueMapping'] = 'CMImporter\KeyValueMappingWizard';

if ( TL_MODE == 'BE' ) {

    $GLOBALS['TL_CSS']['catalogManagerImporterKeyValueMappingWizard'] = $GLOBALS['TL_CONFIG']['debugMode']
        ? 'system/modules/catalog-manager-importer/assets/key_value_mapping_wizard.css'
        : 'system/modules/catalog-manager-importer/assets/key_value_mapping_wizard.css';
}

$GLOBALS['CTLG_IMPORT_GLOBALS'] = [];

$GLOBALS['CTLG_IMPORT_GLOBALS']['DATA_TYPES'] = [

    'TEXT_UTF8',
    'TEXT',
    'DATE',
    'FILE',
    'INT'
];

$GLOBALS['CTLG_IMPORT_GLOBALS']['STATES'] = [

    '200',
    '500'
];
}

namespace  {
$GLOBALS['TL_HOOKS']['catalogManagerViewQuery'][] = ['CMWishlist\WishlistModule', 'setQuery'];
$GLOBALS['TL_HOOKS']['catalogManagerInitializeView'][] = ['CMWishlist\WishlistModule', 'initialize'];
$GLOBALS['TL_HOOKS']['catalogManagerRenderCatalog'][] = ['CMWishlist\WishlistModule', 'renderCatalog'];
$GLOBALS['TL_HOOKS']['replaceInsertTags'][] = ['CMWishlist\WishlistInserttag', 'getInsertTagValue'];
$GLOBALS['TL_HOOKS']['replaceInsertTags'][] = [ 'CMWishlist\NewInsertTags', 'ReplaceMyInsertTag' ];
}

namespace  {
/*
 * Copyright MADE/YOUR/DAY OG <mail@madeyourday.net>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/**
 * RockSolid Slider back end modules configuration
 *
 * @author Martin Auswöger <martin@madeyourday.net>
 */

$GLOBALS['TL_MODELS']['tl_rocksolid_slider'] = 'MadeYourDay\\RockSolidSlider\\Model\\SliderModel';
$GLOBALS['TL_MODELS']['tl_rocksolid_slide'] = 'MadeYourDay\\RockSolidSlider\\Model\\SlideModel';

$GLOBALS['TL_CTE']['includes']['rocksolid_slider'] = 'MadeYourDay\\RockSolidSlider\\Module\\Slider';

$GLOBALS['BE_MOD']['content']['rocksolid_slider'] = array(
	'tables' => array(
		'tl_rocksolid_slider',
		'tl_rocksolid_slide',
		'tl_content',
		'tl_rocksolid_slider_license',
	),
	'icon' => 'bundles/rocksolidslider/img/icon.png',
);

array_insert($GLOBALS['FE_MOD'], 2, array(
	'miscellaneous' => array(
		'rocksolid_slider' => 'MadeYourDay\\RockSolidSlider\\Module\\Slider',
	)
));

$GLOBALS['TL_PERMISSIONS'][] = 'rsts_sliders';
$GLOBALS['TL_PERMISSIONS'][] = 'rsts_permissions';
}

namespace  {
$GLOBALS['TL_HOOKS']['catalogManagerModifyBackendModule'][] = [ 'CatalogManager\ExportBundle\Library\Hooks', 'modifyBackendModule' ];
$GLOBALS['TL_HOOKS']['loadDataContainer'][] = [ 'export.library.hooks', 'setExport' ];
}

namespace  {
/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2012 Leo Feyer
 * 
 * @package   Contao 
 * @author    Kester Mielke 
 * @license   LGPL 
 * @copyright Kester Mielke 2010-2013 
 */

$GLOBALS['TL_CONFIG']['tl_calendar_events']['maxRepeatExceptions'] = 365;

/**
 * the range of days for the move date option
 * 14 means from -14 days to 14 days
 */
$GLOBALS['TL_CONFIG']['tl_calendar_events']['moveDays'] = 7;

/**
 * the start, end and interval of times for the move time option
 * 00:00|23:59|30 means start at 00:00 and add 30 min. to the time
 *
 * this will be used for start and end time
 *
 * examples
 * interval 15: 00:15, 00:30, 00:45, 01:00...
 * interval 30: 00:00, 00:30, 01:00, 01:30...
 */
$GLOBALS['TL_CONFIG']['tl_calendar_events']['moveTimes'] = '10:00|22:00|30';

$GLOBALS['TL_LANG']['DAYS']['sunday']    = 0;
$GLOBALS['TL_LANG']['DAYS']['monday']    = 1;
$GLOBALS['TL_LANG']['DAYS']['tuesday']   = 2;
$GLOBALS['TL_LANG']['DAYS']['wednesday'] = 3;
$GLOBALS['TL_LANG']['DAYS']['thursday']  = 4;
$GLOBALS['TL_LANG']['DAYS']['friday']    = 5;
$GLOBALS['TL_LANG']['DAYS']['saturday']  = 6;

$GLOBALS['TL_CONFIG']['tl_calendar_events']['weekdays'][0] = 'sunday';
$GLOBALS['TL_CONFIG']['tl_calendar_events']['weekdays'][1] = 'monday';
$GLOBALS['TL_CONFIG']['tl_calendar_events']['weekdays'][2] = 'tuesday';
$GLOBALS['TL_CONFIG']['tl_calendar_events']['weekdays'][3] = 'wednesday';
$GLOBALS['TL_CONFIG']['tl_calendar_events']['weekdays'][4] = 'thursday';
$GLOBALS['TL_CONFIG']['tl_calendar_events']['weekdays'][5] = 'friday';
$GLOBALS['TL_CONFIG']['tl_calendar_events']['weekdays'][6] = 'saturday';

// Event Filter
$GLOBALS['TL_CONFIG']['tl_calendar_events']['filter']['title'] = [];
$GLOBALS['TL_CONFIG']['tl_calendar_events']['filter']['location_name'] = [];
$GLOBALS['TL_CONFIG']['tl_calendar_events']['filter']['location_str'] = [];
$GLOBALS['TL_CONFIG']['tl_calendar_events']['filter']['location_plz'] = [];

/**
 * Front end modules
 */
array_insert($GLOBALS['FE_MOD'], 99, array
(
    'events' => array
    (
        'timetable'	        => 'Kmielke\CalendarExtendedBundle\ModuleTimeTable',
        'yearview'	        => 'Kmielke\CalendarExtendedBundle\ModuleYearView',
        'evr_registration'  => 'Kmielke\CalendarExtendedBundle\ModuleEventRegistration',
        'fullcalendar'      => 'Kmielke\CalendarExtendedBundle\ModuleFullcalendar'
    )
));

// Replace Contao Module
$GLOBALS['FE_MOD']['events']['calendar']    = 'Kmielke\CalendarExtendedBundle\ModuleCalendar';
$GLOBALS['FE_MOD']['events']['eventlist']   = 'Kmielke\CalendarExtendedBundle\ModuleEventlist';
$GLOBALS['FE_MOD']['events']['eventmenu']   = 'Kmielke\CalendarExtendedBundle\ModuleEventMenu';
$GLOBALS['FE_MOD']['events']['eventreader'] = 'Kmielke\CalendarExtendedBundle\ModuleEventReader';

/**
 * BACK END FORM FIELDS
 */

array_insert($GLOBALS['BE_FFL'], 99, array
(
    'timePeriodExt'     => 'Kmielke\CalendarExtendedBundle\TimePeriodExt',
));

// config.php
$GLOBALS['NOTIFICATION_CENTER']['NOTIFICATION_TYPE']['event_registration'] = array
(
    // Type
    'registration_status' => array(
        'recipients'           => array('recipient_email', 'admin_email'),
        'email_subject'        => array('page_title', 'recipient_*'),
        'email_text'           => array('recipient_*', 'raw_data', ),
        'email_html'           => array('recipient_*', 'raw_data', ),
        'email_sender_name'    => array('admin_email', 'page_title'),
        'email_sender_address' => array('admin_email'),
        'email_recipient_cc'   => array('recipient_email'),
        'email_recipient_bcc'  => array('recipient_email'),
        'email_replyTo'        => array('recipient_email'),
        'file_name'            => array('recipient_email'),
        'file_content'         => array('recipient_email')
    )
);

/**
 * Event Hook
 */
$GLOBALS['TL_HOOKS']['getAllEvents'][] = array('Kmielke\CalendarExtendedBundle\EventUrls', 'modifyEventUrl');
}

namespace  {
$GLOBALS['TL_CRON']['minutely'][] = ['CatalogManager\Mailer\Cronjob', 'mailer'];
$GLOBALS['TL_CRON']['minutely'][] = ['CatalogManager\Mailer\Cronjob', 'reminder'];

$GLOBALS['BE_MOD']['catalog-manager-extensions']['cm-mailer'] = [
    'name' => 'catalog-manager-mailer',
    'icon' => 'system/modules/catalog-manager-mailer/assets/icon.svg',
    'tables' => [
        'tl_mailer',
        'tl_mailer_queue'
    ]
];

$GLOBALS['BE_MOD']['catalog-manager-extensions']['cm-reminder'] = [
    'name' => 'catalog-manager-reminder',
    'icon' => 'system/modules/catalog-manager-mailer/assets/icon.svg',
    'tables' => [
        'tl_reminder'
    ]
];

$GLOBALS['TL_HOOKS']['catalogManagerEntityOnCreate'][] = [ 'CatalogManager\Mailer\FrontendEditingTrigger', 'onCreate' ];
$GLOBALS['TL_HOOKS']['catalogManagerEntityOnUpdate'][] = [ 'CatalogManager\Mailer\FrontendEditingTrigger', 'onUpdate' ];
$GLOBALS['TL_HOOKS']['catalogManagerEntityOnDelete'][] = [ 'CatalogManager\Mailer\FrontendEditingTrigger', 'onDelete' ];

$GLOBALS['NOTIFICATION_CENTER']['NOTIFICATION_TYPE']['CATALOG_MAILER'] = [

    'DEFAULT_MAILER' => [

        'recipients' => [ 'admin_email', 'recipient', 'raw_*', 'clean_*', 'post_raw_*', 'post_clean_*', 'post_old_raw_*', 'post_old_clean_*', 'post_type' ],
        'attachment_tokens' => [ 'raw_*', 'clean_*', 'field_*', 'table_*', 'post_raw_*', 'post_clean_*', 'post_old_raw_*', 'post_old_clean_*', 'post_type' ],
        'email_replyTo' => [ 'admin_email','recipient', 'raw_*', 'clean_*', 'post_raw_*', 'post_clean_*', 'post_old_raw_*', 'post_old_clean_*', 'post_type' ],
        'email_sender_name' => [ 'admin_email', 'recipient', 'raw_*', 'clean_*', 'post_raw_*', 'post_clean_*', 'post_old_raw_*', 'post_old_clean_*', 'post_type' ],
        'email_recipient_cc' => [ 'admin_email', 'recipient', 'raw_*', 'clean_*', 'post_raw_*', 'post_clean_*', 'post_old_raw_*', 'post_old_clean_*', 'post_type' ],
        'file_name' => [ 'admin_email', 'raw_*', 'clean_*', 'field_*', 'table_*', 'post_raw_*', 'post_clean_*', 'post_old_raw_*', 'post_old_clean_*', 'post_type' ],
        'email_recipient_bcc' => [ 'admin_email', 'recipient', 'raw_*', 'clean_*', 'post_raw_*', 'post_clean_*', 'post_old_raw_*', 'post_old_clean_*', 'post_type' ],
        'email_sender_address' => [ 'admin_email', 'recipient', 'raw_*', 'clean_*', 'post_raw_*', 'post_clean_*', 'post_old_raw_*', 'post_old_clean_*', 'post_type' ],
        'file_content' => [ 'admin_email', 'raw_*', 'clean_*', 'field_*', 'table_*', 'post_raw_*', 'post_clean_*', 'post_old_raw_*', 'post_old_clean_*', 'post_type', 'reminder_attachment' ],
        'email_subject' => [ 'admin_email', 'domain', 'recipient', 'raw_*', 'clean_*', 'post_raw_*', 'post_clean_*', 'post_old_raw_*', 'post_old_clean_*', 'post_type' ],
        'email_text' => [ 'admin_email', 'recipient', 'raw_*', 'clean_*', 'field_*', 'table_*', 'post_raw_*', 'post_clean_*', 'post_old_raw_*', 'post_old_clean_*', 'post_type', 'reminder_attachment' ],
        'email_html' => [ 'admin_email', 'recipient', 'raw_*', 'clean_*', 'field_*', 'table_*', 'post_raw_*', 'post_clean_*', 'post_old_raw_*', 'post_old_clean_*', 'post_type', 'reminder_attachment' ]
    ]
];
}

namespace  {
/**
 * news_newsletter extension for Contao Open Source CMS
 *
 * @copyright  Copyright (c) 2008-2014, terminal42 gmbh
 * @author     terminal42 gmbh <info@terminal42.ch>
 * @license    http://opensource.org/licenses/lgpl-3.0.html LGPL
 * @link       http://github.com/terminal42/contao-news_newsletter
 */

/**
 * Add new notification type
 */
$GLOBALS['NOTIFICATION_CENTER']['NOTIFICATION_TYPE']['news_newsletter']['news_newsletter_default'] = array
(
     'recipients'           => array('admin_email', 'recipient_email'),
     'email_subject'        => array('admin_email', 'news_archive_*', 'news_*', 'news_text', 'news_url', 'recipient_email'),
     'email_text'           => array('admin_email', 'news_archive_*', 'news_*', 'news_text', 'news_url', 'recipient_email'),
     'email_html'           => array('admin_email', 'news_archive_*', 'news_*', 'news_text', 'news_url', 'recipient_email'),
     'file_name'            => array('admin_email', 'news_archive_*', 'news_*', 'news_text', 'news_url', 'recipient_email'),
     'file_content'         => array('admin_email', 'news_archive_*', 'news_*', 'news_text', 'news_url', 'recipient_email'),
     'email_recipient_cc'   => array('admin_email', 'news_archive_*', 'news_*', 'news_text', 'news_url', 'recipient_email'),
     'email_recipient_bcc'  => array('admin_email', 'news_archive_*', 'news_*', 'news_text', 'news_url', 'recipient_email'),
     'email_replyTo'        => array('admin_email', 'news_archive_*', 'news_*', 'news_text', 'news_url', 'recipient_email'),
     'attachment_tokens'    => array(),
);
}
