<?php

namespace  {
/*
 * This file is part of Contao.
 *
 * (c) Leo Feyer
 *
 * @license LGPL-3.0-or-later
 */

use Contao\Automator;
use Contao\Backend;
use Contao\BackendUser;
use Contao\Calendar;
use Contao\CalendarEventsModel;
use Contao\CalendarModel;
use Contao\Config;
use Contao\CoreBundle\Exception\AccessDeniedException;
use Contao\DataContainer;
use Contao\Date;
use Contao\Events;
use Contao\Image;
use Contao\Input;
use Contao\LayoutModel;
use Contao\PageModel;
use Contao\StringUtil;
use Contao\System;
use Contao\Versions;
use Symfony\Component\HttpFoundation\Session\SessionInterface;

System::loadLanguageFile('tl_content');

$GLOBALS['TL_DCA']['tl_calendar_events'] = array
(
	// Config
	'config' => array
	(
		'dataContainer'               => 'Table',
		'ptable'                      => 'tl_calendar',
		'ctable'                      => array('tl_content'),
		'switchToEdit'                => true,
		'enableVersioning'            => true,
		'markAsCopy'                  => 'title',
		'onload_callback' => array
		(
			array('tl_calendar_events', 'checkPermission'),
			array('tl_calendar_events', 'generateFeed')
		),
		'oncut_callback' => array
		(
			array('tl_calendar_events', 'scheduleUpdate')
		),
		'ondelete_callback' => array
		(
			array('tl_calendar_events', 'scheduleUpdate')
		),
		'onsubmit_callback' => array
		(
			array('tl_calendar_events', 'adjustTime'),
			array('tl_calendar_events', 'scheduleUpdate')
		),
		'sql' => array
		(
			'keys' => array
			(
				'id' => 'primary',
				'alias' => 'index',
				'pid,published,featured,start,stop' => 'index'
			)
		)
	),

	// List
	'list' => array
	(
		'sorting' => array
		(
			'mode'                    => 4,
			'fields'                  => array('startTime'),
			'headerFields'            => array('title', 'jumpTo', 'tstamp', 'protected', 'allowComments'),
			'panelLayout'             => 'filter;sort,search,limit',
			'child_record_callback'   => array('tl_calendar_events', 'listEvents')
		),
		'global_operations' => array
		(
			'all' => array
			(
				'href'                => 'act=select',
				'class'               => 'header_edit_all',
				'attributes'          => 'onclick="Backend.getScrollOffset()" accesskey="e"'
			)
		),
		'operations' => array
		(
			'edit' => array
			(
				'href'                => 'table=tl_content',
				'icon'                => 'edit.svg'
			),
			'editheader' => array
			(
				'href'                => 'act=edit',
				'icon'                => 'header.svg'
			),
			'copy' => array
			(
				'href'                => 'act=paste&amp;mode=copy',
				'icon'                => 'copy.svg'
			),
			'cut' => array
			(
				'href'                => 'act=paste&amp;mode=cut',
				'icon'                => 'cut.svg'
			),
			'delete' => array
			(
				'href'                => 'act=delete',
				'icon'                => 'delete.svg',
				'attributes'          => 'onclick="if(!confirm(\'' . $GLOBALS['TL_LANG']['MSC']['deleteConfirm'] . '\'))return false;Backend.getScrollOffset()"'
			),
			'toggle' => array
			(
				'icon'                => 'visible.svg',
				'attributes'          => 'onclick="Backend.getScrollOffset();return AjaxRequest.toggleVisibility(this,%s)"',
				'button_callback'     => array('tl_calendar_events', 'toggleIcon'),
				'showInHeader'        => true
			),
			'feature' => array
			(
				'icon'                => 'featured.svg',
				'attributes'          => 'onclick="Backend.getScrollOffset();return AjaxRequest.toggleFeatured(this,%s)"',
				'button_callback'     => array('tl_calendar_events', 'iconFeatured')
			),
			'show' => array
			(
				'href'                => 'act=show',
				'icon'                => 'show.svg'
			)
		)
	),

	// Palettes
	'palettes' => array
	(
		'__selector__'                => array('source', 'addTime', 'addImage', 'recurring', 'addEnclosure', 'overwriteMeta'),
		'default'                     => '{title_legend},title,alias,author;{date_legend},addTime,startDate,endDate;{source_legend:hide},source;{meta_legend},pageTitle,robots,description,serpPreview;{details_legend},location,address,teaser;{image_legend},addImage;{recurring_legend},recurring;{enclosure_legend:hide},addEnclosure;{expert_legend:hide},cssClass,noComments,featured;{publish_legend},published,start,stop',
		'internal'                    => '{title_legend},title,alias,author;{date_legend},addTime,startDate,endDate;{source_legend},source,jumpTo;{details_legend},location,address,teaser;{image_legend},addImage;{recurring_legend},recurring;{enclosure_legend:hide},addEnclosure;{expert_legend:hide},cssClass,noComments,featured;{publish_legend},published,start,stop',
		'article'                     => '{title_legend},title,alias,author;{date_legend},addTime,startDate,endDate;{source_legend},source,articleId;{details_legend},location,address,teaser;{image_legend},addImage;{recurring_legend},recurring;{enclosure_legend:hide},addEnclosure;{expert_legend:hide},cssClass,noComments,featured;{publish_legend},published,start,stop',
		'external'                    => '{title_legend},title,alias,author;{date_legend},addTime,startDate,endDate;{source_legend},source,url,target;{details_legend},location,address,teaser;{image_legend},addImage;{recurring_legend},recurring;{enclosure_legend:hide},addEnclosure;{expert_legend:hide},cssClass,noComments,featured;{publish_legend},published,start,stop'
	),

	// Subpalettes
	'subpalettes' => array
	(
		'addTime'                     => 'startTime,endTime',
		'addImage'                    => 'singleSRC,size,floating,imagemargin,fullsize,overwriteMeta',
		'recurring'                   => 'repeatEach,recurrences',
		'addEnclosure'                => 'enclosure',
		'overwriteMeta'               => 'alt,imageTitle,imageUrl,caption'
	),

	// Fields
	'fields' => array
	(
		'id' => array
		(
			'sql'                     => "int(10) unsigned NOT NULL auto_increment"
		),
		'pid' => array
		(
			'foreignKey'              => 'tl_calendar.title',
			'sql'                     => "int(10) unsigned NOT NULL default 0",
			'relation'                => array('type'=>'belongsTo', 'load'=>'lazy')
		),
		'tstamp' => array
		(
			'sql'                     => "int(10) unsigned NOT NULL default 0"
		),
		'title' => array
		(
			'exclude'                 => true,
			'search'                  => true,
			'sorting'                 => true,
			'flag'                    => 1,
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'maxlength'=>255, 'tl_class'=>'w50'),
			'sql'                     => "varchar(255) NOT NULL default ''"
		),
		'alias' => array
		(
			'exclude'                 => true,
			'search'                  => true,
			'inputType'               => 'text',
			'eval'                    => array('rgxp'=>'alias', 'doNotCopy'=>true, 'unique'=>true, 'maxlength'=>255, 'tl_class'=>'w50 clr'),
			'save_callback' => array
			(
				array('tl_calendar_events', 'generateAlias')
			),
			'sql'                     => "varchar(255) BINARY NOT NULL default ''"
		),
		'author' => array
		(
			'default'                 => BackendUser::getInstance()->id,
			'exclude'                 => true,
			'search'                  => true,
			'filter'                  => true,
			'sorting'                 => true,
			'flag'                    => 11,
			'inputType'               => 'select',
			'foreignKey'              => 'tl_user.name',
			'eval'                    => array('doNotCopy'=>true, 'chosen'=>true, 'mandatory'=>true, 'includeBlankOption'=>true, 'tl_class'=>'w50'),
			'sql'                     => "int(10) unsigned NOT NULL default 0",
			'relation'                => array('type'=>'hasOne', 'load'=>'lazy')
		),
		'addTime' => array
		(
			'exclude'                 => true,
			'inputType'               => 'checkbox',
			'eval'                    => array('submitOnChange'=>true, 'doNotCopy'=>true),
			'sql'                     => "char(1) NOT NULL default ''"
		),
		'startTime' => array
		(
			'default'                 => time(),
			'exclude'                 => true,
			'filter'                  => true,
			'sorting'                 => true,
			'flag'                    => 8,
			'inputType'               => 'text',
			'eval'                    => array('rgxp'=>'time', 'mandatory'=>true, 'doNotCopy'=>true, 'tl_class'=>'w50'),
			'load_callback' => array
			(
				array('tl_calendar_events', 'loadTime')
			),
			'sql'                     => "int(10) NULL"
		),
		'endTime' => array
		(
			'default'                 => time(),
			'exclude'                 => true,
			'inputType'               => 'text',
			'eval'                    => array('rgxp'=>'time', 'doNotCopy'=>true, 'tl_class'=>'w50'),
			'load_callback' => array
			(
				array('tl_calendar_events', 'loadEndTime')
			),
			'save_callback' => array
			(
				array('tl_calendar_events', 'setEmptyEndTime')
			),
			'sql'                     => "int(10) NULL"
		),
		'startDate' => array
		(
			'exclude'                 => true,
			'inputType'               => 'text',
			'eval'                    => array('rgxp'=>'date', 'mandatory'=>true, 'doNotCopy'=>true, 'datepicker'=>true, 'tl_class'=>'w50 wizard'),
			'sql'                     => "int(10) unsigned NULL"
		),
		'endDate' => array
		(
			'exclude'                 => true,
			'inputType'               => 'text',
			'eval'                    => array('rgxp'=>'date', 'doNotCopy'=>true, 'datepicker'=>true, 'tl_class'=>'w50 wizard'),
			'sql'                     => "int(10) unsigned NULL"
		),
		'pageTitle' => array
		(
			'exclude'                 => true,
			'search'                  => true,
			'inputType'               => 'text',
			'eval'                    => array('maxlength'=>255, 'decodeEntities'=>true, 'tl_class'=>'w50'),
			'sql'                     => "varchar(255) NOT NULL default ''"
		),
		'robots' => array
		(
			'exclude'                 => true,
			'search'                  => true,
			'inputType'               => 'select',
			'options'                 => array('index,follow', 'index,nofollow', 'noindex,follow', 'noindex,nofollow'),
			'eval'                    => array('tl_class'=>'w50', 'includeBlankOption' => true),
			'sql'                     => "varchar(32) NOT NULL default ''"
		),
		'description' => array
		(
			'exclude'                 => true,
			'search'                  => true,
			'inputType'               => 'textarea',
			'eval'                    => array('style'=>'height:60px', 'decodeEntities'=>true, 'tl_class'=>'clr'),
			'sql'                     => "text NULL"
		),
		'serpPreview' => array
		(
			'label'                   => &$GLOBALS['TL_LANG']['MSC']['serpPreview'],
			'exclude'                 => true,
			'inputType'               => 'serpPreview',
			'eval'                    => array('url_callback'=>array('tl_calendar_events', 'getSerpUrl'), 'title_tag_callback'=>array('tl_calendar_events', 'getTitleTag'), 'titleFields'=>array('pageTitle', 'title'), 'descriptionFields'=>array('description', 'teaser')),
			'sql'                     => null
		),
		'location' => array
		(
			'exclude'                 => true,
			'search'                  => true,
			'inputType'               => 'text',
			'eval'                    => array('maxlength'=>255, 'tl_class'=>'w50'),
			'sql'                     => "varchar(255) NOT NULL default ''"
		),
		'address' => array
		(
			'exclude'                 => true,
			'search'                  => true,
			'inputType'               => 'text',
			'eval'                    => array('maxlength'=>255, 'tl_class'=>'w50'),
			'sql'                     => "varchar(255) NOT NULL default ''"
		),
		'teaser' => array
		(
			'exclude'                 => true,
			'search'                  => true,
			'inputType'               => 'textarea',
			'eval'                    => array('rte'=>'tinyMCE', 'tl_class'=>'clr'),
			'sql'                     => "text NULL"
		),
		'addImage' => array
		(
			'exclude'                 => true,
			'inputType'               => 'checkbox',
			'eval'                    => array('submitOnChange'=>true),
			'sql'                     => "char(1) NOT NULL default ''"
		),
		'overwriteMeta' => array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_content']['overwriteMeta'],
			'exclude'                 => true,
			'inputType'               => 'checkbox',
			'eval'                    => array('submitOnChange'=>true, 'tl_class'=>'w50 clr'),
			'sql'                     => "char(1) NOT NULL default ''"
		),
		'singleSRC' => array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_content']['singleSRC'],
			'exclude'                 => true,
			'inputType'               => 'fileTree',
			'eval'                    => array('filesOnly'=>true, 'fieldType'=>'radio', 'extensions'=>Config::get('validImageTypes'), 'mandatory'=>true),
			'sql'                     => "binary(16) NULL"
		),
		'alt' => array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_content']['alt'],
			'exclude'                 => true,
			'search'                  => true,
			'inputType'               => 'text',
			'eval'                    => array('maxlength'=>255, 'tl_class'=>'w50'),
			'sql'                     => "varchar(255) NOT NULL default ''"
		),
		'imageTitle' => array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_content']['imageTitle'],
			'exclude'                 => true,
			'search'                  => true,
			'inputType'               => 'text',
			'eval'                    => array('maxlength'=>255, 'tl_class'=>'w50'),
			'sql'                     => "varchar(255) NOT NULL default ''"
		),
		'size' => array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_content']['size'],
			'exclude'                 => true,
			'inputType'               => 'imageSize',
			'reference'               => &$GLOBALS['TL_LANG']['MSC'],
			'eval'                    => array('rgxp'=>'natural', 'includeBlankOption'=>true, 'nospace'=>true, 'helpwizard'=>true, 'tl_class'=>'w50'),
			'options_callback' => static function ()
			{
				return System::getContainer()->get('contao.image.image_sizes')->getOptionsForUser(BackendUser::getInstance());
			},
			'sql'                     => "varchar(64) NOT NULL default ''"
		),
		'imagemargin' => array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_content']['imagemargin'],
			'exclude'                 => true,
			'inputType'               => 'trbl',
			'options'                 => $GLOBALS['TL_CSS_UNITS'],
			'eval'                    => array('includeBlankOption'=>true, 'tl_class'=>'w50'),
			'sql'                     => "varchar(128) NOT NULL default ''"
		),
		'imageUrl' => array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_content']['imageUrl'],
			'exclude'                 => true,
			'search'                  => true,
			'inputType'               => 'text',
			'eval'                    => array('rgxp'=>'url', 'decodeEntities'=>true, 'maxlength'=>255, 'dcaPicker'=>true, 'tl_class'=>'w50 wizard'),
			'sql'                     => "varchar(255) NOT NULL default ''"
		),
		'fullsize' => array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_content']['fullsize'],
			'exclude'                 => true,
			'inputType'               => 'checkbox',
			'eval'                    => array('tl_class'=>'w50 m12'),
			'sql'                     => "char(1) NOT NULL default ''"
		),
		'caption' => array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_content']['caption'],
			'exclude'                 => true,
			'search'                  => true,
			'inputType'               => 'text',
			'eval'                    => array('maxlength'=>255, 'allowHtml'=>true, 'tl_class'=>'w50'),
			'sql'                     => "varchar(255) NOT NULL default ''"
		),
		'floating' => array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_content']['floating'],
			'exclude'                 => true,
			'inputType'               => 'radioTable',
			'options'                 => array('above', 'left', 'right', 'below'),
			'eval'                    => array('cols'=>4, 'tl_class'=>'w50'),
			'reference'               => &$GLOBALS['TL_LANG']['MSC'],
			'sql'                     => "varchar(32) NOT NULL default 'above'"
		),
		'recurring' => array
		(
			'exclude'                 => true,
			'search'                  => true,
			'inputType'               => 'checkbox',
			'eval'                    => array('submitOnChange'=>true),
			'sql'                     => "char(1) NOT NULL default ''"
		),
		'repeatEach' => array
		(
			'exclude'                 => true,
			'inputType'               => 'timePeriod',
			'options'                 => array('days', 'weeks', 'months', 'years'),
			'reference'               => &$GLOBALS['TL_LANG']['tl_calendar_events'],
			'eval'                    => array('mandatory'=>true, 'rgxp'=>'natural', 'minval'=>1, 'tl_class'=>'w50'),
			'sql'                     => "varchar(64) NOT NULL default ''"
		),
		'repeatEnd' => array
		(
			'sql'                     => "int(10) unsigned NOT NULL default 0"
		),
		'recurrences' => array
		(
			'exclude'                 => true,
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'rgxp'=>'natural', 'tl_class'=>'w50'),
			'sql'                     => "smallint(5) unsigned NOT NULL default 0"
		),
		'addEnclosure' => array
		(
			'exclude'                 => true,
			'inputType'               => 'checkbox',
			'eval'                    => array('submitOnChange'=>true),
			'sql'                     => "char(1) NOT NULL default ''"
		),
		'enclosure' => array
		(
			'exclude'                 => true,
			'inputType'               => 'fileTree',
			'eval'                    => array('multiple'=>true, 'fieldType'=>'checkbox', 'filesOnly'=>true, 'isDownloads'=>true, 'extensions'=>Config::get('allowedDownload'), 'mandatory'=>true, 'isSortable'=>true),
			'sql'                     => "blob NULL"
		),
		'source' => array
		(
			'exclude'                 => true,
			'filter'                  => true,
			'inputType'               => 'radio',
			'options_callback'        => array('tl_calendar_events', 'getSourceOptions'),
			'reference'               => &$GLOBALS['TL_LANG']['tl_calendar_events'],
			'eval'                    => array('submitOnChange'=>true, 'helpwizard'=>true),
			'sql'                     => "varchar(32) NOT NULL default 'default'"
		),
		'jumpTo' => array
		(
			'exclude'                 => true,
			'inputType'               => 'pageTree',
			'foreignKey'              => 'tl_page.title',
			'eval'                    => array('mandatory'=>true, 'fieldType'=>'radio'),
			'sql'                     => "int(10) unsigned NOT NULL default 0",
			'relation'                => array('type'=>'belongsTo', 'load'=>'lazy')
		),
		'articleId' => array
		(
			'exclude'                 => true,
			'inputType'               => 'select',
			'options_callback'        => array('tl_calendar_events', 'getArticleAlias'),
			'eval'                    => array('chosen'=>true, 'mandatory'=>true, 'tl_class'=>'w50'),
			'sql'                     => "int(10) unsigned NOT NULL default 0",
			'relation'                => array('table'=>'tl_article', 'type'=>'hasOne', 'load'=>'lazy'),
		),
		'url' => array
		(
			'label'                   => &$GLOBALS['TL_LANG']['MSC']['url'],
			'exclude'                 => true,
			'search'                  => true,
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'rgxp'=>'url', 'decodeEntities'=>true, 'maxlength'=>255, 'dcaPicker'=>true, 'addWizardClass'=>false, 'tl_class'=>'w50'),
			'sql'                     => "varchar(255) NOT NULL default ''"
		),
		'target' => array
		(
			'label'                   => &$GLOBALS['TL_LANG']['MSC']['target'],
			'exclude'                 => true,
			'inputType'               => 'checkbox',
			'eval'                    => array('tl_class'=>'w50 m12'),
			'sql'                     => "char(1) NOT NULL default ''"
		),
		'cssClass' => array
		(
			'exclude'                 => true,
			'inputType'               => 'text',
			'eval'                    => array('tl_class'=>'w50'),
			'sql'                     => "varchar(255) NOT NULL default ''"
		),
		'noComments' => array
		(
			'exclude'                 => true,
			'filter'                  => true,
			'inputType'               => 'checkbox',
			'eval'                    => array('tl_class'=>'w50 clr'),
			'sql'                     => "char(1) NOT NULL default ''"
		),
		'featured' => array
		(
			'exclude'                 => true,
			'filter'                  => true,
			'inputType'               => 'checkbox',
			'eval'                    => array('tl_class'=>'w50'),
			'sql'                     => "char(1) NOT NULL default ''"
		),
		'published' => array
		(
			'exclude'                 => true,
			'filter'                  => true,
			'flag'                    => 2,
			'inputType'               => 'checkbox',
			'eval'                    => array('doNotCopy'=>true),
			'sql'                     => "char(1) NOT NULL default ''"
		),
		'start' => array
		(
			'exclude'                 => true,
			'inputType'               => 'text',
			'eval'                    => array('rgxp'=>'datim', 'datepicker'=>true, 'tl_class'=>'w50 wizard'),
			'sql'                     => "varchar(10) NOT NULL default ''"
		),
		'stop' => array
		(
			'exclude'                 => true,
			'inputType'               => 'text',
			'eval'                    => array('rgxp'=>'datim', 'datepicker'=>true, 'tl_class'=>'w50 wizard'),
			'sql'                     => "varchar(10) NOT NULL default ''"
		)
	)
);

/**
 * Provide miscellaneous methods that are used by the data configuration array.
 *
 * @property Calendar $Calendar
 *
 * @author Leo Feyer <https://github.com/leofeyer>
 */
class tl_calendar_events extends Backend
{
	/**
	 * Import the back end user object
	 */
	public function __construct()
	{
		parent::__construct();
		$this->import(BackendUser::class, 'User');
	}

	/**
	 * Check permissions to edit table tl_calendar_events
	 *
	 * @throws AccessDeniedException
	 */
	public function checkPermission()
	{
		$bundles = System::getContainer()->getParameter('kernel.bundles');

		// HOOK: comments extension required
		if (!isset($bundles['ContaoCommentsBundle']))
		{
			$key = array_search('allowComments', $GLOBALS['TL_DCA']['tl_calendar_events']['list']['sorting']['headerFields']);
			unset($GLOBALS['TL_DCA']['tl_calendar_events']['list']['sorting']['headerFields'][$key]);
		}

		if ($this->User->isAdmin)
		{
			return;
		}

		// Set root IDs
		if (empty($this->User->calendars) || !is_array($this->User->calendars))
		{
			$root = array(0);
		}
		else
		{
			$root = $this->User->calendars;
		}

		$id = strlen(Input::get('id')) ? Input::get('id') : CURRENT_ID;

		// Check current action
		switch (Input::get('act'))
		{
			case 'paste':
			case 'select':
				// Check CURRENT_ID here (see #247)
				if (!in_array(CURRENT_ID, $root))
				{
					throw new AccessDeniedException('Not enough permissions to access calendar ID ' . $id . '.');
				}
				break;

			case 'create':
				if (!Input::get('pid') || !in_array(Input::get('pid'), $root))
				{
					throw new AccessDeniedException('Not enough permissions to create events in calendar ID ' . Input::get('pid') . '.');
				}
				break;

			case 'cut':
			case 'copy':
				if (!in_array(Input::get('pid'), $root))
				{
					throw new AccessDeniedException('Not enough permissions to ' . Input::get('act') . ' event ID ' . $id . ' to calendar ID ' . Input::get('pid') . '.');
				}
				// no break

			case 'edit':
			case 'show':
			case 'delete':
			case 'toggle':
			case 'feature':
				$objCalendar = $this->Database->prepare("SELECT pid FROM tl_calendar_events WHERE id=?")
											  ->limit(1)
											  ->execute($id);

				if ($objCalendar->numRows < 1)
				{
					throw new AccessDeniedException('Invalid event ID ' . $id . '.');
				}

				if (!in_array($objCalendar->pid, $root))
				{
					throw new AccessDeniedException('Not enough permissions to ' . Input::get('act') . ' event ID ' . $id . ' of calendar ID ' . $objCalendar->pid . '.');
				}
				break;

			case 'editAll':
			case 'deleteAll':
			case 'overrideAll':
			case 'cutAll':
			case 'copyAll':
				if (!in_array($id, $root))
				{
					throw new AccessDeniedException('Not enough permissions to access calendar ID ' . $id . '.');
				}

				$objCalendar = $this->Database->prepare("SELECT id FROM tl_calendar_events WHERE pid=?")
											  ->execute($id);

				/** @var SessionInterface $objSession */
				$objSession = System::getContainer()->get('session');

				$session = $objSession->all();
				$session['CURRENT']['IDS'] = array_intersect((array) $session['CURRENT']['IDS'], $objCalendar->fetchEach('id'));
				$objSession->replace($session);
				break;

			default:
				if (Input::get('act'))
				{
					throw new AccessDeniedException('Invalid command "' . Input::get('act') . '".');
				}

				if (!in_array($id, $root))
				{
					throw new AccessDeniedException('Not enough permissions to access calendar ID ' . $id . '.');
				}
				break;
		}
	}

	/**
	 * Auto-generate the event alias if it has not been set yet
	 *
	 * @param mixed         $varValue
	 * @param DataContainer $dc
	 *
	 * @return mixed
	 *
	 * @throws Exception
	 */
	public function generateAlias($varValue, DataContainer $dc)
	{
		$aliasExists = function (string $alias) use ($dc): bool
		{
			return $this->Database->prepare("SELECT id FROM tl_calendar_events WHERE alias=? AND id!=?")->execute($alias, $dc->id)->numRows > 0;
		};

		// Generate the alias if there is none
		if (!$varValue)
		{
			$varValue = System::getContainer()->get('contao.slug')->generate($dc->activeRecord->title, CalendarModel::findByPk($dc->activeRecord->pid)->jumpTo, $aliasExists);
		}
		elseif (preg_match('/^[1-9]\d*$/', $varValue))
		{
			throw new Exception(sprintf($GLOBALS['TL_LANG']['ERR']['aliasNumeric'], $varValue));
		}
		elseif ($aliasExists($varValue))
		{
			throw new Exception(sprintf($GLOBALS['TL_LANG']['ERR']['aliasExists'], $varValue));
		}

		return $varValue;
	}

	/**
	 * Set the timestamp to 1970-01-01 (see #26)
	 *
	 * @param integer $value
	 *
	 * @return integer
	 */
	public function loadTime($value)
	{
		return strtotime('1970-01-01 ' . date('H:i:s', $value));
	}

	/**
	 * Set the end time to an empty string (see #23)
	 *
	 * @param integer       $value
	 * @param DataContainer $dc
	 *
	 * @return integer
	 */
	public function loadEndTime($value, DataContainer $dc)
	{
		$return = strtotime('1970-01-01 ' . date('H:i:s', $value));

		// Return an empty string if the start time is the same as the end time (see #23)
		if ($dc->activeRecord && $return == $dc->activeRecord->startTime)
		{
			return '';
		}

		// Return an empty string if no time has been set yet
		if ($dc->activeRecord && $return - $dc->activeRecord->startTime == 86399)
		{
			return '';
		}

		return strtotime('1970-01-01 ' . date('H:i:s', $value));
	}

	/**
	 * Automatically set the end time if not set
	 *
	 * @param mixed         $varValue
	 * @param DataContainer $dc
	 *
	 * @return string
	 */
	public function setEmptyEndTime($varValue, DataContainer $dc)
	{
		if ($varValue === null)
		{
			$varValue = $dc->activeRecord->startTime;
		}

		return $varValue;
	}

	/**
	 * Return the SERP URL
	 *
	 * @param CalendarEventsModel $model
	 *
	 * @return string
	 */
	public function getSerpUrl(CalendarEventsModel $model)
	{
		return Events::generateEventUrl($model, true);
	}

	/**
	 * Return the title tag from the associated page layout
	 *
	 * @param CalendarEventsModel $model
	 *
	 * @return string
	 */
	public function getTitleTag(CalendarEventsModel $model)
	{
		/** @var CalendarModel $calendar */
		if (!$calendar = $model->getRelated('pid'))
		{
			return '';
		}

		/** @var PageModel $page */
		if (!$page = $calendar->getRelated('jumpTo'))
		{
			return '';
		}

		$page->loadDetails();

		/** @var LayoutModel $layout */
		if (!$layout = $page->getRelated('layout'))
		{
			return '';
		}

		global $objPage;

		// Set the global page object so we can replace the insert tags
		$objPage = $page;

		return self::replaceInsertTags(str_replace('{{page::pageTitle}}', '%s', $layout->titleTag ?: '{{page::pageTitle}} - {{page::rootPageTitle}}'));
	}

	/**
	 * Add the type of input field
	 *
	 * @param array $arrRow
	 *
	 * @return string
	 */
	public function listEvents($arrRow)
	{
		$span = Calendar::calculateSpan($arrRow['startTime'], $arrRow['endTime']);

		if ($span > 0)
		{
			$date = Date::parse(Config::get(($arrRow['addTime'] ? 'datimFormat' : 'dateFormat')), $arrRow['startTime']) . $GLOBALS['TL_LANG']['MSC']['cal_timeSeparator'] . Date::parse(Config::get(($arrRow['addTime'] ? 'datimFormat' : 'dateFormat')), $arrRow['endTime']);
		}
		elseif ($arrRow['startTime'] == $arrRow['endTime'])
		{
			$date = Date::parse(Config::get('dateFormat'), $arrRow['startTime']) . ($arrRow['addTime'] ? ' ' . Date::parse(Config::get('timeFormat'), $arrRow['startTime']) : '');
		}
		else
		{
			$date = Date::parse(Config::get('dateFormat'), $arrRow['startTime']) . ($arrRow['addTime'] ? ' ' . Date::parse(Config::get('timeFormat'), $arrRow['startTime']) . $GLOBALS['TL_LANG']['MSC']['cal_timeSeparator'] . Date::parse(Config::get('timeFormat'), $arrRow['endTime']) : '');
		}

		return '<div class="tl_content_left">' . $arrRow['title'] . ' <span style="color:#999;padding-left:3px">[' . $date . ']</span></div>';
	}

	/**
	 * Get all articles and return them as array
	 *
	 * @param DataContainer $dc
	 *
	 * @return array
	 */
	public function getArticleAlias(DataContainer $dc)
	{
		$arrPids = array();
		$arrAlias = array();

		if (!$this->User->isAdmin)
		{
			foreach ($this->User->pagemounts as $id)
			{
				$arrPids[] = array($id);
				$arrPids[] = $this->Database->getChildRecords($id, 'tl_page');
			}

			if (!empty($arrPids))
			{
				$arrPids = array_merge(...$arrPids);
			}
			else
			{
				return $arrAlias;
			}

			$objAlias = $this->Database->prepare("SELECT a.id, a.title, a.inColumn, p.title AS parent FROM tl_article a LEFT JOIN tl_page p ON p.id=a.pid WHERE a.pid IN(" . implode(',', array_map('\intval', array_unique($arrPids))) . ") ORDER BY parent, a.sorting")
									   ->execute($dc->id);
		}
		else
		{
			$objAlias = $this->Database->prepare("SELECT a.id, a.title, a.inColumn, p.title AS parent FROM tl_article a LEFT JOIN tl_page p ON p.id=a.pid ORDER BY parent, a.sorting")
									   ->execute($dc->id);
		}

		if ($objAlias->numRows)
		{
			System::loadLanguageFile('tl_article');

			while ($objAlias->next())
			{
				$arrAlias[$objAlias->parent][$objAlias->id] = $objAlias->title . ' (' . ($GLOBALS['TL_LANG']['COLS'][$objAlias->inColumn] ?: $objAlias->inColumn) . ', ID ' . $objAlias->id . ')';
			}
		}

		return $arrAlias;
	}

	/**
	 * Add the source options depending on the allowed fields (see #5498)
	 *
	 * @param DataContainer $dc
	 *
	 * @return array
	 */
	public function getSourceOptions(DataContainer $dc)
	{
		if ($this->User->isAdmin)
		{
			return array('default', 'internal', 'article', 'external');
		}

		$arrOptions = array('default');

		// Add the "internal" option
		if ($this->User->hasAccess('tl_calendar_events::jumpTo', 'alexf'))
		{
			$arrOptions[] = 'internal';
		}

		// Add the "article" option
		if ($this->User->hasAccess('tl_calendar_events::articleId', 'alexf'))
		{
			$arrOptions[] = 'article';
		}

		// Add the "external" option
		if ($this->User->hasAccess('tl_calendar_events::url', 'alexf'))
		{
			$arrOptions[] = 'external';
		}

		// Add the option currently set
		if ($dc->activeRecord && $dc->activeRecord->source)
		{
			$arrOptions[] = $dc->activeRecord->source;
			$arrOptions = array_unique($arrOptions);
		}

		return $arrOptions;
	}

	/**
	 * Adjust start end end time of the event based on date, span, startTime and endTime
	 *
	 * @param DataContainer $dc
	 */
	public function adjustTime(DataContainer $dc)
	{
		// Return if there is no active record (override all)
		if (!$dc->activeRecord)
		{
			return;
		}

		$arrSet['startTime'] = $dc->activeRecord->startDate;
		$arrSet['endTime'] = $dc->activeRecord->startDate;

		// Set end date
		if ($dc->activeRecord->endDate)
		{
			if ($dc->activeRecord->endDate > $dc->activeRecord->startDate)
			{
				$arrSet['endDate'] = $dc->activeRecord->endDate;
				$arrSet['endTime'] = $dc->activeRecord->endDate;
			}
			else
			{
				$arrSet['endDate'] = $dc->activeRecord->startDate;
				$arrSet['endTime'] = $dc->activeRecord->startDate;
			}
		}

		// Add time
		if ($dc->activeRecord->addTime)
		{
			$arrSet['startTime'] = strtotime(date('Y-m-d', $arrSet['startTime']) . ' ' . date('H:i:s', $dc->activeRecord->startTime));
			$arrSet['endTime'] = strtotime(date('Y-m-d', $arrSet['endTime']) . ' ' . date('H:i:s', $dc->activeRecord->endTime ?: $dc->activeRecord->startTime));
		}

		// Adjust end time of "all day" events
		elseif (($dc->activeRecord->endDate && $arrSet['endDate'] == $arrSet['endTime']) || $arrSet['startTime'] == $arrSet['endTime'])
		{
			$arrSet['endTime'] = (strtotime('+ 1 day', $arrSet['endTime']) - 1);
		}

		$arrSet['repeatEnd'] = 0;

		// Recurring events
		if ($dc->activeRecord->recurring)
		{
			// Unlimited recurrences end on 2106-02-07 07:28:15 (see #4862 and #510)
			if ($dc->activeRecord->recurrences == 0)
			{
				$arrSet['repeatEnd'] = min(4294967295, PHP_INT_MAX);
			}
			else
			{
				$arrRange = StringUtil::deserialize($dc->activeRecord->repeatEach);

				if (isset($arrRange['unit'], $arrRange['value']))
				{
					$arg = $arrRange['value'] * $dc->activeRecord->recurrences;
					$unit = $arrRange['unit'];

					$strtotime = '+ ' . $arg . ' ' . $unit;
					$arrSet['repeatEnd'] = strtotime($strtotime, $arrSet['endTime']);
				}
			}
		}

		$this->Database->prepare("UPDATE tl_calendar_events %s WHERE id=?")->set($arrSet)->execute($dc->id);
	}

	/**
	 * Check for modified calendar feeds and update the XML files if necessary
	 */
	public function generateFeed()
	{
		/** @var SessionInterface $objSession */
		$objSession = System::getContainer()->get('session');

		$session = $objSession->get('calendar_feed_updater');

		if (empty($session) || !is_array($session))
		{
			return;
		}

		$request = System::getContainer()->get('request_stack')->getCurrentRequest();

		if ($request)
		{
			$origScope = $request->attributes->get('_scope');
			$request->attributes->set('_scope', 'frontend');
		}

		$this->import(Calendar::class, 'Calendar');

		foreach ($session as $id)
		{
			$this->Calendar->generateFeedsByCalendar($id);
		}

		$this->import(Automator::class, 'Automator');
		$this->Automator->generateSitemap();

		if ($request)
		{
			$request->attributes->set('_scope', $origScope);
		}

		$objSession->set('calendar_feed_updater', null);
	}

	/**
	 * Schedule a calendar feed update
	 *
	 * This method is triggered when a single event or multiple events are
	 * modified (edit/editAll), moved (cut/cutAll) or deleted (delete/deleteAll).
	 * Since duplicated events are unpublished by default, it is not necessary
	 * to schedule updates on copyAll as well.
	 *
	 * @param DataContainer $dc
	 */
	public function scheduleUpdate(DataContainer $dc)
	{
		// Return if there is no ID
		if (!$dc->activeRecord || !$dc->activeRecord->pid || Input::get('act') == 'copy')
		{
			return;
		}

		/** @var SessionInterface $objSession */
		$objSession = System::getContainer()->get('session');

		// Store the ID in the session
		$session = $objSession->get('calendar_feed_updater');
		$session[] = $dc->activeRecord->pid;
		$objSession->set('calendar_feed_updater', array_unique($session));
	}

	/**
	 * Return the "feature/unfeature element" button
	 *
	 * @param array  $row
	 * @param string $href
	 * @param string $label
	 * @param string $title
	 * @param string $icon
	 * @param string $attributes
	 *
	 * @return string
	 */
	public function iconFeatured($row, $href, $label, $title, $icon, $attributes)
	{
		if (Input::get('fid'))
		{
			$this->toggleFeatured(Input::get('fid'), (Input::get('state') == 1), (@func_get_arg(12) ?: null));
			$this->redirect($this->getReferer());
		}

		// Check permissions AFTER checking the fid, so hacking attempts are logged
		if (!$this->User->hasAccess('tl_calendar_events::featured', 'alexf'))
		{
			return '';
		}

		$href .= '&amp;fid=' . $row['id'] . '&amp;state=' . ($row['featured'] ? '' : 1);

		if (!$row['featured'])
		{
			$icon = 'featured_.svg';
		}

		return '<a href="' . $this->addToUrl($href) . '" title="' . StringUtil::specialchars($title) . '"' . $attributes . '>' . Image::getHtml($icon, $label, 'data-state="' . ($row['featured'] ? 1 : 0) . '"') . '</a> ';
	}

	/**
	 * Feature/unfeature an event
	 *
	 * @param integer       $intId
	 * @param boolean       $blnVisible
	 * @param DataContainer $dc
	 *
	 * @throws AccessDeniedException
	 */
	public function toggleFeatured($intId, $blnVisible, DataContainer $dc=null)
	{
		// Check permissions to edit
		Input::setGet('id', $intId);
		Input::setGet('act', 'feature');

		if ($dc)
		{
			$dc->id = $intId; // see #8043
		}

		// Trigger the onload_callback
		if (is_array($GLOBALS['TL_DCA']['tl_calendar_events']['config']['onload_callback']))
		{
			foreach ($GLOBALS['TL_DCA']['tl_calendar_events']['config']['onload_callback'] as $callback)
			{
				if (is_array($callback))
				{
					$this->import($callback[0]);
					$this->{$callback[0]}->{$callback[1]}($dc);
				}
				elseif (is_callable($callback))
				{
					$callback($dc);
				}
			}
		}

		// Check permissions to feature
		if (!$this->User->hasAccess('tl_calendar_events::featured', 'alexf'))
		{
			throw new AccessDeniedException('Not enough permissions to feature/unfeature event ID ' . $intId . '.');
		}

		$objRow = $this->Database->prepare("SELECT * FROM tl_calendar_events WHERE id=?")
								 ->limit(1)
								 ->execute($intId);

		if ($objRow->numRows < 1)
		{
			throw new AccessDeniedException('Invalid event ID ' . $intId . '.');
		}

		// Set the current record
		if ($dc)
		{
			$dc->activeRecord = $objRow;
		}

		$objVersions = new Versions('tl_calendar_events', $intId);
		$objVersions->initialize();

		// Trigger the save_callback
		if (is_array($GLOBALS['TL_DCA']['tl_calendar_events']['fields']['featured']['save_callback']))
		{
			foreach ($GLOBALS['TL_DCA']['tl_calendar_events']['fields']['featured']['save_callback'] as $callback)
			{
				if (is_array($callback))
				{
					$this->import($callback[0]);
					$blnVisible = $this->{$callback[0]}->{$callback[1]}($blnVisible, $dc);
				}
				elseif (is_callable($callback))
				{
					$blnVisible = $callback($blnVisible, $this);
				}
			}
		}

		$time = time();

		// Update the database
		$this->Database->prepare("UPDATE tl_calendar_events SET tstamp=$time, featured='" . ($blnVisible ? 1 : '') . "' WHERE id=?")
					   ->execute($intId);

		if ($dc)
		{
			$dc->activeRecord->tstamp = $time;
			$dc->activeRecord->published = ($blnVisible ? '1' : '');
		}

		// Trigger the onsubmit_callback
		if (is_array($GLOBALS['TL_DCA']['tl_calendar_events']['config']['onsubmit_callback']))
		{
			foreach ($GLOBALS['TL_DCA']['tl_calendar_events']['config']['onsubmit_callback'] as $callback)
			{
				if (is_array($callback))
				{
					$this->import($callback[0]);
					$this->{$callback[0]}->{$callback[1]}($dc);
				}
				elseif (is_callable($callback))
				{
					$callback($dc);
				}
			}
		}

		$objVersions->create();

		if ($dc)
		{
			$dc->invalidateCacheTags();
		}
	}

	/**
	 * Return the "toggle visibility" button
	 *
	 * @param array  $row
	 * @param string $href
	 * @param string $label
	 * @param string $title
	 * @param string $icon
	 * @param string $attributes
	 *
	 * @return string
	 */
	public function toggleIcon($row, $href, $label, $title, $icon, $attributes)
	{
		if (Input::get('tid'))
		{
			$this->toggleVisibility(Input::get('tid'), (Input::get('state') == 1), (@func_get_arg(12) ?: null));
			$this->redirect($this->getReferer());
		}

		// Check permissions AFTER checking the tid, so hacking attempts are logged
		if (!$this->User->hasAccess('tl_calendar_events::published', 'alexf'))
		{
			return '';
		}

		$href .= '&amp;tid=' . $row['id'] . '&amp;state=' . ($row['published'] ? '' : 1);

		if (!$row['published'])
		{
			$icon = 'invisible.svg';
		}

		return '<a href="' . $this->addToUrl($href) . '" title="' . StringUtil::specialchars($title) . '"' . $attributes . '>' . Image::getHtml($icon, $label, 'data-state="' . ($row['published'] ? 1 : 0) . '"') . '</a> ';
	}

	/**
	 * Disable/enable a user group
	 *
	 * @param integer       $intId
	 * @param boolean       $blnVisible
	 * @param DataContainer $dc
	 *
	 * @throws AccessDeniedException
	 */
	public function toggleVisibility($intId, $blnVisible, DataContainer $dc=null)
	{
		// Set the ID and action
		Input::setGet('id', $intId);
		Input::setGet('act', 'toggle');

		if ($dc)
		{
			$dc->id = $intId; // see #8043
		}

		// Trigger the onload_callback
		if (is_array($GLOBALS['TL_DCA']['tl_calendar_events']['config']['onload_callback']))
		{
			foreach ($GLOBALS['TL_DCA']['tl_calendar_events']['config']['onload_callback'] as $callback)
			{
				if (is_array($callback))
				{
					$this->import($callback[0]);
					$this->{$callback[0]}->{$callback[1]}($dc);
				}
				elseif (is_callable($callback))
				{
					$callback($dc);
				}
			}
		}

		// Check the field access
		if (!$this->User->hasAccess('tl_calendar_events::published', 'alexf'))
		{
			throw new AccessDeniedException('Not enough permissions to publish/unpublish event ID ' . $intId . '.');
		}

		$objRow = $this->Database->prepare("SELECT * FROM tl_calendar_events WHERE id=?")
								 ->limit(1)
								 ->execute($intId);

		if ($objRow->numRows < 1)
		{
			throw new AccessDeniedException('Invalid event ID ' . $intId . '.');
		}

		// Set the current record
		if ($dc)
		{
			$dc->activeRecord = $objRow;
		}

		$objVersions = new Versions('tl_calendar_events', $intId);
		$objVersions->initialize();

		// Trigger the save_callback
		if (is_array($GLOBALS['TL_DCA']['tl_calendar_events']['fields']['published']['save_callback']))
		{
			foreach ($GLOBALS['TL_DCA']['tl_calendar_events']['fields']['published']['save_callback'] as $callback)
			{
				if (is_array($callback))
				{
					$this->import($callback[0]);
					$blnVisible = $this->{$callback[0]}->{$callback[1]}($blnVisible, $dc);
				}
				elseif (is_callable($callback))
				{
					$blnVisible = $callback($blnVisible, $dc);
				}
			}
		}

		$time = time();

		// Update the database
		$this->Database->prepare("UPDATE tl_calendar_events SET tstamp=$time, published='" . ($blnVisible ? '1' : '') . "' WHERE id=?")
					   ->execute($intId);

		if ($dc)
		{
			$dc->activeRecord->tstamp = $time;
			$dc->activeRecord->published = ($blnVisible ? '1' : '');
		}

		// Trigger the onsubmit_callback
		if (is_array($GLOBALS['TL_DCA']['tl_calendar_events']['config']['onsubmit_callback']))
		{
			foreach ($GLOBALS['TL_DCA']['tl_calendar_events']['config']['onsubmit_callback'] as $callback)
			{
				if (is_array($callback))
				{
					$this->import($callback[0]);
					$this->{$callback[0]}->{$callback[1]}($dc);
				}
				elseif (is_callable($callback))
				{
					$callback($dc);
				}
			}
		}

		$objVersions->create();

		// The onsubmit_callback has triggered scheduleUpdate(), so run generateFeed() now
		$this->generateFeed();

		if ($dc)
		{
			$dc->invalidateCacheTags();
		}
	}
}
}

namespace  {
/**
 * Contao Open Source CMS
 *
 * Copyright (C) 2005-2012 Leo Feyer
 *
 * @package   Contao
 * @author    Kester Mielke
 * @license   LGPL
 * @copyright Kester Mielke 2009-2018
 */

foreach ($GLOBALS['TL_DCA']['tl_calendar_events']['config']['onsubmit_callback'] as $k => $v) {
    if ($v[0] == 'tl_calendar_events' && $v[1] == 'adjustTime') {
        unset($GLOBALS['TL_DCA']['tl_calendar_events']['config']['onsubmit_callback'][$k]);
        array_insert($GLOBALS['TL_DCA']['tl_calendar_events']['config']['onsubmit_callback'], 0,
            array(
                array('tl_calendar_events_ext', 'adjustTime'),
                array('tl_calendar_events_ext', 'checkOverlapping')
            ));
    }
}

foreach($GLOBALS['TL_DCA']['tl_calendar_events']['palettes'] as &$palette) {
  $palette = str_replace('endDate', 'endDate,hideOnWeekend', $palette);
}

$GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['default'] = str_replace
(
    'addTime,',
    'showOnFreeDay,addTime,',
    $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['default']
);

if (class_exists('leads\leads')) {
    $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['default'] = str_replace
    (
        '{recurring_legend},recurring;',
        '{location_legend},location_name,location_str,location_plz,location_ort;{contact_legend},location_link,location_contact,location_mail;{regform_legend},useRegistration;{recurring_legend},recurring;{recurring_legend_ext},recurringExt;{repeatFixedDates_legend},repeatFixedDates;{exception_legend},useExceptions;',
        $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['default']
    );
    $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['article'] = str_replace
    (
        '{recurring_legend},recurring;',
        '{location_legend},location_name,location_str,location_plz,location_ort;{contact_legend},location_link,location_contact,location_mail;{regform_legend},useRegistration;{recurring_legend},recurring;{recurring_legend_ext},recurringExt;{repeatFixedDates_legend},repeatFixedDates;{exception_legend},useExceptions;',
        $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['article']
    );
    $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['internal'] = str_replace
    (
        '{recurring_legend},recurring;',
        '{location_legend},location_name,location_str,location_plz,location_ort;{contact_legend},location_link,location_contact,location_mail;{regform_legend},useRegistration;{recurring_legend},recurring;{recurring_legend_ext},recurringExt;{repeatFixedDates_legend},repeatFixedDates;{exception_legend},useExceptions;',
        $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['internal']
    );
    $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['external'] = str_replace
    (
        '{recurring_legend},recurring;',
        '{location_legend},location_name,location_str,location_plz,location_ort;{contact_legend},location_link,location_contact,location_mail;{regform_legend},useRegistration;{recurring_legend},recurring;{recurring_legend_ext},recurringExt;{repeatFixedDates_legend},repeatFixedDates;{exception_legend},useExceptions;',
        $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['external']
    );
} else {
    $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['default'] = str_replace
    (
        '{recurring_legend},recurring;',
        '{location_legend},location_name,location_str,location_plz,location_ort;{contact_legend},location_link,location_contact,location_mail;{recurring_legend},recurring;{recurring_legend_ext},recurringExt;{repeatFixedDates_legend},repeatFixedDates;{exception_legend},useExceptions;',
        $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['default']
    );
    $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['article'] = str_replace
    (
        '{recurring_legend},recurring;',
        '{location_legend},location_name,location_str,location_plz,location_ort;{contact_legend},location_link,location_contact,location_mail;{recurring_legend},recurring;{recurring_legend_ext},recurringExt;{repeatFixedDates_legend},repeatFixedDates;{exception_legend},useExceptions;',
        $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['article']
    );
    $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['internal'] = str_replace
    (
        '{recurring_legend},recurring;',
        '{location_legend},location_name,location_str,location_plz,location_ort;{contact_legend},location_link,location_contact,location_mail;{recurring_legend},recurring;{recurring_legend_ext},recurringExt;{repeatFixedDates_legend},repeatFixedDates;{exception_legend},useExceptions;',
        $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['internal']
    );
    $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['external'] = str_replace
    (
        '{recurring_legend},recurring;',
        '{location_legend},location_name,location_str,location_plz,location_ort;{contact_legend},location_link,location_contact,location_mail;{recurring_legend},recurring;{recurring_legend_ext},recurringExt;{repeatFixedDates_legend},repeatFixedDates;{exception_legend},useExceptions;',
        $GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['external']
    );
}

// change the default palettes
array_insert($GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['__selector__'], 99, 'recurringExt');
array_insert($GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['__selector__'], 99, 'useExceptions');
array_insert($GLOBALS['TL_DCA']['tl_calendar_events']['palettes']['__selector__'], 99, 'useRegistration');

// change the default palettes
$GLOBALS['TL_DCA']['tl_calendar_events']['subpalettes']['recurring'] = str_replace
(
    'repeatEach,recurrences',
    'hideOnWeekend,repeatEach,recurrences,repeatWeekday,repeatEnd',
    $GLOBALS['TL_DCA']['tl_calendar_events']['subpalettes']['recurring']
);

// change the default palettes
$GLOBALS['TL_DCA']['tl_calendar_events']['subpalettes']['addTime'] = str_replace
(
    'startTime,endTime',
    'ignoreEndTime,startTime,endTime',
    $GLOBALS['TL_DCA']['tl_calendar_events']['subpalettes']['addTime']
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['repeatWeekday'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['repeatWeekday'],
    'exclude' => true,
    'filter' => true,
    'inputType' => 'checkbox',
    'options' => array(1, 2, 3, 4, 5, 6, 0),
    'reference' => &$GLOBALS['TL_LANG']['DAYS'],
    'load_callback' => array(array('tl_calendar_events_ext', 'getWeekday')),
    'eval' => array('multiple' => true, 'tl_class' => 'clr'),
    'sql' => "varchar(128) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['repeatFixedDates'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['repeatFixedDates'],
    'exclude' => true,
    'inputType' => 'multiColumnWizard',
    'eval' => array
    (
        'columnsCallback' => array('tl_calendar_events_ext', 'listFixedDates'),
        'buttons' => array('up' => false, 'down' => false)
    ),
    'sql' => "blob NULL"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['ignoreEndTime'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['ignoreEndTime'],
    'default' => 0,
    'exclude' => true,
    'inputType' => 'checkbox',
    'eval' => array('tl_class' => 'long clr'),
    'sql' => "char(1) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['useExceptions'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['useExceptions'],
    'exclude' => true,
    'inputType' => 'checkbox',
    'eval' => array('submitOnChange' => true, 'tl_class' => 'long clr'),
    'sql' => "char(1) NOT NULL default ''",
    'save_callback' => array
    (
        array('tl_calendar_events_ext', 'checkExceptions')
    )
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['showOnFreeDay'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['showOnFreeDay'],
    'exclude' => true,
    'filter' => false,
    'inputType' => 'checkbox',
    'sql' => "char(1) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['weekday'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['weekday'],
    'exclude' => true,
    'filter' => true,
    'inputType' => 'select',
    'options' => array(0, 1, 2, 3, 4, 5, 6),
    'reference' => &$GLOBALS['TL_LANG']['DAYS'],
    'sql' => "char(1) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['recurring'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['recurring'],
    'exclude' => true,
    'filter' => true,
    'inputType' => 'checkbox',
    'eval' => array('submitOnChange' => true, 'tl_class' => 'w50'),
    'sql' => "char(1) NOT NULL default ''",
    'save_callback' => array
    (
        array('tl_calendar_events_ext', 'checkRecurring')
    )
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['repeatEach'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['repeatEach'],
    'default' => 1,
    'exclude' => true,
    'inputType' => 'timePeriod',
    'options' => array('days', 'weeks', 'months', 'years'),
    'reference' => &$GLOBALS['TL_LANG']['tl_calendar_events'],
    'eval' => array('mandatory' => true, 'rgxp' => 'natural', 'minval' => 1, 'tl_class' => 'w50'),
    'sql' => "varchar(64) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['hideOnWeekend'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['hideOnWeekend'],
    'exclude' => true,
    'filter' => false,
    'inputType' => 'checkbox',
    'eval' => array('tl_class' => 'w50'),
    'sql' => "char(1) NOT NULL default ''"
);

// change the default palettes
$GLOBALS['TL_DCA']['tl_calendar_events']['subpalettes']['recurringExt'] = 'repeatEachExt,recurrences,repeatEnd';
$GLOBALS['TL_DCA']['tl_calendar_events']['subpalettes']['useExceptions'] = 'repeatExceptionsInt,repeatExceptionsPer,repeatExceptions';
$GLOBALS['TL_DCA']['tl_calendar_events']['subpalettes']['useRegistration'] = 'regconfirm,regperson,regform,regstartdate,regenddate';

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['useRegistration'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['useRegistration'],
    'exclude' => true,
    'inputType' => 'checkbox',
    'eval' => array('submitOnChange' => true, 'tl_class' => 'w50'),
    'sql' => "char(1) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['regconfirm'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['regconfirm'],
    'default' => 0,
    'exclude' => true,
    'inputType' => 'checkbox',
    'eval' => array('tl_class' => 'w50 m12'),
    'sql' => "char(1) NOT NULL default ''",
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['regperson'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['regperson'],
    'default' => 0,
    'exclude' => true,
    'filter' => false,
    'inputType' => 'multiColumnWizard',
    'load_callback' => array(array('tl_calendar_events_ext', 'getmaxperson')),
    'eval' => array
    (
        'tl_class' => 'w50 clr',
        'columnsCallback' => array('tl_calendar_events_ext', 'setmaxperson'),
        'buttons' => array('add' => false, 'new' => false, 'up' => false, 'down' => false, 'delete' => false, 'copy' => false)
    ),
    'sql' => "blob NULL"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['regform'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['regform'],
    'exclude' => true,
    'filter' => true,
    'inputType' => 'select',
    'options_callback' => array('tl_calendar_events_ext', 'listRegForms'),
    'eval' => array('mandatory' => true, 'tl_class' => 'w50 m12', 'includeBlankOption' => true, 'chosen' => true),
    'sql' => "int(10) unsigned NOT NULL default '0'"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['regstartdate'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['regstartdate'],
    'default' => time(),
    'exclude' => true,
    'inputType' => 'text',
    'eval' => array('rgxp' => 'datim', 'mandatory' => false, 'doNotCopy' => true, 'datepicker' => true, 'tl_class' => 'w50 wizard'),
    'sql' => "int(10) unsigned NULL"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['regenddate'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['regenddate'],
    'default' => time(),
    'exclude' => true,
    'inputType' => 'text',
    'eval' => array('rgxp' => 'datim', 'mandatory' => false, 'doNotCopy' => true, 'datepicker' => true, 'tl_class' => 'w50 wizard'),
    'sql' => "int(10) unsigned NULL"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['recurringExt'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['recurringExt'],
    'exclude' => true,
    'inputType' => 'checkbox',
    'eval' => array('submitOnChange' => true, 'tl_class' => 'long clr'),
    'sql' => "char(1) NOT NULL default ''",
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['location_name'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['location_name'],
    'exclude' => true,
    'search' => true,
    'inputType' => 'text',
    'eval' => array('maxlength' => 255, 'tl_class' => 'w50'),
    'sql' => "varchar(255) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['location_str'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['location_str'],
    'exclude' => true,
    'search' => true,
    'inputType' => 'text',
    'eval' => array('maxlength' => 255, 'tl_class' => 'w50'),
    'sql' => "varchar(255) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['location_plz'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['location_plz'],
    'exclude' => true,
    'search' => true,
    'inputType' => 'text',
    'eval' => array('rgxp' => 'digit', 'maxlength' => 10, 'tl_class' => 'w50'),
    'sql' => "varchar(10) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['location_ort'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['location_ort'],
    'exclude' => true,
    'search' => true,
    'inputType' => 'text',
    'eval' => array('maxlength' => 255, 'tl_class' => 'w50'),
    'sql' => "varchar(255) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['location_link'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['location_link'],
    'exclude' => true,
    'search' => true,
    'inputType' => 'text',
    'eval' => array('rgxp' => 'url', 'maxlength' => 255, 'tl_class' => 'long'),
    'sql' => "varchar(255) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['location_contact'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['location_contact'],
    'exclude' => true,
    'search' => true,
    'inputType' => 'text',
    'eval' => array('maxlength' => 255, 'tl_class' => 'w50'),
    'sql' => "varchar(255) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['location_mail'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['location_mail'],
    'exclude' => true,
    'search' => true,
    'inputType' => 'text',
    'eval' => array('rgxp' => 'email', 'maxlength' => 255, 'decodeEntities' => true, 'tl_class' => 'w50'),
    'sql' => "varchar(255) NOT NULL default ''"
);

// new repeat options for events
$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['repeatEachExt'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['repeatEachExt'],
    'exclude' => true,
    'inputType' => 'timePeriodExt',
    'options' => array
    (
        array('first', 'second', 'third', 'fourth', 'fifth', 'last'),
        array('monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday')
    ),
    'reference' => &$GLOBALS['TL_LANG']['tl_calendar_events'],
    'eval' => array('mandatory' => true, 'tl_class' => 'w50'),
    'default' => &$GLOBALS['TL_CONFIG']['tl_calendar_events']['weekdays'][date('w', time())],
    'sql' => "text NULL"
);

// added submitOnChange to recurrences
$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['recurrences'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['recurrences'],
    'exclude' => true,
    'inputType' => 'text',
    'eval' => array('mandatory' => true, 'rgxp' => 'digit', 'submitOnChange' => true, 'tl_class' => 'w50'),
    'sql' => "smallint(5) unsigned NOT NULL default '0'"
);

// list of exceptions
$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['repeatExceptions'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['repeatExceptions'],
    'exclude' => true,
    'inputType' => 'multiColumnWizard',
    'eval' => array
    (
        'columnsCallback' => array('tl_calendar_events_ext', 'listMultiExceptions'),
        'buttons' => array('up' => false, 'down' => false)
    ),
    'sql' => "blob NULL"
);

// list of exceptions
$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['repeatExceptionsInt'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['repeatExceptionsInt'],
    'exclude' => true,
    'inputType' => 'multiColumnWizard',
    'eval' => array
    (
        'columnsCallback' => array('tl_calendar_events_ext', 'listMultiExceptions'),
        'buttons' => array('up' => false, 'down' => false)
    ),
    'sql' => "blob NULL"
);

// list of exceptions
$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['repeatExceptionsPer'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['repeatExceptionsPer'],
    'exclude' => true,
    'inputType' => 'multiColumnWizard',
    'eval' => array
    (
        'columnsCallback' => array('tl_calendar_events_ext', 'listMultiExceptions'),
        'buttons' => array('up' => false, 'down' => false)
    ),
    'sql' => "blob NULL"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['repeatDates'] = array
(
    'sql' => "blob NULL"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['allRecurrences'] = array
(
    'sql' => "blob NULL"
);

$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['exceptionList'] = array
(
    'sql' => "blob NULL"
);

// display the end of the recurrences (read only)
$GLOBALS['TL_DCA']['tl_calendar_events']['fields']['repeatEnd'] = array
(
    'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['repeatEnd'],
    'exclude' => true,
    'inputType' => 'text',
    'eval' => array('readonly' => true, 'rgxp' => 'date', 'tl_class' => 'clr'),
    'sql' => "int(10) unsigned NOT NULL default '0'"
);

use Kmielke\CalendarExtendedBundle\CalendarLeadsModel;
use Kmielke\CalendarExtendedBundle\CalendarEventsModelExt;

/**
 * Class tl_calendar_events_ext
 *
 * Provide miscellaneous methods that are used by the data configuration array.
 * @copyright  Kester Mielke 2009-2018
 * @author     Kester Mielke
 * @package    Controller
 */
class tl_calendar_events_ext extends \Backend
{

    /**
     * Import the back end user object
     */
    public function __construct()
    {
        parent::__construct();
        $this->import('BackendUser', 'User');
    }


    /**
     * @param $varValue
     * @return null
     */
    public function getWeekday($varValue)
    {
        if ($varValue === '') {
            return 9;
        }
        return $varValue;
    }


    /**
     * Just check that only one option is active for recurring events
     * @param $varValue
     * @param \DataContainer $dc
     * @return mixed
     * @throws \Exception
     */
    public function checkRecurring($varValue, \DataContainer $dc)
    {
        if ($varValue) {
            if ($dc->activeRecord->recurring && $dc->activeRecord->recurringExt) {
                throw new \Exception($GLOBALS['TL_LANG']['tl_calendar_events']['checkRecurring']);
            }
        }

        return $varValue;
    }


    /**
     * Just check if any kind of recurring is in use
     * @param $varValue
     * @param \DataContainer $dc
     * @return mixed
     * @throws \Exception
     */
    public function checkExceptions($varValue, \DataContainer $dc)
    {
        if ($varValue) {
            if (!$dc->activeRecord->recurring && !$dc->activeRecord->recurringExt) {
                throw new \Exception($GLOBALS['TL_LANG']['tl_calendar_events']['checkExceptions']);
            }
        }

        return $varValue;
    }


    /**
     * @param \DataContainer $dc
     * @return mixed
     * @throws \Exception
     * @return boolean
     */
    public function checkOverlapping(\DataContainer $dc)
    {
        // Return if there is no active record (override all)
        if (!$dc->activeRecord) {
            return false;
        }

        // Return if there event is recurring
        if ($dc->activeRecord->recurring || $dc->activeRecord->recurringExt) {
            return false;
        }

        // Set start date
        $intStart = $dc->activeRecord->startDate;
        $intEnd = $dc->activeRecord->startDate;

        $intStart = strtotime(date('d.m.Y', $intStart) . ' 00:00');

        // Set end date
        if (strlen($dc->activeRecord->endDate)) {
            if ($dc->activeRecord->endDate > $dc->activeRecord->startDate) {
                $intEnd = $dc->activeRecord->endDate;
            } else {
                $intEnd = $dc->activeRecord->startDate;
            }
        }
        $intEnd = strtotime(date('d.m.Y', $intEnd) . ' 23:59');

        // Add time
        if ($dc->activeRecord->addTime) {
            $intStart = strtotime(date('d.m.Y', $intStart) . ' ' . date('H:i:s', $dc->activeRecord->startTime));
            $intEnd = strtotime(date('d.m.Y', $intEnd) . ' ' . date('H:i:s', $dc->activeRecord->endTime));
        }

        // Check if we have time overlapping events
        $uniqueEvents = (\CalendarModel::findById($dc->activeRecord->pid)->uniqueEvents) ? true : false;
        if ($uniqueEvents) {
            // array for events
            $nonUniqueEvents = array();

            // find all events
            $objEvents = CalendarEventsModelExt::findCurrentByPid(
                (int)$dc->activeRecord->pid,
                (int)$dc->activeRecord->startTime,
                (int)$dc->activeRecord->endTime);
            if ($objEvents !== null) {
                while ($objEvents->next()) {
                    // do not add the event with the current id
                    if ($objEvents->id === $dc->activeRecord->id) {
                        continue;
                    }

                    // findCurrentByPid also returns recurring events. therefor we have to check the times
                    if (($intStart > $objEvents->startTime && $intStart < $objEvents->endTime) ||
                        ($intEnd > $objEvents->startTime && $intEnd < $objEvents->endTime) ||
                        ($intStart < $objEvents->startTime && $intEnd > $objEvents->endTime) ||
                        ($intStart == $objEvents->startTime && $intEnd == $objEvents->endTime)
                    ) {
                        $nonUniqueEvents[] = $objEvents->id;
                    }
                }

                if (count($nonUniqueEvents) > 0) {
                    \Message::addError($GLOBALS['TL_LANG']['tl_calendar_events']['nonUniqueEvents'] . ' (' . implode(',', $nonUniqueEvents) . ')');
                    $this->redirect($this->addToUrl());
                }
            }
        }
        return true;
    }


    /**
     * @param \DataContainer $dc
     */
    public function adjustTime(\DataContainer $dc)
    {
        // Return if there is no active record (override all)
        if (!$dc->activeRecord) {
            return;
        }

        $maxCount = ($GLOBALS['TL_CONFIG']['tl_calendar_events']['maxRepeatExceptions']) ? $GLOBALS['TL_CONFIG']['tl_calendar_events']['maxRepeatExceptions'] : 365;
        $maxELCount = 250;

        $arrSet['weekday'] = (int)date("w", $dc->activeRecord->startDate);
        $arrSet['startTime'] = (int)$dc->activeRecord->startDate;
        $arrSet['endTime'] = (int)$dc->activeRecord->startDate;

        // Set end date
        if (strlen($dc->activeRecord->endDate)) {
            if ($dc->activeRecord->endDate > $dc->activeRecord->startDate) {
                $arrSet['endDate'] = (int)$dc->activeRecord->endDate;
                $arrSet['endTime'] = (int)$dc->activeRecord->endDate;
            } else {
                $arrSet['endDate'] = (int)$dc->activeRecord->startDate;
                $arrSet['endTime'] = (int)$dc->activeRecord->startDate;
            }
        }

        // Add time
        if ($dc->activeRecord->addTime) {
            $arrSet['startTime'] = strtotime(date('d.m.Y', $arrSet['startTime']) . ' ' . date('H:i:s', $dc->activeRecord->startTime));
            if (!$dc->activeRecord->ignoreEndTime) {
                $arrSet['endTime'] = strtotime(date('d.m.Y', $arrSet['endTime']) . ' ' . date('H:i:s', $dc->activeRecord->endTime));
            }
        }

        // Set endtime to starttime always...
        if ($dc->activeRecord->addTime && $dc->activeRecord->ignoreEndTime) {
            //$arrSet['endTime'] = strtotime(date('d.m.Y', $arrSet['endTime']) . ' 23:59:59');
            $arrSet['endTime'] = $arrSet['startTime'];
        } // Adjust end time of "all day" events
        elseif ((strlen($dc->activeRecord->endDate) && $arrSet['endDate'] == $arrSet['endTime']) || $arrSet['startTime'] == $arrSet['endTime']) {
            $arrSet['endTime'] = (strtotime('+ 1 day', $arrSet['endTime']) - 1);
        }

        $arrSet['repeatEnd'] = $arrSet['endTime'];

        // Array of possible repeatEnd dates...
        $maxRepeatEnd = array();
        $maxRepeatEnd[] = $arrSet['repeatEnd'];

        // Array of all recurrences
        $arrAllRecurrences =  array();

        // Set the repeatEnd date
        $arrFixDates = array();
        $arrayFixedDates = deserialize($dc->activeRecord->repeatFixedDates) ? deserialize($dc->activeRecord->repeatFixedDates) : null;
        if (!is_null($arrayFixedDates)) {
            usort($arrayFixedDates, function ($a, $b) {
                $intTimeStampA = strtotime($a["new_repeat"].$a['new_start']);
                $intTimeStampB = strtotime($b["new_repeat"].$b['new_start']);
                return strcmp($intTimeStampA, $intTimeStampB);
            });

            foreach ($arrayFixedDates as $fixedDate) {
                // Check if we have a date
                if (!strlen($fixedDate['new_repeat'])) {
                    continue;
                }

                // Check the date
                try {
                    $newDate = new \Date($fixedDate['new_repeat']);
                } catch (\Exception $e) {
                    return false;
                }

                //$new_fix_date = strtotime($fixedDate['new_repeat']);
                $new_fix_date = $fixedDate['new_repeat'];

                // Check if we have a new start time
                //$new_fix_start_time = strlen($fixedDate['new_start']) ? $fixedDate['new_start'] : date('H:i', $arrSet['startTime']);
                $new_fix_start_time = strlen($fixedDate['new_start']) ? date('H:i', $fixedDate['new_start']) : date('H:i', $arrSet['startTime']);
                //$new_fix_end_time = strlen($fixedDate['new_end']) ? $fixedDate['new_end'] : date('H:i', $arrSet['endTime']);
                $new_fix_end_time = strlen($fixedDate['new_end']) ? date('H:i', $fixedDate['new_end']) : date('H:i', $arrSet['endTime']);

                $new_fix_start_date = strtotime(date("d.m.Y", $new_fix_date) . ' ' . date("H:i", strtotime($new_fix_start_time)));
                $new_fix_end_date = strtotime(date("d.m.Y", $new_fix_date) . ' ' . date("H:i", strtotime($new_fix_end_time)));

                $arrFixDates[$new_fix_start_date] = date('d.m.Y H:i', $new_fix_start_date);
                $arrAllRecurrences[$new_fix_start_date] = array(
                    'int_start' => $new_fix_start_date,
                    'int_end' => $new_fix_end_date,
                    'str_start' => \Date::parse($GLOBALS['TL_CONFIG']['datimFormat'], $new_fix_start_date),
                    'str_end' => \Date::parse($GLOBALS['TL_CONFIG']['datimFormat'], $new_fix_end_date)
                );
                $maxRepeatEnd[] = $new_fix_end_date;
            }
            // PW: keep custom sorting
            //$arrSet['repeatFixedDates'] = $arrayFixedDates;
        } else {
            $arrSet['repeatFixedDates'] = null;
        }

        // changed default recurring
        if ($dc->activeRecord->recurring) {
            $arrRange = deserialize($dc->activeRecord->repeatEach);

            $arg = $arrRange['value'] * $dc->activeRecord->recurrences;
            $unit = $arrRange['unit'];

            $strtotime = '+ ' . $arg . ' ' . $unit;
            $arrSet['repeatEnd'] = strtotime($strtotime, $arrSet['endTime']);

            // store the list of dates
            $next = $arrSet['startTime'];
            $nextEnd = $arrSet['endTime'];
            $count = $dc->activeRecord->recurrences;

            // array of the exception dates
            $arrDates = array();
            $arrDates[$next] = date('d.m.Y H:i', $next);

            // array of all recurrences
            $arrAllRecurrences[$next] = array(
                'int_start' => $next,
                'int_end' => $nextEnd,
                'str_start' => \Date::parse($GLOBALS['TL_CONFIG']['datimFormat'], $next),
                'str_end' => \Date::parse($GLOBALS['TL_CONFIG']['datimFormat'], $nextEnd)
            );

            if ($count == 0) {
                $arrSet['repeatEnd'] = 2145913200;
            }

            // last date of the recurrences
            $end = $arrSet['repeatEnd'];

            while ($next <= $end) {
                $timetoadd = '+ ' . $arrRange['value'] . ' ' . $unit;

                // Check if we are at the end
                if (!strtotime($timetoadd, $next)) {
                    break;
                }

                $strtotime = strtotime($timetoadd, $next);
                $next = $strtotime;
                $weekday = date('w', $next);

                //check if we are at the end
                if ($next >= $end) {
                    break;
                }

                $value = (int)$arrRange['value'];
                $wdays = (is_array(deserialize($dc->activeRecord->repeatWeekday)))
                    ? deserialize($dc->activeRecord->repeatWeekday)
                    : false;

                if ($unit === 'days' && $value === 1 && $wdays) {
                    $wday = date('N', $next);
                    $store = in_array($wday, $wdays);
                }

                $store = true;
                if ($dc->activeRecord->hideOnWeekend) {
                    if ($weekday == 0 || $weekday == 6) {
                        $store = false;
                    }
                }
                if ($store === true) {
                    $arrDates[$next] = date('d.m.Y H:i', $next);

                    // array of all recurrences
                    $strtotime = strtotime($timetoadd, $nextEnd);
                    $nextEnd = $strtotime;
                    $arrAllRecurrences[$next] = array(
                        'int_start' => $next,
                        'int_end' => $nextEnd,
                        'str_start' => \Date::parse($GLOBALS['TL_CONFIG']['datimFormat'], $next),
                        'str_end' => \Date::parse($GLOBALS['TL_CONFIG']['datimFormat'], $nextEnd)
                    );
                }

                //check if have the configured max value
                if (count($arrDates) == $maxCount) {
                    break;
                }
            }
            $maxRepeatEnd[] = $arrSet['repeatEnd'];
        }

        //list of months we need
        $arrMonth = array(1 => 'january', 2 => 'february', 3 => 'march', 4 => 'april', 5 => 'may', 6 => 'june',
            7 => 'july', 8 => 'august', 9 => 'september', 10 => 'october', 11 => 'november', 12 => 'december',
        );

        // extended version recurring
        if ($dc->activeRecord->recurringExt) {
            $arrRange = deserialize($dc->activeRecord->repeatEachExt);

            $arg = $arrRange['value'];
            $unit = $arrRange['unit'];

            // next month of the event
            $month = (int)date('n', $dc->activeRecord->startDate);
            // year of the event
            $year = (int)date('Y', $dc->activeRecord->startDate);
            // search date for the next event
            $next = (int)$arrSet['startTime'];
            $nextEnd = $arrSet['endTime'];

            // last month
            $count = (int)$dc->activeRecord->recurrences;

            // array of the exception dates
            $arrDates = array();
            $arrDates[$next] = date('d.m.Y H:i', $next);

            // array of all recurrences
            $arrAllRecurrences[$next] = array(
                'int_start' => $next,
                'int_end' => $nextEnd,
                'str_start' => \Date::parse($GLOBALS['TL_CONFIG']['datimFormat'], $next),
                'str_end' => \Date::parse($GLOBALS['TL_CONFIG']['datimFormat'], $nextEnd)
            );

            if ($count > 0) {
                for ($i = 0; $i < $count; $i++) {
                    $month++;
                    if (($month % 13) == 0) {
                        $month = 1;
                        $year += 1;
                    }

                    $timetoadd = $arg . ' ' . $unit . ' of ' . $arrMonth[$month] . ' ' . $year;
                    if (!strtotime($timetoadd, $next)) {
                        break;
                    }

                    $strtotime = strtotime($timetoadd, $next);
                    $next = strtotime(date("d.m.Y", $strtotime) . ' ' . date("H:i", $arrSet['startTime']));
                    $arrDates[$next] = date('d.m.Y H:i', $next);

                    // array of all recurrences
                    $strtotime = strtotime($timetoadd, $nextEnd);
                    $nextEnd = strtotime(date("d.m.Y", $strtotime) . ' ' . date("H:i", $arrSet['endTime']));
                    $arrAllRecurrences[$next] = array(
                        'int_start' => $next,
                        'int_end' => $nextEnd,
                        'str_start' => \Date::parse($GLOBALS['TL_CONFIG']['datimFormat'], $next),
                        'str_end' => \Date::parse($GLOBALS['TL_CONFIG']['datimFormat'], $nextEnd)
                    );

                    //check if have the configured max value
                    if (count($arrDates) == $maxCount) {
                        break;
                    }
                }
                $arrSet['repeatEnd'] = $next;
            } else {
                // 2038.01.01
                $arrSet['repeatEnd'] = 2145913200;
                $end = $arrSet['repeatEnd'];

                while ($next <= $end) {
                    $timetoadd = $arg . ' ' . $unit . ' of ' . $arrMonth[$month] . ' ' . $year;

                    if (!strtotime($timetoadd, $next)) {
                        break;
                    }

                    $strtotime = strtotime($timetoadd, $next);
                    $next = strtotime(date("d.m.Y", $strtotime) . ' ' . date("H:i", $arrSet['startTime']));
                    $arrDates[$next] = date('d.m.Y H:i', $next);

                    $month++;
                    if (($month % 13) == 0) {
                        $month = 1;
                        $year += 1;
                    }

                    //check if have the configured max value
                    if (count($arrDates) == $maxCount) {
                        break;
                    }
                }
            }

            $maxRepeatEnd[] = $arrSet['repeatEnd'];
        }
        unset($next);

        // the last repeatEnd Date
        if (count($maxRepeatEnd) > 1) {
            $arrSet['repeatEnd'] = max($maxRepeatEnd);
        }
        $currentEndDate = $arrSet['repeatEnd'];

        if ($dc->activeRecord->useExceptions) {
            // list of the exception
            $exceptionRows = array();

            // ... then we check them by interval...
            if ($dc->activeRecord->repeatExceptionsInt) {
                // weekday
                $unit = $GLOBALS['TL_CONFIG']['tl_calendar_events']['weekdays'][$dc->activeRecord->weekday];

                // exception rules
                $rows = deserialize($dc->activeRecord->repeatExceptionsInt);

                // run thru all dates
                foreach ($rows as $row) {
                    if (!$row['exception']) {
                        continue;
                    }

                    // now we have to find all dates matching the exception rules...
                    $arg = $row['exception'];

                    $searchNext = $arrSet['startTime'];
                    $searchEnd = $arrSet['repeatEnd'];
                    $month = (int)date('n', $searchNext);
                    $year = (int)date('Y', $searchNext);
                    while ($searchNext <= $searchEnd) {
                        $strDateToFind = $arg . ' ' . $unit . ' of ' . $arrMonth[$month] . ' ' . $year;
                        $strDateToFind = strtotime($strDateToFind);
                        $searchNext = strtotime(date("d.m.Y", $strDateToFind) . ' ' . date('H:i', $arrSet['startTime']));
                        if ($searchNext < $arrSet['startTime']) {
                            $month++;
                            if (($month % 13) == 0) {
                                $month = 1;
                                $year += 1;
                            }
                            continue;
                        }

                        $row['new_start'] = ($row['new_start']) ? $row['new_start'] : date('H:i', $dc->activeRecord->startTime); #'00:00';
                        $row['new_end'] = ($row['new_end']) ? $row['new_end'] : date('H:i', $dc->activeRecord->endTime); #'23:59';
                        // Set endtime to starttime always...
                        if ($dc->activeRecord->ignoreEndTime) {
                            $row['new_end'] = '';
                        }

                        $row['exception'] = $searchNext;
                        $row['exception_date'] = date('d.m.Y H:i', $searchNext);
                        if (count($exceptionRows) < $maxELCount) {
                            $exceptionRows[$searchNext] = $row;
                        }

                        $month++;
                        if (($month % 13) == 0) {
                            $month = 1;
                            $year += 1;
                        }
                    }
                }
            }

            // ... and last but not least by range
            if ($dc->activeRecord->repeatExceptionsPer) {
                // exception rules
                $rows = deserialize($dc->activeRecord->repeatExceptionsPer);

                // all recurrences...
                $repeatDates = deserialize($dc->activeRecord->repeatDates);

                // run thru all dates
                foreach ($rows as $row) {
                    if (!$row['exception']) {
                        continue;
                    }

                    $row['new_start'] = ($row['new_start']) ? $row['new_start'] : date('H:i', $dc->activeRecord->startTime); #'00:00';
                    // Set endtime to starttime always...
                    if ($dc->activeRecord->ignoreEndTime) {
                        $row['new_end'] = '';
                    } else {
                        $row['new_end'] = ($row['new_end']) ? $row['new_end'] : date('H:i', $dc->activeRecord->endTime); #'23:59';
                    }

                    // now we have to find all dates matching the exception rules...
                    $dateFrom = strtotime(date('Y-m-d', $row['exception']) . ' ' . $row['new_start']);
                    $dateTo = strtotime(date('Y-m-d', $row['exceptionTo'] ?: $row['exception']) . ' ' . $row['new_end']);
                    unset($row['exceptionTo']);

                    foreach ($repeatDates as $k => $repeatDate) {
                        if ($k >= $dateFrom && $k <= $dateTo) {
                            $row['exception'] = $k;
                            $row['exception_date'] = date('d.m.Y H:i', $k);
                            if (count($exceptionRows) < $maxELCount) {
                                $exceptionRows[$k] = $row;
                            }
                        }
                    }
                }
            }

            // first we check the exceptions by date...
            if ($dc->activeRecord->repeatExceptions) {
                $rows = deserialize($dc->activeRecord->repeatExceptions);
                // set repeatEnd
                // my be we have an exception move that is later then the repeatEnd
                foreach ($rows as $row) {
                    if (!$row['exception']) {
                        continue;
                    }

                    $row['new_start'] = ($row['new_start']) ? $row['new_start'] : date('H:i', $dc->activeRecord->startTime); #'00:00';

                    if (!$dc->activeRecord->ignoreEndTime) {
                        $row['new_end'] = ($row['new_end']) ? $row['new_end'] : date('H:i', $dc->activeRecord->endTime); #'23:59';
                    }
                    $row['exception_date'] = date('d.m.Y H:i', $row['exception']);

                    $dateToFind = strtotime(date("d.m.Y", $row['exception']) . ' ' . date("H:i", $dc->activeRecord->startTime));
                    $dateToSave = strtotime(date("d.m.Y", $row['exception']) . ' ' . $row['new_start']);
                    $dateToSaveEnd = strtotime(date("d.m.Y", $row['exception']) . ' ' . $row['new_end']);

                    // Set endtime to starttime always...
                    if ($dc->activeRecord->ignoreEndTime) {
                        $row['new_end'] = '';
                    }

                    if ($row['action'] == 'move') {
                        $newDate = strtotime($row['new_exception'], $row['exception']);
                        if ($newDate > $currentEndDate) {
                            $arrSet['repeatEnd'] = $newDate;
                            $maxRepeatEnd[] = $arrSet['repeatEnd'];
                        }

                        // Find the date and replace it
                        if (array_key_exists($dateToFind, $arrDates)) {
                            $arrDates[$dateToFind] = date('d.m.Y H:i', $dateToSave);
                        }

                        // Find the date and replace it
                        if (array_key_exists($dateToFind, $arrAllRecurrences)) {
                            $arrAllRecurrences[$dateToFind] = array(
                                'int_start'     => $dateToSave,
                                'int_end'       => $dateToSaveEnd,
                                'str_start'     => \Date::parse($GLOBALS['TL_CONFIG']['datimFormat'], $dateToSave),
                                'str_end'       => \Date::parse($GLOBALS['TL_CONFIG']['datimFormat'], $dateToSaveEnd),
                                'moveReason'    => ($row['reason']) ? $row['reason'] : ''
                            );
                        }
                    }
                    if (count($exceptionRows) < $maxELCount) {
                        $exceptionRows[$row['exception']] = $row;
                    }
                }
            }

            if (count($exceptionRows) > 1) {
                ksort($exceptionRows);
            }
            $arrSet['exceptionList'] = (count($exceptionRows) > 0) ? serialize($exceptionRows) : null;
        }

        if (count($maxRepeatEnd) > 1) {
            $arrSet['repeatEnd'] = max($maxRepeatEnd);
        }

        // Set the array of dates
        if (is_array($arrDates) && is_array($arrFixDates)) {
            $arrAllDates = $arrDates + $arrFixDates;
            ksort($arrAllDates);
            $arrSet['repeatDates'] = $arrAllDates;
        }

        // sort $arrAllRecurrences
        if (is_array($arrAllRecurrences)) {
            ksort($arrAllRecurrences);
            $arrSet['allRecurrences'] = $arrAllRecurrences;
        }

        // Execute the update sql
        $this->Database->prepare("UPDATE tl_calendar_events %s WHERE id=?")->set($arrSet)->execute($dc->id);
        unset($maxRepeatEnd);
    }


    /**
     * @param $dc
     * @return string
     */
    public function getmaxperson($var, $dc)
    {
        $values = deserialize($var);
        if (!is_array($values)) {
            $values = array();
            $values[0]['mini'] = 0;
            $values[0]['maxi'] = 0;
            $values[0]['curr'] = 0;
            $values[0]['free'] = 0;
            return $values;
        }

        $eid = (int)$dc->activeRecord->id;
        $fid = (int)$dc->activeRecord->regform;
        $regCount = CalendarLeadsModel::regCountByFormEvent($fid, $eid);

        $values[0]['curr'] = (int)$regCount;
        $values[0]['mini'] = ($values[0]['mini']) ? (int)$values[0]['mini'] : 0;
        $values[0]['maxi'] = ($values[0]['maxi']) ? (int)$values[0]['maxi'] : 0;
        $useMaxi = ($values[0]['maxi'] > 0) ? true : false;
        $values[0]['free'] = ($useMaxi) ? $values[0]['maxi'] - $values[0]['curr'] : 0;

        return serialize($values);
    }


    /**
     * @param $dc
     * @return array
     */
    public function setmaxperson($dc)
    {
        $columnFields = null;

        $columnFields = array
        (
            'mini' => array
            (
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['mini'],
                'default' => '0',
                'exclude' => true,
                'inputType' => 'text',
                'eval' => array('style' => 'width:60px')
            ),
            'maxi' => array
            (
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['maxi'],
                'exclude' => true,
                'inputType' => 'text',
                'eval' => array('style' => 'width:60px')
            ),
            'curr' => array
            (
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['curr'],
                'exclude' => true,
                'inputType' => 'text',
                'eval' => array('style' => 'width:60px', 'disabled' => 'true')
            ),
            'free' => array
            (
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['free'],
                'exclude' => true,
                'inputType' => 'text',
                'eval' => array('style' => 'width:60px', 'disabled' => 'true')
            )
        );

        return $columnFields;
    }


    /**
     * listMultiExceptions()
     *
     * Read the list of exception dates from the db
     * to fill the select list
     */
    public function listMultiExceptions($var1)
    {
        $columnFields = null;
        $activeRecord = $var1->activeRecord;

        // arrays for the select fields
        $arrSource1 = array();
        $arrSource2 = array();
        $arrSource3 = array();
        $arrSource4 = array();

        if (\Input::get('id')) {
            // Probably an AJAX request where activeRecord is not available
            if ($activeRecord === null) {
                $activeRecord = \Database::getInstance()
                    ->prepare("SELECT * FROM {$var1->strTable} WHERE id=?")
                    ->limit(1)
                    ->execute(\Input::get('id'))
                ;
            }

            if ($activeRecord->repeatDates) {
                $arrDates = deserialize($activeRecord->repeatDates);
                if (is_array($arrDates)) {
                    if ($var1->id == "repeatExceptions") {
                        // fill array for option date
                        foreach ($arrDates as $k => $arrDate) {
                            $date = \Date::parse($GLOBALS['TL_CONFIG']['dateFormat'], $k);
                            $arrSource1[$k] = $date;
                        }
                    }

                    // fill array for option action
                    $arrSource2['move'] = $GLOBALS['TL_LANG']['tl_calendar_events']['move'];
                    $arrSource2['hide'] = $GLOBALS['TL_LANG']['tl_calendar_events']['hide'];
                }
            }

            // fill array for option new date
            $moveDays = ((int)$GLOBALS['TL_CONFIG']['tl_calendar_events']['moveDays']) ? (int)$GLOBALS['TL_CONFIG']['tl_calendar_events']['moveDays'] : 7;
            $start = $moveDays * -1;
            $end = $moveDays * 2;

            for ($i = 0; $i <= $end; $i++) {
                $arrSource3[$start . ' days'] = $start . ' ' . $GLOBALS['TL_LANG']['tl_calendar_events']['days'];
                $start++;
            }

            list($start, $end, $interval) = explode("|", $GLOBALS['TL_CONFIG']['tl_calendar_events']['moveTimes']);

            // fill array for option new time
            $start = strtotime($start);
            $end = strtotime($end);
            while ($start <= $end) {
                $newTime = \Date::parse($GLOBALS['TL_CONFIG']['timeFormat'], $start);
                $arrSource4[$newTime] = $newTime;
                $start = strtotime('+ ' . $interval . ' minutes', $start);
            }
        }

        $columnFields = array
        (
            'new_start' => array
            (
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['new_start'],
                'exclude' => true,
                'inputType' => 'select',
                'options' => $arrSource4,
                'eval' => array('style' => 'width:60px', 'includeBlankOption' => true)
            ),
            'new_end' => array
            (
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['new_end'],
                'exclude' => true,
                'inputType' => 'select',
                'options' => $arrSource4,
                'eval' => array('style' => 'width:60px', 'includeBlankOption' => true)
            ),
            'action' => array
            (
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['action'],
                'exclude' => true,
                'inputType' => 'select',
                'options' => $arrSource2,
                'eval' => array('style' => 'width:80px', 'includeBlankOption' => true)
            ),
            'new_exception' => array
            (
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['new_exception'],
                'exclude' => true,
                'inputType' => 'select',
                'options' => $arrSource3,
                'eval' => array('style' => 'width:80px', 'includeBlankOption' => true)
            ),
            'cssclass' => array
            (
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['cssclass'],
                'exclude' => true,
                'inputType' => 'text',
                'eval' => array('style' => 'width:50px')
            ),
            'reason' => array
            (
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['reason'],
                'exclude' => true,
                'inputType' => 'text',
                'eval' => array('style' => 'width:150px')
            )
        );

        // normal exceptions by date
        if ($var1->id == "repeatExceptions") {
            $firstField = array(
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['exception'],
                'exclude' => true,
                'inputType' => 'select',
                'options' => $arrSource1,
                'eval' => array('style' => 'width:120px', 'includeBlankOption' => true)
            );
        } // exceptions by interval
        elseif ($var1->id == "repeatExceptionsInt") {
            $firstField = array(
                'label' => $GLOBALS['TL_LANG']['tl_calendar_events']['exceptionInt'] . $GLOBALS['TL_LANG']['DAYS'][$activeRecord->weekday],
                'exclude' => true,
                'inputType' => 'select',
                'options' => array('first', 'second', 'third', 'fourth', 'fifth', 'last'),
                'reference' => &$GLOBALS['TL_LANG']['tl_calendar_events'],
                'eval' => array('style' => 'width:120px', 'includeBlankOption' => true)
            );
        } // exceptions by time period
        elseif ($var1->id == "repeatExceptionsPer") {
            $firstField = array(
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['exceptionFr'],
                'exclude' => true,
                'inputType' => 'text',
                'eval' => array('rgxp' => 'date', 'doNotCopy' => true, 'style' => 'width:100px', 'datepicker' => true, 'tl_class' => 'wizard')
            );
            $secondField = array(
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['exceptionTo'],
                'exclude' => true,
                'inputType' => 'text',
                'eval' => array('rgxp' => 'date', 'doNotCopy' => true, 'style' => 'width:100px', 'datepicker' => true, 'tl_class' => 'wizard')
            );
            $columnFields['reason'] = array
            (
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['reason'],
                'exclude' => true,
                'inputType' => 'text',
                'eval' => array('style' => 'width:80px')
            );

            // add the field to the columnFields array
            array_insert($columnFields, 0, array("exceptionTo" => $secondField));
        }

        // add the field to the columnFields array
        array_insert($columnFields, 0, array("exception" => $firstField));

        return $columnFields;
    }


    /**
     * listFixedDates()
     */
    public function listFixedDates($var1)
    {
        $columnFields = null;

        $columnFields = array
        (
            'new_repeat' => array(
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['exception'],
                'exclude' => true,
                'inputType' => 'text',
                'eval' => array('rgxp' => 'date', 'datepicker' => true, 'doNotCopy' => true, 'style' => 'width:100px', 'tl_class' => 'wizard')
            ),
            'new_start' => array
            (
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['new_start'],
                'exclude' => true,
                'inputType' => 'text',
                'eval' => array('rgxp' => 'time', 'datepicker' => true, 'doNotCopy' => true, 'style' => 'width:40px')
            ),
            'new_end' => array
            (
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['new_end'],
                'exclude' => true,
                'inputType' => 'text',
                'eval' => array('rgxp' => 'time', 'datepicker' => true, 'doNotCopy' => true, 'style' => 'width:40px')
            ),
            'reason' => array
            (
                'label' => &$GLOBALS['TL_LANG']['tl_calendar_events']['reason'],
                'exclude' => true,
                'inputType' => 'text',
                'eval' => array('doNotCopy' => true, 'style' => 'width:350px')
            )
        );

        return $columnFields;
    }


    /**
     * @param $a
     * @return array
     */
    public function listRegForms($a)
    {
        if ($this->User->isAdmin) {
            $objForms = \FormModel::findAll();
        } else {
            $objForms = \FormModel::findMultipleByIds($this->User->forms);
        }

        $return = array();

        if ($objForms !== null) {
            while ($objForms->next()) {
                $return[$objForms->id] = $objForms->title;
            }
        }

        return $return;
    }

}
}
