<?php

namespace  {
/*
 * This file is part of Contao.
 *
 * (c) Leo Feyer
 *
 * @license LGPL-3.0-or-later
 */

use Contao\System;

$GLOBALS['TL_DCA']['tl_settings'] = array
(
	// Config
	'config' => array
	(
		'dataContainer'               => 'File',
		'closed'                      => true
	),

	// Palettes
	'palettes' => array
	(
		'default'                     => '{global_legend},adminEmail;{date_legend},dateFormat,timeFormat,datimFormat,timeZone;{backend_legend:hide},doNotCollapse,resultsPerPage,maxResultsPerPage;{security_legend:hide},disableRefererCheck,allowedTags;{files_legend:hide},allowedDownload,gdMaxImgWidth,gdMaxImgHeight;{uploads_legend:hide},uploadTypes,maxFileSize,imageWidth,imageHeight;{cron_legend:hide},disableCron;{chmod_legend},defaultUser,defaultGroup,defaultChmod'
	),

	// Fields
	'fields' => array
	(
		'dateFormat' => array
		(
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'helpwizard'=>true, 'decodeEntities'=>true, 'tl_class'=>'w50'),
			'explanation'             => 'dateFormat'
		),
		'timeFormat' => array
		(
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'decodeEntities'=>true, 'tl_class'=>'w50')
		),
		'datimFormat' => array
		(
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'decodeEntities'=>true, 'tl_class'=>'w50')
		),
		'timeZone' => array
		(
			'inputType'               => 'select',
			'options_callback' => static function ()
			{
				return System::getTimeZones();
			},
			'eval'                    => array('chosen'=>true, 'tl_class'=>'w50')
		),
		'adminEmail' => array
		(
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'rgxp'=>'friendly', 'decodeEntities'=>true, 'tl_class'=>'w50')
		),
		'disableCron' => array
		(
			'inputType'               => 'checkbox',
			'eval'                    => array('tl_class'=>'w50')
		),
		'resultsPerPage' => array
		(
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'rgxp'=>'natural', 'minval'=>1, 'nospace'=>true, 'tl_class'=>'w50 clr')
		),
		'maxResultsPerPage' => array
		(
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'rgxp'=>'natural', 'nospace'=>true, 'tl_class'=>'w50')
		),
		'doNotCollapse' => array
		(
			'inputType'               => 'checkbox',
			'eval'                    => array('tl_class'=>'w50')
		),
		'disableRefererCheck' => array
		(
			'inputType'               => 'checkbox',
		),
		'allowedTags' => array
		(
			'inputType'               => 'text',
			'eval'                    => array('useRawRequestData'=>true, 'tl_class'=>'long')
		),
		'allowedDownload' => array
		(
			'inputType'               => 'text',
			'eval'                    => array('tl_class'=>'w50')
		),
		'gdMaxImgWidth' => array
		(
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'rgxp'=>'natural', 'nospace'=>true, 'tl_class'=>'w50 clr')
		),
		'gdMaxImgHeight' => array
		(
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'rgxp'=>'natural', 'nospace'=>true, 'tl_class'=>'w50')
		),
		'uploadTypes' => array
		(
			'inputType'               => 'text',
			'eval'                    => array('tl_class'=>'w50')
		),
		'maxFileSize' => array
		(
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'rgxp'=>'natural', 'nospace'=>true, 'tl_class'=>'w50')
		),
		'imageWidth' => array
		(
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'rgxp'=>'natural', 'nospace'=>true, 'tl_class'=>'w50')
		),
		'imageHeight' => array
		(
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'rgxp'=>'natural', 'nospace'=>true, 'tl_class'=>'w50')
		),
		'defaultUser' => array(
			'inputType'               => 'select',
			'foreignKey'              => 'tl_user.username',
			'eval'                    => array('chosen'=>true, 'includeBlankOption'=>true, 'tl_class'=>'w50')
		),
		'defaultGroup' => array(
			'inputType'               => 'select',
			'foreignKey'              => 'tl_user_group.name',
			'eval'                    => array('chosen'=>true, 'includeBlankOption'=>true, 'tl_class'=>'w50')
		),
		'defaultChmod' => array
		(
			'inputType'               => 'chmod',
			'eval'                    => array('tl_class'=>'clr')
		)
	)
);
}

namespace  {
/**
 * Contao Open Source CMS
 *
 * @copyright  MEN AT WORK 2014
 * @package    ctoCommunication
 * @license    GNU/LGPL 
 * @filesource
 */

/**
 * System configuration
 */

// Palettes Insert
$arrPalettes = explode(";", $GLOBALS['TL_DCA']['tl_settings']['palettes']['default']);
$GLOBALS['TL_DCA']['tl_settings']['palettes']['default'] = implode(";", array_merge(array_slice($arrPalettes, 0, 1), array('{ctoCommunication_legend},ctoCom_APIKey,ctoCom_responseLength,ctoCom_handshake'), array_slice($arrPalettes, 1)));

// Fields
$GLOBALS['TL_DCA']['tl_settings']['fields']['ctoCom_APIKey'] = array(
    'label' => &$GLOBALS['TL_LANG']['tl_settings']['ctoCom_APIKey'],
    'inputType' => 'text',
    'explanation' => 'ctoComKey',
    'eval' => array('helpwizard' => true, 'tl_class' => 'long', 'minlength' => '32', 'maxlength' => '64'),
    'exclude' => true,
    'save_callback' => array(array('CtoCommunicationSettings', 'save_callback')),
);

$GLOBALS['TL_DCA']['tl_settings']['fields']['ctoCom_responseLength'] = array(
    'label' => &$GLOBALS['TL_LANG']['tl_settings']['ctoCom_responseLength'],
    'inputType' => 'text',
    'eval' => array('rgxp' => 'digit', 'tl_class' => 'long', 'maxlength' => '64', 'minlength' => '5'),
    'exclude' => true,
);

$GLOBALS['TL_DCA']['tl_settings']['fields']['ctoCom_handshake'] = array(
    'label' => &$GLOBALS['TL_LANG']['tl_settings']['handshake'],
    'inputType' => 'checkbox',
    'exclude' => true,
);

class CtoCommunicationSettings extends \Backend
{

    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Generate the sec key for server
     * 
     * @param type $varValue
     * @param DataContainer $dca
     * @return type 
     */
    public function save_callback($varValue, DataContainer $dca)
    {
        if ($varValue == "")
        {
            $objKey = $this->Database->prepare("SELECT UUID() as uid")->execute();
            return $objKey->uid;
        }

        return $varValue;
    }

}
}

namespace  {
$GLOBALS['TL_DCA']['tl_settings']['config']['onload_callback'] = [
    [ 'CatalogManager\tl_settings', 'changeGlobals' ]
];

$GLOBALS['TL_DCA']['tl_settings']['palettes']['__selector__'][] = 'catalogMapProtected';
$GLOBALS['TL_DCA']['tl_settings']['palettes']['default'] .= ';{catalog_manager_settings},catalogLicence,catalogGoogleMapsServerKey,catalogGoogleMapsClientKey,catalogNavigationAreas;{catalog_manager_dsgvo_legend},catalogMapProtected';
$GLOBALS['TL_DCA']['tl_settings']['subpalettes']['catalogMapProtected'] = 'catalogMapPrivacyText,catalogMapPrivacyButtonText';

$GLOBALS['TL_DCA']['tl_settings']['fields']['catalogGoogleMapsServerKey'] = [
    'label' => &$GLOBALS['TL_LANG']['tl_settings']['catalogGoogleMapsServerKey'],
    'inputType' => 'text',
    'eval' => [
        'tl_class' => 'w50'
    ]
];
$GLOBALS['TL_DCA']['tl_settings']['fields']['catalogGoogleMapsClientKey'] = [
    'label' => &$GLOBALS['TL_LANG']['tl_settings']['catalogGoogleMapsClientKey'],
    'inputType' => 'text',
    'eval' => [
        'tl_class' => 'w50'
    ]
];
$GLOBALS['TL_DCA']['tl_settings']['fields']['catalogLicence'] = [
    'label' => &$GLOBALS['TL_LANG']['tl_settings']['catalogLicence'],
    'inputType' => 'text',
    'eval' => [
        'tl_class' => 'w50'
    ],
    'save_callback' => [ [ 'CatalogManager\tl_settings', 'verifyLicence' ] ]
];
$GLOBALS['TL_DCA']['tl_settings']['fields']['catalogNavigationAreas'] = [
    'label' => &$GLOBALS['TL_LANG']['tl_settings']['catalogNavigationAreas'],
    'inputType' => 'keyValueWizard',
    'eval' => [
        'tl_class' => 'clr'
    ],
    'save_callback' => [['CatalogManager\tl_settings', 'parseNavigationName']]
];
$GLOBALS['TL_DCA']['tl_settings']['fields']['catalogMapProtected'] = [
    'label' => &$GLOBALS['TL_LANG']['tl_settings']['catalogMapProtected'],
    'inputType' => 'checkbox',
    'eval' => [
        'tl_class' => 'clr',
        'submitOnChange' => true,
    ],
    'exclude' => true
];
$GLOBALS['TL_DCA']['tl_settings']['fields']['catalogMapPrivacyText'] = [
    'label' => &$GLOBALS['TL_LANG']['tl_settings']['catalogMapPrivacyText'],
    'inputType' => 'textarea',
    'eval' => [
        'tl_class' => 'clr',
        'allowHtml' => true
    ],
    'exclude' => true
];
$GLOBALS['TL_DCA']['tl_settings']['fields']['catalogMapPrivacyButtonText'] = [
    'label' => &$GLOBALS['TL_LANG']['tl_settings']['catalogMapPrivacyButtonText'],
    'inputType' => 'text',
    'eval' => [
        'tl_class' => 'w50',
        'allowHtml' => true
    ],
    'exclude' => true
];
}

namespace  {
/**
 * @copyright  Softleister 2007-2017
 * @author     Softleister <info@softleister.de>
 * @package    BackupDB - Database backup
 * @license    LGPL
 * @see	       https://github.com/do-while/contao-BackupDB
 */


/**
 * System configuration
 */
$GLOBALS['TL_DCA']['tl_settings']['fields']['backupdb_saveddb'] = array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_settings']['backupdb_saveddb'],
			'inputType'               => 'text',
			'eval'                    => array('tl_class'=>'w50')
		);

$GLOBALS['TL_DCA']['tl_settings']['fields']['AutoBackupCount'] = array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_settings']['AutoBackupCount'],
			'inputType'               => 'text',
			'eval'                    => array('rgxp'=>'digit', 'nospace'=>true, 'tl_class'=>'w50')
		);
		
$GLOBALS['TL_DCA']['tl_settings']['fields']['WsTemplatePath'] = array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_settings']['WsTemplatePath'],
			'inputType'               => 'text',
			'eval'                    => array('nospace'=>'true', 'trailingSlash'=>false, 'tl_class'=>'w50')
		);

$GLOBALS['TL_DCA']['tl_settings']['fields']['backupdb_blacklist'] = array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_settings']['backupdb_blacklist'],
			'inputType'               => 'text',
			'eval'                    => array('tl_class'=>'long')
		);

$GLOBALS['TL_DCA']['tl_settings']['fields']['backupdb_sendmail'] = array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_settings']['backupdb_sendmail'],
			'inputType'               => 'checkbox',
			'eval'                    => array('tl_class'=>'w50 clr')
		);

$GLOBALS['TL_DCA']['tl_settings']['fields']['backupdb_attmail'] = array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_settings']['backupdb_attmail'],
			'inputType'               => 'checkbox',
			'eval'                    => array('tl_class'=>'w50')
		);

$GLOBALS['TL_DCA']['tl_settings']['fields']['backupdb_zip'] = array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_settings']['backupdb_zip'],
			'inputType'               => 'checkbox',
			'eval'                    => array('tl_class'=>'w50 m12')
		);

$GLOBALS['TL_DCA']['tl_settings']['fields']['backupdb_var'] = array
		(
			'label'                   => &$GLOBALS['TL_LANG']['tl_settings']['backupdb_var'],
			'inputType'               => 'text',
			'eval'                    => array('rgxp'=>'alias', 'maxlength'=>128, 'tl_class'=>'w50 clr')
		);


/**
 * Modify palette
 */
$GLOBALS['TL_DCA']['tl_settings']['palettes']['default'] .= ';{backupdb_legend:hide},backupdb_blacklist,backupdb_saveddb,WsTemplatePath,backupdb_zip,backupdb_var,AutoBackupCount,backupdb_sendmail,backupdb_attmail';
}
